@@ -4,13 +4,13 @@ import (
 	"fmt"
 	"os"
 
-	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/idmap"
 )
 
 func help(me string, status int) {
 	fmt.Printf("Usage: %s directory [-t] [-r] <range1> [<range2> ...]\n", me)
 	fmt.Printf("  -t implies test mode.  No file ownerships will be changed.\n")
-	fmt.Printf("  -r means reverse, that is shift the uids out of hte container.\n")
+	fmt.Printf("  -r means reverse, that is shift the uids out of the container.\n")
 	fmt.Printf("\n")
 	fmt.Printf("  A range is [u|b|g]:<first_container_id:first_host_id:range>.\n")
 	fmt.Printf("  where u means shift uids, g means shift gids, b means shift both.\n")
@@ -35,7 +35,7 @@ func run() error {
 	}
 
 	directory := os.Args[1]
-	idmap := shared.IdmapSet{}
+	idmapSet := idmap.IdmapSet{}
 	testmode := false
 	reverse := false
 
@@ -48,14 +48,14 @@ func run() error {
 			testmode = true
 		default:
 			var err error
-			idmap, err = idmap.Append(os.Args[pos])
+			idmapSet, err = idmapSet.Append(os.Args[pos])
 			if err != nil {
 				return err
 			}
 		}
 	}
 
-	if idmap.Len() == 0 {
+	if idmapSet.Len() == 0 {
 		fmt.Printf("No idmaps given\n")
 		help(os.Args[0], 1)
 	}
@@ -66,7 +66,7 @@ func run() error {
 	}
 
 	if reverse {
-		return idmap.UidshiftFromContainer(directory, testmode)
+		return idmapSet.UidshiftFromContainer(directory, testmode)
 	}
-	return idmap.UidshiftIntoContainer(directory, testmode)
+	return idmapSet.UidshiftIntoContainer(directory, testmode)
 }
