@@ -2,6 +2,7 @@ package main
 
 import (
 	"crypto/x509"
+	"encoding/base64"
 	"encoding/pem"
 	"fmt"
 	"io/ioutil"
@@ -13,8 +14,10 @@ import (
 	"github.com/olekukonko/tablewriter"
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd/lxc/config"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 	"github.com/lxc/lxd/shared/termios"
@@ -29,7 +32,7 @@ func (c *configCmd) showByDefault() bool {
 }
 
 func (c *configCmd) flags() {
-	gnuflag.BoolVar(&c.expanded, "expanded", false, i18n.G("Whether to show the expanded configuration"))
+	gnuflag.BoolVar(&c.expanded, "expanded", false, i18n.G("Show the expanded configuration"))
 }
 
 func (c *configCmd) configEditHelp() string {
@@ -55,51 +58,91 @@ func (c *configCmd) configEditHelp() string {
 
 func (c *configCmd) usage() string {
 	return i18n.G(
-		`Manage configuration.
+		`Usage: lxc config <subcommand> [options]
 
-lxc config device add <[remote:]container> <name> <type> [key=value]...     Add a device to a container.
-lxc config device get <[remote:]container> <name> <key>                     Get a device property.
-lxc config device set <[remote:]container> <name> <key> <value>             Set a device property.
-lxc config device unset <[remote:]container> <name> <key>                   Unset a device property.
-lxc config device list <[remote:]container>                                 List devices for container.
-lxc config device show <[remote:]container>                                 Show full device details for container.
-lxc config device remove <[remote:]container> <name>                        Remove device from container.
+Change container or server configuration options.
 
-lxc config get [remote:][container] <key>                                   Get container or server configuration key.
-lxc config set [remote:][container] <key> <value>                           Set container or server configuration key.
-lxc config unset [remote:][container] <key>                                 Unset container or server configuration key.
-lxc config show [remote:][container] [--expanded]                           Show container or server configuration.
-lxc config edit [remote:][container]                                        Edit container or server configuration in external editor.
+*Container configuration*
+
+lxc config get [<remote>:][container] <key>
+    Get container or server configuration key.
+
+lxc config set [<remote>:][container] <key> <value>
+    Set container or server configuration key.
+
+lxc config unset [<remote>:][container] <key>
+    Unset container or server configuration key.
+
+lxc config show [<remote>:][container] [--expanded]
+    Show container or server configuration.
+
+lxc config edit [<remote>:][container]
     Edit configuration, either by launching external editor or reading STDIN.
-    Example: lxc config edit <container> # launch editor
-             cat config.yaml | lxc config edit <config> # read from config.yaml
 
-lxc config trust list [remote]                                              List all trusted certs.
-lxc config trust add [remote] <certfile.crt>                                Add certfile.crt to trusted hosts.
-lxc config trust remove [remote] [hostname|fingerprint]                     Remove the cert from trusted hosts.
+*Device management*
 
-Examples:
-To mount host's /share/c1 onto /opt in the container:
-    lxc config device add [remote:]container1 <device-name> disk source=/share/c1 path=opt
+lxc config device add [<remote>:]<container> <device> <type> [key=value...]
+    Add a device to a container.
 
-To set an lxc config value:
-    lxc config set [remote:]<container> raw.lxc 'lxc.aa_allow_incomplete = 1'
+lxc config device get [<remote>:]<container> <device> <key>
+    Get a device property.
 
-To listen on IPv4 and IPv6 port 8443 (you can omit the 8443 its the default):
-    lxc config set core.https_address [::]:8443
+lxc config device set [<remote>:]<container> <device> <key> <value>
+    Set a device property.
 
-To set the server trust password:
-    lxc config set core.trust_password blah`)
+lxc config device unset [<remote>:]<container> <device> <key>
+    Unset a device property.
+
+lxc config device list [<remote>:]<container>
+    List devices for container.
+
+lxc config device show [<remote>:]<container>
+    Show full device details for container.
+
+lxc config device remove [<remote>:]<container> <name>...
+    Remove device from container.
+
+*Client trust store management*
+
+lxc config trust list [<remote>:]
+    List all trusted certs.
+
+lxc config trust add [<remote>:] <certfile.crt>
+    Add certfile.crt to trusted hosts.
+
+lxc config trust remove [<remote>:] [hostname|fingerprint]
+    Remove the cert from trusted hosts.
+
+*Examples*
+
+cat config.yaml | lxc config edit <container>
+    Update the container configuration from config.yaml.
+
+lxc config device add [<remote>:]container1 <device-name> disk source=/share/c1 path=opt
+    Will mount the host's /share/c1 onto /opt in the container.
+
+lxc config set [<remote>:]<container> limits.cpu 2
+    Will set a CPU limit of "2" for the container.
+
+lxc config set core.https_address [::]:8443
+    Will have LXD listen on IPv4 and IPv6 port 8443.
+
+lxc config set core.trust_password blah
+    Will set the server's trust password to blah.`)
 }
 
-func (c *configCmd) doSet(config *lxd.Config, args []string, unset bool) error {
+func (c *configCmd) doSet(conf *config.Config, args []string, unset bool) error {
 	if len(args) != 4 {
 		return errArgs
 	}
 
 	// [[lxc config]] set dakara:c1 limits.memory 200000
-	remote, container := config.ParseRemoteAndContainer(args[1])
-	d, err := lxd.NewClient(config, remote)
+	remote, name, err := conf.ParseRemote(args[1])
+	if err != nil {
+		return err
+	}
+
+	d, err := conf.GetContainerServer(remote)
 	if err != nil {
 		return err
 	}
@@ -115,24 +158,33 @@ func (c *configCmd) doSet(config *lxd.Config, args []string, unset bool) error {
 		value = string(buf[:])
 	}
 
-	if unset {
-		st, err := d.ContainerInfo(container)
-		if err != nil {
-			return err
-		}
-
-		_, ok := st.Config[key]
-		if !ok {
-			return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set."), key)
-		}
+	container, etag, err := d.GetContainer(name)
+	if err != nil {
+		return err
 	}
 
-	return d.SetContainerConfig(container, key, value)
+	if unset {
+		_, ok := container.Config[key]
+		if !ok {
+			return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set"), key)
+		}
+
+		delete(container.Config, key)
+	} else {
+		container.Config[key] = value
+	}
+
+	op, err := d.UpdateContainer(name, container.Writable(), etag)
+	if err != nil {
+		return err
+	}
+
+	return op.Wait()
 }
 
-func (c *configCmd) run(config *lxd.Config, args []string) error {
+func (c *configCmd) run(conf *config.Config, args []string) error {
 	if len(args) < 1 {
-		return errArgs
+		return errUsage
 	}
 
 	switch args[0] {
@@ -144,50 +196,54 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 
 		// Deal with local server
 		if len(args) == 2 {
-			c, err := lxd.NewClient(config, config.DefaultRemote)
+			c, err := conf.GetContainerServer(conf.DefaultRemote)
 			if err != nil {
 				return err
 			}
 
-			ss, err := c.ServerStatus()
+			server, etag, err := c.GetServer()
 			if err != nil {
 				return err
 			}
 
-			_, ok := ss.Config[args[1]]
+			_, ok := server.Config[args[1]]
 			if !ok {
 				return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set."), args[1])
 			}
 
-			_, err = c.SetServerConfig(args[1], "")
-			return err
+			delete(server.Config, args[1])
+			return c.UpdateServer(server.Writable(), etag)
 		}
 
 		// Deal with remote server
-		remote, container := config.ParseRemoteAndContainer(args[1])
+		remote, container, err := conf.ParseRemote(args[1])
+		if err != nil {
+			return err
+		}
+
 		if container == "" {
-			c, err := lxd.NewClient(config, remote)
+			c, err := conf.GetContainerServer(remote)
 			if err != nil {
 				return err
 			}
 
-			ss, err := c.ServerStatus()
+			server, etag, err := c.GetServer()
 			if err != nil {
 				return err
 			}
 
-			_, ok := ss.Config[args[1]]
+			_, ok := server.Config[args[2]]
 			if !ok {
 				return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set."), args[1])
 			}
 
-			_, err = c.SetServerConfig(args[2], "")
-			return err
+			delete(server.Config, args[2])
+			return c.UpdateServer(server.Writable(), etag)
 		}
 
 		// Deal with container
 		args = append(args, "")
-		return c.doSet(config, args, true)
+		return c.doSet(conf, args, true)
 
 	case "set":
 		if len(args) < 3 {
@@ -196,29 +252,45 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 
 		// Deal with local server
 		if len(args) == 3 {
-			c, err := lxd.NewClient(config, config.DefaultRemote)
+			c, err := conf.GetContainerServer(conf.DefaultRemote)
 			if err != nil {
 				return err
 			}
 
-			_, err = c.SetServerConfig(args[1], args[2])
-			return err
+			server, etag, err := c.GetServer()
+			if err != nil {
+				return err
+			}
+
+			server.Config[args[1]] = args[2]
+
+			return c.UpdateServer(server.Writable(), etag)
 		}
 
 		// Deal with remote server
-		remote, container := config.ParseRemoteAndContainer(args[1])
+		remote, container, err := conf.ParseRemote(args[1])
+		if err != nil {
+			return err
+		}
+
 		if container == "" {
-			c, err := lxd.NewClient(config, remote)
+			c, err := conf.GetContainerServer(remote)
 			if err != nil {
 				return err
 			}
 
-			_, err = c.SetServerConfig(args[2], args[3])
-			return err
+			server, etag, err := c.GetServer()
+			if err != nil {
+				return err
+			}
+
+			server.Config[args[2]] = args[3]
+
+			return c.UpdateServer(server.Writable(), etag)
 		}
 
 		// Deal with container
-		return c.doSet(config, args, false)
+		return c.doSet(conf, args, false)
 
 	case "trust":
 		if len(args) < 2 {
@@ -229,17 +301,21 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 		case "list":
 			var remote string
 			if len(args) == 3 {
-				remote = config.ParseRemote(args[2])
+				var err error
+				remote, _, err = conf.ParseRemote(args[2])
+				if err != nil {
+					return err
+				}
 			} else {
-				remote = config.DefaultRemote
+				remote = conf.DefaultRemote
 			}
 
-			d, err := lxd.NewClient(config, remote)
+			d, err := conf.GetContainerServer(remote)
 			if err != nil {
 				return err
 			}
 
-			trust, err := d.CertificateList()
+			trust, err := d.GetCertificates()
 			if err != nil {
 				return err
 			}
@@ -283,52 +359,69 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 			if len(args) < 3 {
 				return fmt.Errorf(i18n.G("No certificate provided to add"))
 			} else if len(args) == 4 {
-				remote = config.ParseRemote(args[2])
+				var err error
+				remote, _, err = conf.ParseRemote(args[2])
+				if err != nil {
+					return err
+				}
 			} else {
-				remote = config.DefaultRemote
+				remote = conf.DefaultRemote
 			}
 
-			d, err := lxd.NewClient(config, remote)
+			d, err := conf.GetContainerServer(remote)
 			if err != nil {
 				return err
 			}
 
 			fname := args[len(args)-1]
-			cert, err := shared.ReadCert(fname)
+			x509Cert, err := shared.ReadCert(fname)
 			if err != nil {
 				return err
 			}
-
 			name, _ := shared.SplitExt(fname)
-			return d.CertificateAdd(cert, name)
+
+			cert := api.CertificatesPost{}
+			cert.Certificate = base64.StdEncoding.EncodeToString(x509Cert.Raw)
+			cert.Name = name
+			cert.Type = "client"
+
+			return d.CreateCertificate(cert)
 		case "remove":
 			var remote string
 			if len(args) < 3 {
 				return fmt.Errorf(i18n.G("No fingerprint specified."))
 			} else if len(args) == 4 {
-				remote = config.ParseRemote(args[2])
+				var err error
+				remote, _, err = conf.ParseRemote(args[2])
+				if err != nil {
+					return err
+				}
 			} else {
-				remote = config.DefaultRemote
+				remote = conf.DefaultRemote
 			}
 
-			d, err := lxd.NewClient(config, remote)
+			d, err := conf.GetContainerServer(remote)
 			if err != nil {
 				return err
 			}
 
-			return d.CertificateRemove(args[len(args)-1])
+			return d.DeleteCertificate(args[len(args)-1])
 		default:
 			return errArgs
 		}
 
 	case "show":
-		remote := config.DefaultRemote
+		remote := conf.DefaultRemote
 		container := ""
 		if len(args) > 1 {
-			remote, container = config.ParseRemoteAndContainer(args[1])
+			var err error
+			remote, container, err = conf.ParseRemote(args[1])
+			if err != nil {
+				return err
+			}
 		}
 
-		d, err := lxd.NewClient(config, remote)
+		d, err := conf.GetContainerServer(remote)
 		if err != nil {
 			return err
 		}
@@ -336,44 +429,50 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 		var data []byte
 
 		if len(args) == 1 || container == "" {
-			config, err := d.ServerStatus()
+			server, _, err := d.GetServer()
 			if err != nil {
 				return err
 			}
 
-			brief := config.Brief()
+			brief := server.Writable()
 			data, err = yaml.Marshal(&brief)
+			if err != nil {
+				return err
+			}
 		} else {
-			var brief shared.BriefContainerInfo
+			var brief api.ContainerPut
 			if shared.IsSnapshot(container) {
-				config, err := d.SnapshotInfo(container)
+				fields := strings.Split(container, shared.SnapshotDelimiter)
+
+				snap, _, err := d.GetContainerSnapshot(fields[0], fields[1])
 				if err != nil {
 					return err
 				}
 
-				brief = shared.BriefContainerInfo{
-					Profiles:  config.Profiles,
-					Config:    config.Config,
-					Devices:   config.Devices,
-					Ephemeral: config.Ephemeral,
+				brief = api.ContainerPut{
+					Profiles:  snap.Profiles,
+					Config:    snap.Config,
+					Devices:   snap.Devices,
+					Ephemeral: snap.Ephemeral,
 				}
 				if c.expanded {
-					brief = shared.BriefContainerInfo{
-						Profiles:  config.Profiles,
-						Config:    config.ExpandedConfig,
-						Devices:   config.ExpandedDevices,
-						Ephemeral: config.Ephemeral,
+					brief = api.ContainerPut{
+						Profiles:  snap.Profiles,
+						Config:    snap.ExpandedConfig,
+						Devices:   snap.ExpandedDevices,
+						Ephemeral: snap.Ephemeral,
 					}
 				}
 			} else {
-				config, err := d.ContainerInfo(container)
+				container, _, err := d.GetContainer(container)
 				if err != nil {
 					return err
 				}
 
-				brief = config.Brief()
+				brief = container.Writable()
 				if c.expanded {
-					brief = config.BriefExpanded()
+					brief.Config = container.ExpandedConfig
+					brief.Devices = container.ExpandedDevices
 				}
 			}
 
@@ -392,27 +491,31 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 			return errArgs
 		}
 
-		remote := config.DefaultRemote
+		remote := conf.DefaultRemote
 		container := ""
 		key := args[1]
 		if len(args) > 2 {
-			remote, container = config.ParseRemoteAndContainer(args[1])
+			var err error
+			remote, container, err = conf.ParseRemote(args[1])
+			if err != nil {
+				return err
+			}
 			key = args[2]
 		}
 
-		d, err := lxd.NewClient(config, remote)
+		d, err := conf.GetContainerServer(remote)
 		if err != nil {
 			return err
 		}
 
 		if container != "" {
-			resp, err := d.ContainerInfo(container)
+			resp, _, err := d.GetContainer(container)
 			if err != nil {
 				return err
 			}
 			fmt.Println(resp.Config[key])
 		} else {
-			resp, err := d.ServerStatus()
+			resp, _, err := d.GetServer()
 			if err != nil {
 				return err
 			}
@@ -437,19 +540,19 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 		}
 		switch args[1] {
 		case "list":
-			return c.deviceList(config, "container", args)
+			return c.deviceList(conf, "container", args)
 		case "add":
-			return c.deviceAdd(config, "container", args)
+			return c.deviceAdd(conf, "container", args)
 		case "remove":
-			return c.deviceRm(config, "container", args)
+			return c.deviceRm(conf, "container", args)
 		case "get":
-			return c.deviceGet(config, "container", args)
+			return c.deviceGet(conf, "container", args)
 		case "set":
-			return c.deviceSet(config, "container", args)
+			return c.deviceSet(conf, "container", args)
 		case "unset":
-			return c.deviceUnset(config, "container", args)
+			return c.deviceUnset(conf, "container", args)
 		case "show":
-			return c.deviceShow(config, "container", args)
+			return c.deviceShow(conf, "container", args)
 		default:
 			return errArgs
 		}
@@ -459,13 +562,17 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 			return errArgs
 		}
 
-		remote := config.DefaultRemote
+		remote := conf.DefaultRemote
 		container := ""
 		if len(args) > 1 {
-			remote, container = config.ParseRemoteAndContainer(args[1])
+			var err error
+			remote, container, err = conf.ParseRemote(args[1])
+			if err != nil {
+				return err
+			}
 		}
 
-		d, err := lxd.NewClient(config, remote)
+		d, err := conf.GetContainerServer(remote)
 		if err != nil {
 			return err
 		}
@@ -483,7 +590,7 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 	return errArgs
 }
 
-func (c *configCmd) doContainerConfigEdit(client *lxd.Client, cont string) error {
+func (c *configCmd) doContainerConfigEdit(client lxd.ContainerServer, cont string) error {
 	// If stdin isn't a terminal, read text from it
 	if !termios.IsTerminal(int(syscall.Stdin)) {
 		contents, err := ioutil.ReadAll(os.Stdin)
@@ -491,21 +598,27 @@ func (c *configCmd) doContainerConfigEdit(client *lxd.Client, cont string) error
 			return err
 		}
 
-		newdata := shared.BriefContainerInfo{}
+		newdata := api.ContainerPut{}
 		err = yaml.Unmarshal(contents, &newdata)
 		if err != nil {
 			return err
 		}
-		return client.UpdateContainerConfig(cont, newdata)
+
+		op, err := client.UpdateContainer(cont, newdata, "")
+		if err != nil {
+			return err
+		}
+
+		return op.Wait()
 	}
 
 	// Extract the current value
-	config, err := client.ContainerInfo(cont)
+	container, etag, err := client.GetContainer(cont)
 	if err != nil {
 		return err
 	}
 
-	brief := config.Brief()
+	brief := container.Writable()
 	data, err := yaml.Marshal(&brief)
 	if err != nil {
 		return err
@@ -519,10 +632,14 @@ func (c *configCmd) doContainerConfigEdit(client *lxd.Client, cont string) error
 
 	for {
 		// Parse the text received from the editor
-		newdata := shared.BriefContainerInfo{}
+		newdata := api.ContainerPut{}
 		err = yaml.Unmarshal(content, &newdata)
 		if err == nil {
-			err = client.UpdateContainerConfig(cont, newdata)
+			var op *lxd.Operation
+			op, err = client.UpdateContainer(cont, newdata, etag)
+			if err == nil {
+				err = op.Wait()
+			}
 		}
 
 		// Respawn the editor
@@ -543,10 +660,11 @@ func (c *configCmd) doContainerConfigEdit(client *lxd.Client, cont string) error
 		}
 		break
 	}
+
 	return nil
 }
 
-func (c *configCmd) doDaemonConfigEdit(client *lxd.Client) error {
+func (c *configCmd) doDaemonConfigEdit(client lxd.ContainerServer) error {
 	// If stdin isn't a terminal, read text from it
 	if !termios.IsTerminal(int(syscall.Stdin)) {
 		contents, err := ioutil.ReadAll(os.Stdin)
@@ -554,23 +672,22 @@ func (c *configCmd) doDaemonConfigEdit(client *lxd.Client) error {
 			return err
 		}
 
-		newdata := shared.BriefServerState{}
+		newdata := api.ServerPut{}
 		err = yaml.Unmarshal(contents, &newdata)
 		if err != nil {
 			return err
 		}
 
-		_, err = client.UpdateServerConfig(newdata)
-		return err
+		return client.UpdateServer(newdata, "")
 	}
 
 	// Extract the current value
-	config, err := client.ServerStatus()
+	server, etag, err := client.GetServer()
 	if err != nil {
 		return err
 	}
 
-	brief := config.Brief()
+	brief := server.Writable()
 	data, err := yaml.Marshal(&brief)
 	if err != nil {
 		return err
@@ -584,10 +701,10 @@ func (c *configCmd) doDaemonConfigEdit(client *lxd.Client) error {
 
 	for {
 		// Parse the text received from the editor
-		newdata := shared.BriefServerState{}
+		newdata := api.ServerPut{}
 		err = yaml.Unmarshal(content, &newdata)
 		if err == nil {
-			_, err = client.UpdateServerConfig(newdata)
+			err = client.UpdateServer(newdata, etag)
 		}
 
 		// Respawn the editor
@@ -608,55 +725,96 @@ func (c *configCmd) doDaemonConfigEdit(client *lxd.Client) error {
 		}
 		break
 	}
+
 	return nil
 }
 
-func (c *configCmd) deviceAdd(config *lxd.Config, which string, args []string) error {
+func (c *configCmd) deviceAdd(conf *config.Config, which string, args []string) error {
 	if len(args) < 5 {
 		return errArgs
 	}
-	remote, name := config.ParseRemoteAndContainer(args[2])
 
-	client, err := lxd.NewClient(config, remote)
+	remote, name, err := conf.ParseRemote(args[2])
+	if err != nil {
+		return err
+	}
+
+	client, err := conf.GetContainerServer(remote)
 	if err != nil {
 		return err
 	}
 
 	devname := args[3]
-	devtype := args[4]
-	var props []string
+	device := map[string]string{}
+	device["type"] = args[4]
 	if len(args) > 5 {
-		props = args[5:]
-	} else {
-		props = []string{}
+		for _, prop := range args[5:] {
+			results := strings.SplitN(prop, "=", 2)
+			if len(results) != 2 {
+				return fmt.Errorf("No value found in %q", prop)
+			}
+			k := results[0]
+			v := results[1]
+			device[k] = v
+		}
 	}
 
-	var resp *lxd.Response
 	if which == "profile" {
-		resp, err = client.ProfileDeviceAdd(name, devname, devtype, props)
+		profile, etag, err := client.GetProfile(name)
+		if err != nil {
+			return err
+		}
+
+		_, ok := profile.Devices[devname]
+		if ok {
+			return fmt.Errorf(i18n.G("The device already exists"))
+		}
+
+		profile.Devices[devname] = device
+
+		err = client.UpdateProfile(name, profile.Writable(), etag)
+		if err != nil {
+			return err
+		}
 	} else {
-		resp, err = client.ContainerDeviceAdd(name, devname, devtype, props)
+		container, etag, err := client.GetContainer(name)
+		if err != nil {
+			return err
+		}
+
+		_, ok := container.Devices[devname]
+		if ok {
+			return fmt.Errorf(i18n.G("The device already exists"))
+		}
+
+		container.Devices[devname] = device
+
+		op, err := client.UpdateContainer(name, container.Writable(), etag)
+		if err != nil {
+			return err
+		}
+
+		err = op.Wait()
+		if err != nil {
+			return err
+		}
 	}
-	if err != nil {
-		return err
-	}
-	if which != "profile" {
-		err = client.WaitForSuccess(resp.Operation)
-	}
-	if err == nil {
-		fmt.Printf(i18n.G("Device %s added to %s")+"\n", devname, name)
-	}
-	return err
+
+	fmt.Printf(i18n.G("Device %s added to %s")+"\n", devname, name)
+	return nil
 }
 
-func (c *configCmd) deviceGet(config *lxd.Config, which string, args []string) error {
+func (c *configCmd) deviceGet(conf *config.Config, which string, args []string) error {
 	if len(args) < 5 {
 		return errArgs
 	}
 
-	remote, name := config.ParseRemoteAndContainer(args[2])
+	remote, name, err := conf.ParseRemote(args[2])
+	if err != nil {
+		return err
+	}
 
-	client, err := lxd.NewClient(config, remote)
+	client, err := conf.GetContainerServer(remote)
 	if err != nil {
 		return err
 	}
@@ -665,24 +823,24 @@ func (c *configCmd) deviceGet(config *lxd.Config, which string, args []string) e
 	key := args[4]
 
 	if which == "profile" {
-		st, err := client.ProfileConfig(name)
+		profile, _, err := client.GetProfile(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := st.Devices[devname]
+		dev, ok := profile.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
 
 		fmt.Println(dev[key])
 	} else {
-		st, err := client.ContainerInfo(name)
+		container, _, err := client.GetContainer(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := st.Devices[devname]
+		dev, ok := container.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
@@ -693,14 +851,17 @@ func (c *configCmd) deviceGet(config *lxd.Config, which string, args []string) e
 	return nil
 }
 
-func (c *configCmd) deviceSet(config *lxd.Config, which string, args []string) error {
+func (c *configCmd) deviceSet(conf *config.Config, which string, args []string) error {
 	if len(args) < 6 {
 		return errArgs
 	}
 
-	remote, name := config.ParseRemoteAndContainer(args[2])
+	remote, name, err := conf.ParseRemote(args[2])
+	if err != nil {
+		return err
+	}
 
-	client, err := lxd.NewClient(config, remote)
+	client, err := conf.GetContainerServer(remote)
 	if err != nil {
 		return err
 	}
@@ -710,54 +871,62 @@ func (c *configCmd) deviceSet(config *lxd.Config, which string, args []string) e
 	value := args[5]
 
 	if which == "profile" {
-		st, err := client.ProfileConfig(name)
+		profile, etag, err := client.GetProfile(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := st.Devices[devname]
+		dev, ok := profile.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
 
 		dev[key] = value
-		st.Devices[devname] = dev
+		profile.Devices[devname] = dev
 
-		err = client.PutProfile(name, *st)
+		err = client.UpdateProfile(name, profile.Writable(), etag)
 		if err != nil {
 			return err
 		}
 	} else {
-		st, err := client.ContainerInfo(name)
+		container, etag, err := client.GetContainer(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := st.Devices[devname]
+		dev, ok := container.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
 
 		dev[key] = value
-		st.Devices[devname] = dev
+		container.Devices[devname] = dev
 
-		err = client.UpdateContainerConfig(name, st.Brief())
+		op, err := client.UpdateContainer(name, container.Writable(), etag)
+		if err != nil {
+			return err
+		}
+
+		err = op.Wait()
 		if err != nil {
 			return err
 		}
 	}
 
-	return err
+	return nil
 }
 
-func (c *configCmd) deviceUnset(config *lxd.Config, which string, args []string) error {
+func (c *configCmd) deviceUnset(conf *config.Config, which string, args []string) error {
 	if len(args) < 5 {
 		return errArgs
 	}
 
-	remote, name := config.ParseRemoteAndContainer(args[2])
+	remote, name, err := conf.ParseRemote(args[2])
+	if err != nil {
+		return err
+	}
 
-	client, err := lxd.NewClient(config, remote)
+	client, err := conf.GetContainerServer(remote)
 	if err != nil {
 		return err
 	}
@@ -766,127 +935,181 @@ func (c *configCmd) deviceUnset(config *lxd.Config, which string, args []string)
 	key := args[4]
 
 	if which == "profile" {
-		st, err := client.ProfileConfig(name)
+		profile, etag, err := client.GetProfile(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := st.Devices[devname]
+		dev, ok := profile.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
-
 		delete(dev, key)
-		st.Devices[devname] = dev
+		profile.Devices[devname] = dev
 
-		err = client.PutProfile(name, *st)
+		err = client.UpdateProfile(name, profile.Writable(), etag)
 		if err != nil {
 			return err
 		}
 	} else {
-		st, err := client.ContainerInfo(name)
+		container, etag, err := client.GetContainer(name)
 		if err != nil {
 			return err
 		}
 
-		dev, ok := st.Devices[devname]
+		dev, ok := container.Devices[devname]
 		if !ok {
 			return fmt.Errorf(i18n.G("The device doesn't exist"))
 		}
-
 		delete(dev, key)
-		st.Devices[devname] = dev
+		container.Devices[devname] = dev
 
-		err = client.UpdateContainerConfig(name, st.Brief())
+		op, err := client.UpdateContainer(name, container.Writable(), etag)
+		if err != nil {
+			return err
+		}
+
+		err = op.Wait()
 		if err != nil {
 			return err
 		}
 	}
 
-	return err
-}
-
-func (c *configCmd) deviceRm(config *lxd.Config, which string, args []string) error {
-	if len(args) < 4 {
-		return errArgs
-	}
-	remote, name := config.ParseRemoteAndContainer(args[2])
-
-	client, err := lxd.NewClient(config, remote)
-	if err != nil {
-		return err
-	}
-
-	devname := args[3]
-	var resp *lxd.Response
-	if which == "profile" {
-		resp, err = client.ProfileDeviceDelete(name, devname)
-	} else {
-		resp, err = client.ContainerDeviceDelete(name, devname)
-	}
-	if err != nil {
-		return err
-	}
-	if which != "profile" {
-		err = client.WaitForSuccess(resp.Operation)
-	}
-	if err == nil {
-		fmt.Printf(i18n.G("Device %s removed from %s")+"\n", devname, name)
-	}
-	return err
-}
-
-func (c *configCmd) deviceList(config *lxd.Config, which string, args []string) error {
-	if len(args) < 3 {
-		return errArgs
-	}
-	remote, name := config.ParseRemoteAndContainer(args[2])
-
-	client, err := lxd.NewClient(config, remote)
-	if err != nil {
-		return err
-	}
-
-	var resp []string
-	if which == "profile" {
-		resp, err = client.ProfileListDevices(name)
-	} else {
-		resp, err = client.ContainerListDevices(name)
-	}
-	if err != nil {
-		return err
-	}
-	fmt.Printf("%s\n", strings.Join(resp, "\n"))
-
 	return nil
 }
 
-func (c *configCmd) deviceShow(config *lxd.Config, which string, args []string) error {
-	if len(args) < 3 {
+func (c *configCmd) deviceRm(conf *config.Config, which string, args []string) error {
+	if len(args) < 4 {
 		return errArgs
 	}
-	remote, name := config.ParseRemoteAndContainer(args[2])
 
-	client, err := lxd.NewClient(config, remote)
+	remote, name, err := conf.ParseRemote(args[2])
+	if err != nil {
+		return err
+	}
+
+	client, err := conf.GetContainerServer(remote)
 	if err != nil {
 		return err
 	}
 
-	var devices map[string]shared.Device
 	if which == "profile" {
-		resp, err := client.ProfileConfig(name)
+		profile, etag, err := client.GetProfile(name)
 		if err != nil {
 			return err
 		}
 
-		devices = resp.Devices
+		for _, devname := range args[3:] {
+			_, ok := profile.Devices[devname]
+			if !ok {
+				return fmt.Errorf(i18n.G("The device doesn't exist"))
+			}
+			delete(profile.Devices, devname)
+		}
+
+		err = client.UpdateProfile(name, profile.Writable(), etag)
+		if err != nil {
+			return err
+		}
 	} else {
-		resp, err := client.ContainerInfo(name)
+		container, etag, err := client.GetContainer(name)
 		if err != nil {
 			return err
 		}
 
-		devices = resp.Devices
+		for _, devname := range args[3:] {
+			_, ok := container.Devices[devname]
+			if !ok {
+				return fmt.Errorf(i18n.G("The device doesn't exist"))
+			}
+			delete(container.Devices, devname)
+		}
+
+		op, err := client.UpdateContainer(name, container.Writable(), etag)
+		if err != nil {
+			return err
+		}
+
+		err = op.Wait()
+		if err != nil {
+			return err
+		}
+	}
+
+	fmt.Printf(i18n.G("Device %s removed from %s")+"\n", strings.Join(args[3:], ", "), name)
+	return nil
+}
+
+func (c *configCmd) deviceList(conf *config.Config, which string, args []string) error {
+	if len(args) < 3 {
+		return errArgs
+	}
+
+	remote, name, err := conf.ParseRemote(args[2])
+	if err != nil {
+		return err
+	}
+
+	client, err := conf.GetContainerServer(remote)
+	if err != nil {
+		return err
+	}
+
+	var devices []string
+	if which == "profile" {
+		profile, _, err := client.GetProfile(name)
+		if err != nil {
+			return err
+		}
+
+		for k := range profile.Devices {
+			devices = append(devices, k)
+		}
+	} else {
+		container, _, err := client.GetContainer(name)
+		if err != nil {
+			return err
+		}
+
+		for k := range container.Devices {
+			devices = append(devices, k)
+		}
+	}
+
+	fmt.Printf("%s\n", strings.Join(devices, "\n"))
+	return nil
+}
+
+func (c *configCmd) deviceShow(conf *config.Config, which string, args []string) error {
+	if len(args) < 3 {
+		return errArgs
+	}
+
+	remote, name, err := conf.ParseRemote(args[2])
+	if err != nil {
+		return err
+	}
+
+	client, err := conf.GetContainerServer(remote)
+	if err != nil {
+		return err
+	}
+
+	var devices map[string]map[string]string
+	if which == "profile" {
+		profile, _, err := client.GetProfile(name)
+		if err != nil {
+			return err
+		}
+
+		devices = profile.Devices
+	} else {
+		container, _, err := client.GetContainer(name)
+		if err != nil {
+			return err
+		}
+
+		devices = container.Devices
 	}
 
 	data, err := yaml.Marshal(&devices)
@@ -895,6 +1118,5 @@ func (c *configCmd) deviceShow(config *lxd.Config, which string, args []string)
 	}
 
 	fmt.Printf(string(data))
-
 	return nil
 }
