@@ -6,8 +6,10 @@ import (
 	"os"
 	"strings"
 
-	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd/lxc/config"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 )
@@ -23,18 +25,16 @@ func (c *deleteCmd) showByDefault() bool {
 
 func (c *deleteCmd) usage() string {
 	return i18n.G(
-		`Delete containers or snapshots.
+		`Usage: lxc delete [<remote>:]<container>[/<snapshot>] [[<remote>:]<container>[/<snapshot>]...]
 
-lxc delete [remote:]<container>[/<snapshot>] [remote:][<container>[/<snapshot>]...]
-
-Destroy containers or snapshots with any attached data (configuration, snapshots, ...).`)
+Delete containers and snapshots.`)
 }
 
 func (c *deleteCmd) flags() {
-	gnuflag.BoolVar(&c.force, "f", false, i18n.G("Force the removal of stopped containers."))
-	gnuflag.BoolVar(&c.force, "force", false, i18n.G("Force the removal of stopped containers."))
-	gnuflag.BoolVar(&c.interactive, "i", false, i18n.G("Require user confirmation."))
-	gnuflag.BoolVar(&c.interactive, "interactive", false, i18n.G("Require user confirmation."))
+	gnuflag.BoolVar(&c.force, "f", false, i18n.G("Force the removal of running containers"))
+	gnuflag.BoolVar(&c.force, "force", false, i18n.G("Force the removal of running containers"))
+	gnuflag.BoolVar(&c.interactive, "i", false, i18n.G("Require user confirmation"))
+	gnuflag.BoolVar(&c.interactive, "interactive", false, i18n.G("Require user confirmation"))
 }
 
 func (c *deleteCmd) promptDelete(name string) error {
@@ -49,24 +49,37 @@ func (c *deleteCmd) promptDelete(name string) error {
 	return nil
 }
 
-func (c *deleteCmd) doDelete(d *lxd.Client, name string) error {
-	resp, err := d.Delete(name)
+func (c *deleteCmd) doDelete(d lxd.ContainerServer, name string) error {
+	var op *lxd.Operation
+	var err error
+
+	if shared.IsSnapshot(name) {
+		// Snapshot delete
+		fields := strings.SplitN(name, shared.SnapshotDelimiter, 2)
+		op, err = d.DeleteContainerSnapshot(fields[0], fields[1])
+	} else {
+		// Container delete
+		op, err = d.DeleteContainer(name)
+	}
 	if err != nil {
 		return err
 	}
 
-	return d.WaitForSuccess(resp.Operation)
+	return op.Wait()
 }
 
-func (c *deleteCmd) run(config *lxd.Config, args []string) error {
+func (c *deleteCmd) run(conf *config.Config, args []string) error {
 	if len(args) == 0 {
 		return errArgs
 	}
 
 	for _, nameArg := range args {
-		remote, name := config.ParseRemoteAndContainer(nameArg)
+		remote, name, err := conf.ParseRemote(nameArg)
+		if err != nil {
+			return err
+		}
 
-		d, err := lxd.NewClient(config, remote)
+		d, err := conf.GetContainerServer(remote)
 		if err != nil {
 			return err
 		}
@@ -82,28 +95,30 @@ func (c *deleteCmd) run(config *lxd.Config, args []string) error {
 			return c.doDelete(d, name)
 		}
 
-		ct, err := d.ContainerInfo(name)
+		ct, _, err := d.GetContainer(name)
 		if err != nil {
 			return err
 		}
 
-		if ct.StatusCode != 0 && ct.StatusCode != shared.Stopped {
+		if ct.StatusCode != 0 && ct.StatusCode != api.Stopped {
 			if !c.force {
 				return fmt.Errorf(i18n.G("The container is currently running, stop it first or pass --force."))
 			}
 
-			resp, err := d.Action(name, shared.Stop, -1, true, false)
+			req := api.ContainerStatePut{
+				Action:  "stop",
+				Timeout: -1,
+				Force:   true,
+			}
+
+			op, err := d.UpdateContainerState(name, req, "")
 			if err != nil {
 				return err
 			}
 
-			op, err := d.WaitFor(resp.Operation)
+			err = op.Wait()
 			if err != nil {
-				return err
-			}
-
-			if op.StatusCode == shared.Failure {
-				return fmt.Errorf(i18n.G("Stopping container failed!"))
+				return fmt.Errorf(i18n.G("Stopping the container failed: %s"), err)
 			}
 
 			if ct.Ephemeral == true {
