@@ -10,22 +10,24 @@ import (
 
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd"
-	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
+	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/termios"
 )
 
-type envFlag []string
+type envList []string
 
-func (f *envFlag) String() string {
+func (f *envList) String() string {
 	return fmt.Sprint(*f)
 }
 
-func (f *envFlag) Set(value string) error {
+func (f *envList) Set(value string) error {
 	if f == nil {
-		*f = make(envFlag, 1)
+		*f = make(envList, 1)
 	} else {
 		*f = append(*f, value)
 	}
@@ -34,7 +36,7 @@ func (f *envFlag) Set(value string) error {
 
 type execCmd struct {
 	modeFlag string
-	envArgs  envFlag
+	envArgs  envList
 }
 
 func (c *execCmd) showByDefault() bool {
@@ -43,15 +45,15 @@ func (c *execCmd) showByDefault() bool {
 
 func (c *execCmd) usage() string {
 	return i18n.G(
-		`Execute the specified command in a container.
+		`Usage: lxc exec [<remote>:]<container> [--mode=auto|interactive|non-interactive] [--env KEY=VALUE...] [--] <command line>
 
-lxc exec [remote:]container [--mode=auto|interactive|non-interactive] [--env EDITOR=/usr/bin/vim]... [--] <command line>
+Execute commands in containers.
 
 Mode defaults to non-interactive, interactive mode is selected if both stdin AND stdout are terminals (stderr is ignored).`)
 }
 
 func (c *execCmd) flags() {
-	gnuflag.Var(&c.envArgs, "env", i18n.G("An environment variable of the form HOME=/home/foo"))
+	gnuflag.Var(&c.envArgs, "env", i18n.G("Environment variable to set (e.g. HOME=/home/foo)"))
 	gnuflag.StringVar(&c.modeFlag, "mode", "auto", i18n.G("Override the terminal mode (auto, interactive or non-interactive)"))
 }
 
@@ -61,14 +63,14 @@ func (c *execCmd) sendTermSize(control *websocket.Conn) error {
 		return err
 	}
 
-	shared.LogDebugf("Window size is now: %dx%d", width, height)
+	logger.Debugf("Window size is now: %dx%d", width, height)
 
 	w, err := control.NextWriter(websocket.TextMessage)
 	if err != nil {
 		return err
 	}
 
-	msg := shared.ContainerExecControl{}
+	msg := api.ContainerExecControl{}
 	msg.Command = "window-resize"
 	msg.Args = make(map[string]string)
 	msg.Args["width"] = strconv.Itoa(width)
@@ -84,17 +86,24 @@ func (c *execCmd) sendTermSize(control *websocket.Conn) error {
 	return err
 }
 
-func (c *execCmd) run(config *lxd.Config, args []string) error {
+func (c *execCmd) run(conf *config.Config, args []string) error {
 	if len(args) < 2 {
 		return errArgs
 	}
 
-	remote, name := config.ParseRemoteAndContainer(args[0])
-	d, err := lxd.NewClient(config, remote)
+	remote, name, err := conf.ParseRemote(args[0])
 	if err != nil {
 		return err
 	}
 
+	d, err := conf.GetContainerServer(remote)
+	if err != nil {
+		return err
+	}
+
+	/* FIXME: Default values for HOME and USER are now handled by LXD.
+	   This code should be removed after most users upgraded.
+	*/
 	env := map[string]string{"HOME": "/root", "USER": "root"}
 	if myTerm, ok := c.getTERM(); ok {
 		env["TERM"] = myTerm
@@ -143,11 +152,39 @@ func (c *execCmd) run(config *lxd.Config, args []string) error {
 	}
 
 	stdout := c.getStdout()
-	ret, err := d.Exec(name, args[1:], env, os.Stdin, stdout, os.Stderr, handler, width, height)
+
+	req := api.ContainerExecPost{
+		Command:     args[1:],
+		WaitForWS:   true,
+		Interactive: interactive,
+		Environment: env,
+		Width:       width,
+		Height:      height,
+	}
+
+	execArgs := lxd.ContainerExecArgs{
+		Stdin:    os.Stdin,
+		Stdout:   stdout,
+		Stderr:   os.Stderr,
+		Control:  handler,
+		DataDone: make(chan bool),
+	}
+
+	// Run the command in the container
+	op, err := d.ExecContainer(name, req, &execArgs)
 	if err != nil {
 		return err
 	}
 
+	// Wait for the operation to complete
+	err = op.Wait()
+	if err != nil {
+		return err
+	}
+
+	// Wait for any remaining I/O to be flushed
+	<-execArgs.DataDone
+
 	if oldttystate != nil {
 		/* A bit of a special case here: we want to exit with the same code as
 		 * the process inside the container, so we explicitly exit here
@@ -159,6 +196,6 @@ func (c *execCmd) run(config *lxd.Config, args []string) error {
 		termios.Restore(cfd, oldttystate)
 	}
 
-	os.Exit(ret)
-	return fmt.Errorf(i18n.G("unreachable return reached"))
+	os.Exit(int(op.Metadata["return"].(float64)))
+	return nil
 }
