@@ -10,8 +10,7 @@ import (
 
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd"
-	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/logger"
 )
 
 func (c *execCmd) getStdout() io.WriteCloser {
@@ -22,18 +21,18 @@ func (c *execCmd) getTERM() (string, bool) {
 	return os.LookupEnv("TERM")
 }
 
-func (c *execCmd) controlSocketHandler(d *lxd.Client, control *websocket.Conn) {
+func (c *execCmd) controlSocketHandler(control *websocket.Conn) {
 	ch := make(chan os.Signal)
 	signal.Notify(ch, syscall.SIGWINCH)
 
 	for {
 		sig := <-ch
 
-		shared.LogDebugf("Received '%s signal', updating window geometry.", sig)
+		logger.Debugf("Received '%s signal', updating window geometry.", sig)
 
 		err := c.sendTermSize(control)
 		if err != nil {
-			shared.LogDebugf("error setting term size %s", err)
+			logger.Debugf("error setting term size %s", err)
 			break
 		}
 	}
