@@ -7,7 +7,8 @@ import (
 
 	"gopkg.in/yaml.v2"
 
-	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd/lxc/config"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
@@ -23,29 +24,38 @@ func (c *infoCmd) showByDefault() bool {
 
 func (c *infoCmd) usage() string {
 	return i18n.G(
-		`List information on LXD servers and containers.
+		`Usage: lxc info [<remote>:][<container>] [--show-log]
 
-For a container:
- lxc info [<remote>:]container [--show-log]
+Show container or server information.
 
-For a server:
- lxc info [<remote>:]`)
+lxc info [<remote>:]<container> [--show-log]
+    For container information.
+
+lxc info [<remote>:]
+    For LXD server information.`)
 }
 
 func (c *infoCmd) flags() {
 	gnuflag.BoolVar(&c.showLog, "show-log", false, i18n.G("Show the container's last 100 log lines?"))
 }
 
-func (c *infoCmd) run(config *lxd.Config, args []string) error {
+func (c *infoCmd) run(conf *config.Config, args []string) error {
 	var remote string
 	var cName string
+	var err error
 	if len(args) == 1 {
-		remote, cName = config.ParseRemoteAndContainer(args[0])
+		remote, cName, err = conf.ParseRemote(args[0])
+		if err != nil {
+			return err
+		}
 	} else {
-		remote, cName = config.ParseRemoteAndContainer("")
+		remote, cName, err = conf.ParseRemote("")
+		if err != nil {
+			return err
+		}
 	}
 
-	d, err := lxd.NewClient(config, remote)
+	d, err := conf.GetContainerServer(remote)
 	if err != nil {
 		return err
 	}
@@ -53,12 +63,12 @@ func (c *infoCmd) run(config *lxd.Config, args []string) error {
 	if cName == "" {
 		return c.remoteInfo(d)
 	} else {
-		return c.containerInfo(d, cName, c.showLog)
+		return c.containerInfo(d, conf.Remotes[remote], cName, c.showLog)
 	}
 }
 
-func (c *infoCmd) remoteInfo(d *lxd.Client) error {
-	serverStatus, err := d.ServerStatus()
+func (c *infoCmd) remoteInfo(d lxd.ContainerServer) error {
+	serverStatus, _, err := d.GetServer()
 	if err != nil {
 		return err
 	}
@@ -73,13 +83,13 @@ func (c *infoCmd) remoteInfo(d *lxd.Client) error {
 	return nil
 }
 
-func (c *infoCmd) containerInfo(d *lxd.Client, name string, showLog bool) error {
-	ct, err := d.ContainerInfo(name)
+func (c *infoCmd) containerInfo(d lxd.ContainerServer, remote config.Remote, name string, showLog bool) error {
+	ct, _, err := d.GetContainer(name)
 	if err != nil {
 		return err
 	}
 
-	cs, err := d.ContainerState(name)
+	cs, _, err := d.GetContainerState(name)
 	if err != nil {
 		return err
 	}
@@ -87,12 +97,13 @@ func (c *infoCmd) containerInfo(d *lxd.Client, name string, showLog bool) error
 	const layout = "2006/01/02 15:04 UTC"
 
 	fmt.Printf(i18n.G("Name: %s")+"\n", ct.Name)
-	if d.Remote != nil && d.Remote.Addr != "" {
-		fmt.Printf(i18n.G("Remote: %s")+"\n", d.Remote.Addr)
+	if remote.Addr != "" {
+		fmt.Printf(i18n.G("Remote: %s")+"\n", remote.Addr)
 	}
+
 	fmt.Printf(i18n.G("Architecture: %s")+"\n", ct.Architecture)
-	if ct.CreationDate.UTC().Unix() != 0 {
-		fmt.Printf(i18n.G("Created: %s")+"\n", ct.CreationDate.UTC().Format(layout))
+	if shared.TimeIsSet(ct.CreatedAt) {
+		fmt.Printf(i18n.G("Created: %s")+"\n", ct.CreatedAt.UTC().Format(layout))
 	}
 
 	fmt.Printf(i18n.G("Status: %s")+"\n", ct.Status)
@@ -134,36 +145,36 @@ func (c *infoCmd) containerInfo(d *lxd.Client, name string, showLog bool) error
 		if cs.Disk != nil {
 			for entry, disk := range cs.Disk {
 				if disk.Usage != 0 {
-					diskInfo += fmt.Sprintf("    %s: %s\n", entry, shared.GetByteSizeString(disk.Usage))
+					diskInfo += fmt.Sprintf("    %s: %s\n", entry, shared.GetByteSizeString(disk.Usage, 2))
 				}
 			}
 		}
 
 		if diskInfo != "" {
-			fmt.Println(i18n.G("  Disk usage:"))
+			fmt.Println(fmt.Sprintf("  %s", i18n.G("Disk usage:")))
 			fmt.Printf(diskInfo)
 		}
 
 		// Memory usage
 		memoryInfo := ""
 		if cs.Memory.Usage != 0 {
-			memoryInfo += fmt.Sprintf("    %s: %s\n", i18n.G("Memory (current)"), shared.GetByteSizeString(cs.Memory.Usage))
+			memoryInfo += fmt.Sprintf("    %s: %s\n", i18n.G("Memory (current)"), shared.GetByteSizeString(cs.Memory.Usage, 2))
 		}
 
 		if cs.Memory.UsagePeak != 0 {
-			memoryInfo += fmt.Sprintf("    %s: %s\n", i18n.G("Memory (peak)"), shared.GetByteSizeString(cs.Memory.UsagePeak))
+			memoryInfo += fmt.Sprintf("    %s: %s\n", i18n.G("Memory (peak)"), shared.GetByteSizeString(cs.Memory.UsagePeak, 2))
 		}
 
 		if cs.Memory.SwapUsage != 0 {
-			memoryInfo += fmt.Sprintf("    %s: %s\n", i18n.G("Swap (current)"), shared.GetByteSizeString(cs.Memory.SwapUsage))
+			memoryInfo += fmt.Sprintf("    %s: %s\n", i18n.G("Swap (current)"), shared.GetByteSizeString(cs.Memory.SwapUsage, 2))
 		}
 
 		if cs.Memory.SwapUsagePeak != 0 {
-			memoryInfo += fmt.Sprintf("    %s: %s\n", i18n.G("Swap (peak)"), shared.GetByteSizeString(cs.Memory.SwapUsagePeak))
+			memoryInfo += fmt.Sprintf("    %s: %s\n", i18n.G("Swap (peak)"), shared.GetByteSizeString(cs.Memory.SwapUsagePeak, 2))
 		}
 
 		if memoryInfo != "" {
-			fmt.Println(i18n.G("  Memory usage:"))
+			fmt.Println(fmt.Sprintf("  %s", i18n.G("Memory usage:")))
 			fmt.Printf(memoryInfo)
 		}
 
@@ -172,22 +183,22 @@ func (c *infoCmd) containerInfo(d *lxd.Client, name string, showLog bool) error
 		if cs.Network != nil {
 			for netName, net := range cs.Network {
 				networkInfo += fmt.Sprintf("    %s:\n", netName)
-				networkInfo += fmt.Sprintf("      %s: %s\n", i18n.G("Bytes received"), shared.GetByteSizeString(net.Counters.BytesReceived))
-				networkInfo += fmt.Sprintf("      %s: %s\n", i18n.G("Bytes sent"), shared.GetByteSizeString(net.Counters.BytesSent))
+				networkInfo += fmt.Sprintf("      %s: %s\n", i18n.G("Bytes received"), shared.GetByteSizeString(net.Counters.BytesReceived, 2))
+				networkInfo += fmt.Sprintf("      %s: %s\n", i18n.G("Bytes sent"), shared.GetByteSizeString(net.Counters.BytesSent, 2))
 				networkInfo += fmt.Sprintf("      %s: %d\n", i18n.G("Packets received"), net.Counters.PacketsReceived)
 				networkInfo += fmt.Sprintf("      %s: %d\n", i18n.G("Packets sent"), net.Counters.PacketsSent)
 			}
 		}
 
 		if networkInfo != "" {
-			fmt.Println(i18n.G("  Network usage:"))
+			fmt.Println(fmt.Sprintf("  %s", i18n.G("Network usage:")))
 			fmt.Printf(networkInfo)
 		}
 	}
 
 	// List snapshots
 	first_snapshot := true
-	snaps, err := d.ListSnapshots(name)
+	snaps, err := d.GetContainerSnapshots(name)
 	if err != nil {
 		return nil
 	}
@@ -200,7 +211,7 @@ func (c *infoCmd) containerInfo(d *lxd.Client, name string, showLog bool) error
 		fields := strings.Split(snap.Name, shared.SnapshotDelimiter)
 		fmt.Printf("  %s", fields[len(fields)-1])
 
-		if snap.CreationDate.UTC().Unix() != 0 {
+		if shared.TimeIsSet(snap.CreationDate) {
 			fmt.Printf(" ("+i18n.G("taken at %s")+")", snap.CreationDate.UTC().Format(layout))
 		}
 
@@ -215,7 +226,7 @@ func (c *infoCmd) containerInfo(d *lxd.Client, name string, showLog bool) error
 	}
 
 	if showLog {
-		log, err := d.GetLog(name, "lxc.log")
+		log, err := d.GetContainerLogfile(name, "lxc.log")
 		if err != nil {
 			return err
 		}
