@@ -4,24 +4,31 @@ import (
 	"fmt"
 	"os"
 	"os/exec"
+	"os/user"
 	"path"
+	"path/filepath"
 	"strings"
 	"syscall"
 
-	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/lxc/config"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
+	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/logging"
+	"github.com/lxc/lxd/shared/version"
 )
 
 var configPath string
+var execName string
 
 func main() {
+	execName = os.Args[0]
+
 	if err := run(); err != nil {
 		msg := fmt.Sprintf(i18n.G("error: %v"), err)
 
-		lxdErr := lxd.GetLocalLXDErr(err)
+		lxdErr := getLocalErr(err)
 		switch lxdErr {
 		case syscall.ENOENT:
 			msg = i18n.G("LXD socket not found; is LXD installed and running?")
@@ -37,14 +44,23 @@ func main() {
 }
 
 func run() error {
-	verbose := gnuflag.Bool("verbose", false, i18n.G("Enables verbose mode."))
-	debug := gnuflag.Bool("debug", false, i18n.G("Enables debug mode."))
-	forceLocal := gnuflag.Bool("force-local", false, i18n.G("Force using the local unix socket."))
-	noAlias := gnuflag.Bool("no-alias", false, i18n.G("Ignore aliases when determining what command to run."))
+	verbose := gnuflag.Bool("verbose", false, i18n.G("Enable verbose mode"))
+	debug := gnuflag.Bool("debug", false, i18n.G("Enable debug mode"))
+	forceLocal := gnuflag.Bool("force-local", false, i18n.G("Force using the local unix socket"))
+	noAlias := gnuflag.Bool("no-alias", false, i18n.G("Ignore aliases when determining what command to run"))
 
-	configDir := "$HOME/.config/lxc"
+	var configDir string
 	if os.Getenv("LXD_CONF") != "" {
 		configDir = os.Getenv("LXD_CONF")
+	} else if os.Getenv("HOME") != "" {
+		configDir = path.Join(os.Getenv("HOME"), ".config", "lxc")
+	} else {
+		user, err := user.Current()
+		if err != nil {
+			return err
+		}
+
+		configDir = path.Join(user.HomeDir, ".config", "lxc")
 	}
 	configPath = os.ExpandEnv(path.Join(configDir, "config.yml"))
 
@@ -58,12 +74,11 @@ func run() error {
 	}
 
 	if len(os.Args) >= 2 && (os.Args[1] == "--all") {
-		os.Args[1] = "help"
-		os.Args = append(os.Args, "--all")
+		os.Args = []string{os.Args[0], "help", "--all"}
 	}
 
-	if len(os.Args) == 2 && os.Args[1] == "--version" {
-		os.Args[1] = "version"
+	if shared.StringInSlice("--version", os.Args) {
+		os.Args = []string{os.Args[0], "version"}
 	}
 
 	if len(os.Args) < 2 {
@@ -71,18 +86,24 @@ func run() error {
 		os.Exit(1)
 	}
 
-	var config *lxd.Config
+	var conf *config.Config
 	var err error
 
 	if *forceLocal {
-		config = &lxd.DefaultConfig
-	} else {
-		config, err = lxd.LoadConfig(configPath)
+		conf = &config.DefaultConfig
+	} else if shared.PathExists(configPath) {
+		conf, err = config.LoadConfig(configPath)
 		if err != nil {
 			return err
 		}
+	} else {
+		conf = &config.DefaultConfig
+		conf.ConfigDir = filepath.Dir(configPath)
 	}
 
+	// Set the user agent
+	conf.UserAgent = version.UserAgent
+
 	// This is quite impolite, but it seems gnuflag needs us to shift our
 	// own exename out of the arguments before parsing them. However, this
 	// is useful for execIfAlias, which wants to know exactly the command
@@ -95,7 +116,7 @@ func run() error {
 	 * --no-alias by hand.
 	 */
 	if !shared.StringInSlice("--no-alias", origArgs) {
-		execIfAliases(config, origArgs)
+		execIfAliases(conf, origArgs)
 	}
 	cmd, ok := commands[name]
 	if !ok {
@@ -105,20 +126,24 @@ func run() error {
 	}
 	cmd.flags()
 	gnuflag.Usage = func() {
-		fmt.Fprintf(os.Stderr, i18n.G("Usage: %s")+"\n\n"+i18n.G("Options:")+"\n\n", strings.TrimSpace(cmd.usage()))
+		fmt.Print(cmd.usage())
+		fmt.Printf("\n\n%s\n", i18n.G("Options:"))
+
+		gnuflag.SetOut(os.Stdout)
 		gnuflag.PrintDefaults()
+		os.Exit(0)
 	}
 
 	os.Args = os.Args[1:]
 	gnuflag.Parse(true)
 
-	shared.Log, err = logging.GetLogger("", "", *verbose, *debug, nil)
+	logger.Log, err = logging.GetLogger("", "", *verbose, *debug, nil)
 	if err != nil {
 		return err
 	}
 
-	certf := config.ConfigPath("client.crt")
-	keyf := config.ConfigPath("client.key")
+	certf := conf.ConfigPath("client.crt")
+	keyf := conf.ConfigPath("client.key")
 
 	if !*forceLocal && os.Args[0] != "help" && os.Args[0] != "version" && (!shared.PathExists(certf) || !shared.PathExists(keyf)) {
 		fmt.Fprintf(os.Stderr, i18n.G("Generating a client certificate. This may take a minute...")+"\n")
@@ -134,17 +159,36 @@ func run() error {
 		}
 	}
 
-	err = cmd.run(config, gnuflag.Args())
-	if err == errArgs {
-		/* If we got an error about invalid arguments, let's try to
-		 * expand this as an alias
-		 */
-		if !*noAlias {
-			execIfAliases(config, origArgs)
+	err = cmd.run(conf, gnuflag.Args())
+	if err == errArgs || err == errUsage {
+		out := os.Stdout
+		if err == errArgs {
+			/* If we got an error about invalid arguments, let's try to
+			 * expand this as an alias
+			 */
+			if !*noAlias {
+				execIfAliases(conf, origArgs)
+			}
+
+			out = os.Stderr
 		}
-		fmt.Fprintf(os.Stderr, "%s\n\n"+i18n.G("error: %v")+"\n", cmd.usage(), err)
-		os.Exit(1)
+		gnuflag.SetOut(out)
+
+		if err == errArgs {
+			fmt.Fprintf(out, i18n.G("error: %v"), err)
+			fmt.Fprintf(out, "\n\n")
+		}
+		fmt.Fprint(out, cmd.usage())
+		fmt.Fprintf(out, "\n\n%s\n", i18n.G("Options:"))
+
+		gnuflag.PrintDefaults()
+
+		if err == errArgs {
+			os.Exit(1)
+		}
+		os.Exit(0)
 	}
+
 	return err
 }
 
@@ -152,7 +196,7 @@ type command interface {
 	usage() string
 	flags()
 	showByDefault() bool
-	run(config *lxd.Config, args []string) error
+	run(config *config.Config, args []string) error
 }
 
 var commands = map[string]command{
@@ -171,40 +215,44 @@ var commands = map[string]command{
 	"monitor": &monitorCmd{},
 	"move":    &moveCmd{},
 	"pause": &actionCmd{
-		action:         shared.Freeze,
-		name:           "pause",
-		additionalHelp: i18n.G("The opposite of `lxc pause` is `lxc start`."),
+		action:      shared.Freeze,
+		description: i18n.G("Pause containers."),
+		name:        "pause",
 	},
 	"profile": &profileCmd{},
 	"publish": &publishCmd{},
 	"remote":  &remoteCmd{},
 	"restart": &actionCmd{
-		action:     shared.Restart,
-		hasTimeout: true,
-		visible:    true,
-		name:       "restart",
-		timeout:    -1,
+		action:      shared.Restart,
+		description: i18n.G("Restart containers."),
+		hasTimeout:  true,
+		visible:     true,
+		name:        "restart",
+		timeout:     -1,
 	},
 	"restore":  &restoreCmd{},
 	"snapshot": &snapshotCmd{},
 	"start": &actionCmd{
-		action:  shared.Start,
-		visible: true,
-		name:    "start",
+		action:      shared.Start,
+		description: i18n.G("Start containers."),
+		visible:     true,
+		name:        "start",
 	},
 	"stop": &actionCmd{
-		action:     shared.Stop,
-		hasTimeout: true,
-		visible:    true,
-		name:       "stop",
-		timeout:    -1,
+		action:      shared.Stop,
+		description: i18n.G("Stop containers."),
+		hasTimeout:  true,
+		visible:     true,
+		name:        "stop",
+		timeout:     -1,
 	},
 	"version": &versionCmd{},
 }
 
 var errArgs = fmt.Errorf(i18n.G("wrong number of subcommand arguments"))
+var errUsage = fmt.Errorf("show usage")
 
-func expandAlias(config *lxd.Config, origArgs []string) ([]string, bool) {
+func expandAlias(config *config.Config, origArgs []string) ([]string, bool) {
 	foundAlias := false
 	aliasKey := []string{}
 	aliasValue := []string{}
@@ -254,7 +302,7 @@ func expandAlias(config *lxd.Config, origArgs []string) ([]string, bool) {
 	return newArgs, true
 }
 
-func execIfAliases(config *lxd.Config, origArgs []string) {
+func execIfAliases(config *config.Config, origArgs []string) {
 	newArgs, expanded := expandAlias(config, origArgs)
 	if !expanded {
 		return
@@ -269,41 +317,3 @@ func execIfAliases(config *lxd.Config, origArgs []string) {
 	fmt.Fprintf(os.Stderr, i18n.G("processing aliases failed %s\n"), ret)
 	os.Exit(5)
 }
-
-type ProgressRenderer struct {
-	Format string
-
-	maxLength int
-}
-
-func (p *ProgressRenderer) Done(msg string) {
-	if msg != "" {
-		msg += "\n"
-	}
-
-	if len(msg) > p.maxLength {
-		p.maxLength = len(msg)
-	} else {
-		fmt.Printf("\r%s", strings.Repeat(" ", p.maxLength))
-	}
-
-	fmt.Print("\r")
-	fmt.Print(msg)
-}
-
-func (p *ProgressRenderer) Update(status string) {
-	msg := "%s"
-	if p.Format != "" {
-		msg = p.Format
-	}
-
-	msg = fmt.Sprintf("\r"+msg, status)
-
-	if len(msg) > p.maxLength {
-		p.maxLength = len(msg)
-	} else {
-		fmt.Printf("\r%s", strings.Repeat(" ", p.maxLength))
-	}
-
-	fmt.Print(msg)
-}
