@@ -1,12 +1,10 @@
 package main
 
 import (
-	"crypto/sha256"
 	"crypto/x509"
 	"encoding/pem"
 	"fmt"
 	"net"
-	"net/http"
 	"net/url"
 	"os"
 	"path/filepath"
@@ -17,10 +15,13 @@ import (
 
 	"golang.org/x/crypto/ssh/terminal"
 
-	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd/lxc/config"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
+	"github.com/lxc/lxd/shared/logger"
 )
 
 type remoteCmd struct {
@@ -36,16 +37,30 @@ func (c *remoteCmd) showByDefault() bool {
 
 func (c *remoteCmd) usage() string {
 	return i18n.G(
-		`Manage remote LXD servers.
+		`Usage: lxc remote <subcommand> [options]
 
-lxc remote add <name> <url> [--accept-certificate] [--password=PASSWORD]
-                            [--public] [--protocol=PROTOCOL]                Add the remote <name> at <url>.
-lxc remote remove <name>                                                    Remove the remote <name>.
-lxc remote list                                                             List all remotes.
-lxc remote rename <old> <new>                                               Rename remote <old> to <new>.
-lxc remote set-url <name> <url>                                             Update <name>'s url to <url>.
-lxc remote set-default <name>                                               Set the default remote.
-lxc remote get-default                                                      Print the default remote.`)
+Manage the list of remote LXD servers.
+
+lxc remote add <remote> <IP|FQDN|URL> [--accept-certificate] [--password=PASSWORD] [--public] [--protocol=PROTOCOL]
+    Add the remote <remote> at <url>.
+
+lxc remote remove <remote>
+    Remove the remote <remote>.
+
+lxc remote list
+    List all remotes.
+
+lxc remote rename <old name> <new name>
+    Rename remote <old name> to <new name>.
+
+lxc remote set-url <remote> <url>
+    Update <remote>'s url to <url>.
+
+lxc remote set-default <remote>
+    Set the default remote.
+
+lxc remote get-default
+    Print the default remote.`)
 }
 
 func (c *remoteCmd) flags() {
@@ -55,43 +70,14 @@ func (c *remoteCmd) flags() {
 	gnuflag.BoolVar(&c.public, "public", false, i18n.G("Public image server"))
 }
 
-func getRemoteCertificate(address string) (*x509.Certificate, error) {
-	// Setup a permissive TLS config
-	tlsConfig, err := shared.GetTLSConfig("", "", nil)
-	if err != nil {
-		return nil, err
-	}
-
-	tlsConfig.InsecureSkipVerify = true
-	tr := &http.Transport{
-		TLSClientConfig: tlsConfig,
-		Dial:            shared.RFC3493Dialer,
-		Proxy:           shared.ProxyFromEnvironment,
-	}
-
-	// Connect
-	client := &http.Client{Transport: tr}
-	resp, err := client.Get(address)
-	if err != nil {
-		return nil, err
-	}
-
-	// Retrieve the certificate
-	if resp.TLS == nil || len(resp.TLS.PeerCertificates) == 0 {
-		return nil, fmt.Errorf(i18n.G("Unable to read remote TLS certificate"))
-	}
-
-	return resp.TLS.PeerCertificates[0], nil
-}
-
-func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, acceptCert bool, password string, public bool, protocol string) error {
+func (c *remoteCmd) addServer(conf *config.Config, server string, addr string, acceptCert bool, password string, public bool, protocol string) error {
 	var rScheme string
 	var rHost string
 	var rPort string
 
 	// Setup the remotes list
-	if config.Remotes == nil {
-		config.Remotes = make(map[string]lxd.RemoteConfig)
+	if conf.Remotes == nil {
+		conf.Remotes = make(map[string]config.Remote)
 	}
 
 	/* Complex remote URL parsing */
@@ -106,7 +92,7 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 			return fmt.Errorf(i18n.G("Only https URLs are supported for simplestreams"))
 		}
 
-		config.Remotes[server] = lxd.RemoteConfig{Addr: addr, Public: true, Protocol: protocol}
+		conf.Remotes[server] = config.Remote{Addr: addr, Public: true, Protocol: protocol}
 		return nil
 	}
 
@@ -162,37 +148,37 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 	}
 
 	/* Actually add the remote */
-	config.Remotes[server] = lxd.RemoteConfig{Addr: addr, Protocol: protocol}
+	conf.Remotes[server] = config.Remote{Addr: addr, Protocol: protocol}
 
-	remote := config.ParseRemote(server)
-	d, err := lxd.NewClient(config, remote)
-	if err != nil {
+	// Attempt to connect
+	var d interface{}
+	if public {
+		d, err = conf.GetImageServer(server)
+	} else {
+		d, err = conf.GetContainerServer(server)
+	}
+
+	// Handle Unix socket connections
+	if strings.HasPrefix(addr, "unix:") {
 		return err
 	}
 
-	if strings.HasPrefix(addr, "unix:") {
-		// NewClient succeeded so there was a lxd there (we fingered
-		// it) so just accept it
-		return nil
-	}
-
+	// Check if the system CA worked for the TLS connection
 	var certificate *x509.Certificate
-
-	/* Attempt to connect using the system root CA */
-	_, err = d.GetServerConfig()
 	if err != nil {
 		// Failed to connect using the system CA, so retrieve the remote certificate
-		certificate, err = getRemoteCertificate(addr)
+		certificate, err = shared.GetRemoteCertificate(addr)
 		if err != nil {
 			return err
 		}
 	}
 
+	// Handle certificate prompt
 	if certificate != nil {
 		if !acceptCert {
-			digest := sha256.Sum256(certificate.Raw)
+			digest := shared.CertFingerprint(certificate)
 
-			fmt.Printf(i18n.G("Certificate fingerprint: %x")+"\n", digest)
+			fmt.Printf(i18n.G("Certificate fingerprint: %s")+"\n", digest)
 			fmt.Printf(i18n.G("ok (y/n)?") + " ")
 			line, err := shared.ReadStdin()
 			if err != nil {
@@ -204,13 +190,13 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 			}
 		}
 
-		dnam := d.Config.ConfigPath("servercerts")
+		dnam := conf.ConfigPath("servercerts")
 		err := os.MkdirAll(dnam, 0750)
 		if err != nil {
 			return fmt.Errorf(i18n.G("Could not create server cert dir"))
 		}
 
-		certf := fmt.Sprintf("%s/%s.crt", dnam, d.Name)
+		certf := fmt.Sprintf("%s/%s.crt", dnam, server)
 		certOut, err := os.Create(certf)
 		if err != nil {
 			return err
@@ -220,27 +206,41 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 		certOut.Close()
 
 		// Setup a new connection, this time with the remote certificate
-		d, err = lxd.NewClient(config, remote)
+		if public {
+			d, err = conf.GetImageServer(server)
+		} else {
+			d, err = conf.GetContainerServer(server)
+		}
+
 		if err != nil {
 			return err
 		}
 	}
 
-	if d.IsPublic() || public {
-		config.Remotes[server] = lxd.RemoteConfig{Addr: addr, Public: true}
-
-		if _, err := d.GetServerConfig(); err != nil {
-			return err
-		}
-
+	// Handle public remotes
+	if public {
+		conf.Remotes[server] = config.Remote{Addr: addr, Public: true}
 		return nil
 	}
 
-	if d.AmTrusted() {
-		// server already has our cert, so we're done
+	// Get server information
+	srv, _, err := d.(lxd.ContainerServer).GetServer()
+	if err != nil {
+		return err
+	}
+
+	// Detect public remotes
+	if srv.Public {
+		conf.Remotes[server] = config.Remote{Addr: addr, Public: true}
 		return nil
 	}
 
+	// Check if our cert is already trusted
+	if srv.Auth == "trusted" {
+		return nil
+	}
+
+	// Prompt for trust password
 	if password == "" {
 		fmt.Printf(i18n.G("Admin password for %s: "), server)
 		pwd, err := terminal.ReadPassword(0)
@@ -256,12 +256,24 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 		password = string(pwd)
 	}
 
-	err = d.AddMyCertToServer(password)
+	// Add client certificate to trust store
+	req := api.CertificatesPost{
+		Password: password,
+	}
+	req.Type = "client"
+
+	err = d.(lxd.ContainerServer).CreateCertificate(req)
 	if err != nil {
 		return err
 	}
 
-	if !d.AmTrusted() {
+	// And check if trusted now
+	srv, _, err = d.(lxd.ContainerServer).GetServer()
+	if err != nil {
+		return err
+	}
+
+	if srv.Auth != "trusted" {
 		return fmt.Errorf(i18n.G("Server doesn't trust us after adding our cert"))
 	}
 
@@ -269,16 +281,16 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 	return nil
 }
 
-func (c *remoteCmd) removeCertificate(config *lxd.Config, remote string) {
-	certf := config.ServerCertPath(remote)
-	shared.LogDebugf("Trying to remove %s", certf)
+func (c *remoteCmd) removeCertificate(conf *config.Config, remote string) {
+	certf := conf.ServerCertPath(remote)
+	logger.Debugf("Trying to remove %s", certf)
 
 	os.Remove(certf)
 }
 
-func (c *remoteCmd) run(config *lxd.Config, args []string) error {
+func (c *remoteCmd) run(conf *config.Config, args []string) error {
 	if len(args) < 1 {
-		return errArgs
+		return errUsage
 	}
 
 	switch args[0] {
@@ -287,14 +299,14 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			return errArgs
 		}
 
-		if rc, ok := config.Remotes[args[1]]; ok {
+		if rc, ok := conf.Remotes[args[1]]; ok {
 			return fmt.Errorf(i18n.G("remote %s exists as <%s>"), args[1], rc.Addr)
 		}
 
-		err := c.addServer(config, args[1], args[2], c.acceptCert, c.password, c.public, c.protocol)
+		err := c.addServer(conf, args[1], args[2], c.acceptCert, c.password, c.public, c.protocol)
 		if err != nil {
-			delete(config.Remotes, args[1])
-			c.removeCertificate(config, args[1])
+			delete(conf.Remotes, args[1])
+			c.removeCertificate(conf, args[1])
 			return err
 		}
 
@@ -303,7 +315,7 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			return errArgs
 		}
 
-		rc, ok := config.Remotes[args[1]]
+		rc, ok := conf.Remotes[args[1]]
 		if !ok {
 			return fmt.Errorf(i18n.G("remote %s doesn't exist"), args[1])
 		}
@@ -312,17 +324,17 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			return fmt.Errorf(i18n.G("remote %s is static and cannot be modified"), args[1])
 		}
 
-		if config.DefaultRemote == args[1] {
+		if conf.DefaultRemote == args[1] {
 			return fmt.Errorf(i18n.G("can't remove the default remote"))
 		}
 
-		delete(config.Remotes, args[1])
+		delete(conf.Remotes, args[1])
 
-		c.removeCertificate(config, args[1])
+		c.removeCertificate(conf, args[1])
 
 	case "list":
 		data := [][]string{}
-		for name, rc := range config.Remotes {
+		for name, rc := range conf.Remotes {
 			strPublic := i18n.G("NO")
 			if rc.Public {
 				strPublic = i18n.G("YES")
@@ -338,7 +350,7 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			}
 
 			strName := name
-			if name == config.DefaultRemote {
+			if name == conf.DefaultRemote {
 				strName = fmt.Sprintf("%s (%s)", name, i18n.G("default"))
 			}
 			data = append(data, []string{strName, rc.Addr, rc.Protocol, strPublic, strStatic})
@@ -365,7 +377,7 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			return errArgs
 		}
 
-		rc, ok := config.Remotes[args[1]]
+		rc, ok := conf.Remotes[args[1]]
 		if !ok {
 			return fmt.Errorf(i18n.G("remote %s doesn't exist"), args[1])
 		}
@@ -374,13 +386,13 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			return fmt.Errorf(i18n.G("remote %s is static and cannot be modified"), args[1])
 		}
 
-		if _, ok := config.Remotes[args[2]]; ok {
+		if _, ok := conf.Remotes[args[2]]; ok {
 			return fmt.Errorf(i18n.G("remote %s already exists"), args[2])
 		}
 
 		// Rename the certificate file
-		oldPath := filepath.Join(config.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[1]))
-		newPath := filepath.Join(config.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[2]))
+		oldPath := filepath.Join(conf.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[1]))
+		newPath := filepath.Join(conf.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[2]))
 		if shared.PathExists(oldPath) {
 			err := os.Rename(oldPath, newPath)
 			if err != nil {
@@ -388,11 +400,11 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			}
 		}
 
-		config.Remotes[args[2]] = rc
-		delete(config.Remotes, args[1])
+		conf.Remotes[args[2]] = rc
+		delete(conf.Remotes, args[1])
 
-		if config.DefaultRemote == args[1] {
-			config.DefaultRemote = args[2]
+		if conf.DefaultRemote == args[1] {
+			conf.DefaultRemote = args[2]
 		}
 
 	case "set-url":
@@ -400,7 +412,7 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			return errArgs
 		}
 
-		rc, ok := config.Remotes[args[1]]
+		rc, ok := conf.Remotes[args[1]]
 		if !ok {
 			return fmt.Errorf(i18n.G("remote %s doesn't exist"), args[1])
 		}
@@ -409,28 +421,28 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			return fmt.Errorf(i18n.G("remote %s is static and cannot be modified"), args[1])
 		}
 
-		config.Remotes[args[1]] = lxd.RemoteConfig{Addr: args[2]}
+		conf.Remotes[args[1]] = config.Remote{Addr: args[2]}
 
 	case "set-default":
 		if len(args) != 2 {
 			return errArgs
 		}
 
-		_, ok := config.Remotes[args[1]]
+		_, ok := conf.Remotes[args[1]]
 		if !ok {
 			return fmt.Errorf(i18n.G("remote %s doesn't exist"), args[1])
 		}
-		config.DefaultRemote = args[1]
+		conf.DefaultRemote = args[1]
 
 	case "get-default":
 		if len(args) != 1 {
 			return errArgs
 		}
-		fmt.Println(config.DefaultRemote)
+		fmt.Println(conf.DefaultRemote)
 		return nil
 	default:
 		return errArgs
 	}
 
-	return lxd.SaveConfig(config, configPath)
+	return conf.SaveConfig(configPath)
 }
