@@ -3,8 +3,9 @@ package main
 import (
 	"fmt"
 
-	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/lxc/config"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 )
@@ -19,31 +20,37 @@ func (c *restoreCmd) showByDefault() bool {
 
 func (c *restoreCmd) usage() string {
 	return i18n.G(
-		`Set the current state of a container back to a snapshot.
+		`Usage: lxc restore [<remote>:]<container> <snapshot> [--stateful]
 
-lxc restore [remote:]<container> <snapshot name> [--stateful]
+Restore containers from snapshots.
 
-Restores a container from a snapshot (optionally with running state, see
-snapshot help for details).
+If --stateful is passed, then the running state will be restored too.
 
-For example:
-lxc snapshot u1 snap0 # create the snapshot
-lxc restore u1 snap0 # restore the snapshot`)
+*Examples*
+lxc snapshot u1 snap0
+    Create the snapshot.
+
+lxc restore u1 snap0
+    Restore the snapshot.`)
 }
 
 func (c *restoreCmd) flags() {
 	gnuflag.BoolVar(&c.stateful, "stateful", false, i18n.G("Whether or not to restore the container's running state from snapshot (if available)"))
 }
 
-func (c *restoreCmd) run(config *lxd.Config, args []string) error {
+func (c *restoreCmd) run(conf *config.Config, args []string) error {
 	if len(args) < 2 {
 		return errArgs
 	}
 
 	var snapname = args[1]
 
-	remote, name := config.ParseRemoteAndContainer(args[0])
-	d, err := lxd.NewClient(config, remote)
+	remote, name, err := conf.ParseRemote(args[0])
+	if err != nil {
+		return err
+	}
+
+	d, err := conf.GetContainerServer(remote)
 	if err != nil {
 		return err
 	}
@@ -52,10 +59,15 @@ func (c *restoreCmd) run(config *lxd.Config, args []string) error {
 		snapname = fmt.Sprintf("%s/%s", name, snapname)
 	}
 
-	resp, err := d.RestoreSnapshot(name, snapname, c.stateful)
+	req := api.ContainerPut{
+		Restore:  snapname,
+		Stateful: c.stateful,
+	}
+
+	op, err := d.UpdateContainer(name, req, "")
 	if err != nil {
 		return err
 	}
 
-	return d.WaitForSuccess(resp.Operation)
+	return op.Wait()
 }
