@@ -3,9 +3,9 @@ package main
 import (
 	"fmt"
 
-	"github.com/lxc/lxd"
-	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/lxc/config"
 	"github.com/lxc/lxd/shared/i18n"
+	"github.com/lxc/lxd/shared/version"
 )
 
 type versionCmd struct{}
@@ -16,18 +16,18 @@ func (c *versionCmd) showByDefault() bool {
 
 func (c *versionCmd) usage() string {
 	return i18n.G(
-		`Prints the version number of this client tool.
+		`Usage: lxc version
 
-lxc version`)
+Print the version number of this client tool.`)
 }
 
 func (c *versionCmd) flags() {
 }
 
-func (c *versionCmd) run(_ *lxd.Config, args []string) error {
+func (c *versionCmd) run(conf *config.Config, args []string) error {
 	if len(args) > 0 {
 		return errArgs
 	}
-	fmt.Println(shared.Version)
+	fmt.Println(version.Version)
 	return nil
 }
