@@ -7,7 +7,7 @@ import (
 
 	"github.com/gorilla/mux"
 
-	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/logger"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
@@ -52,14 +52,14 @@ func internalContainerOnStart(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
-	c, err := containerLoadById(d, id)
+	c, err := containerLoadById(d.State(), d.Storage, id)
 	if err != nil {
 		return SmartError(err)
 	}
 
 	err = c.OnStart()
 	if err != nil {
-		shared.Log.Error("start hook failed", log.Ctx{"container": c.Name(), "err": err})
+		logger.Error("start hook failed", log.Ctx{"container": c.Name(), "err": err})
 		return SmartError(err)
 	}
 
@@ -77,14 +77,14 @@ func internalContainerOnStop(d *Daemon, r *http.Request) Response {
 		target = "unknown"
 	}
 
-	c, err := containerLoadById(d, id)
+	c, err := containerLoadById(d.State(), d.Storage, id)
 	if err != nil {
 		return SmartError(err)
 	}
 
 	err = c.OnStop(target)
 	if err != nil {
-		shared.Log.Error("stop hook failed", log.Ctx{"container": c.Name(), "err": err})
+		logger.Error("stop hook failed", log.Ctx{"container": c.Name(), "err": err})
 		return SmartError(err)
 	}
 
