@@ -1,8 +1,8 @@
 package main
 
 import (
-	"crypto/sha256"
 	"crypto/x509"
+	"database/sql"
 	"encoding/base64"
 	"encoding/pem"
 	"fmt"
@@ -11,25 +11,26 @@ import (
 
 	"github.com/gorilla/mux"
 
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/version"
 )
 
-func certGenerateFingerprint(cert *x509.Certificate) string {
-	return fmt.Sprintf("%x", sha256.Sum256(cert.Raw))
-}
-
 func certificatesGet(d *Daemon, r *http.Request) Response {
-	recursion := d.isRecursionRequest(r)
+	recursion := util.IsRecursionRequest(r)
 
 	if recursion {
-		certResponses := []shared.CertInfo{}
+		certResponses := []api.Certificate{}
 
-		baseCerts, err := dbCertsGet(d.db)
+		baseCerts, err := db.CertsGet(d.db)
 		if err != nil {
 			return SmartError(err)
 		}
 		for _, baseCert := range baseCerts {
-			resp := shared.CertInfo{}
+			resp := api.Certificate{}
 			resp.Fingerprint = baseCert.Fingerprint
 			resp.Certificate = baseCert.Certificate
 			if baseCert.Type == 1 {
@@ -44,66 +45,60 @@ func certificatesGet(d *Daemon, r *http.Request) Response {
 
 	body := []string{}
 	for _, cert := range d.clientCerts {
-		fingerprint := fmt.Sprintf("/%s/certificates/%s", shared.APIVersion, certGenerateFingerprint(&cert))
+		fingerprint := fmt.Sprintf("/%s/certificates/%s", version.APIVersion, shared.CertFingerprint(&cert))
 		body = append(body, fingerprint)
 	}
 
 	return SyncResponse(true, body)
 }
 
-type certificatesPostBody struct {
-	Type        string `json:"type"`
-	Certificate string `json:"certificate"`
-	Name        string `json:"name"`
-	Password    string `json:"password"`
-}
-
 func readSavedClientCAList(d *Daemon) {
 	d.clientCerts = []x509.Certificate{}
 
-	dbCerts, err := dbCertsGet(d.db)
+	dbCerts, err := db.CertsGet(d.db)
 	if err != nil {
-		shared.LogInfof("Error reading certificates from database: %s", err)
+		logger.Infof("Error reading certificates from database: %s", err)
 		return
 	}
 
 	for _, dbCert := range dbCerts {
 		certBlock, _ := pem.Decode([]byte(dbCert.Certificate))
 		if certBlock == nil {
-			shared.LogInfof("Error decoding certificate for %s: %s", dbCert.Name, err)
+			logger.Infof("Error decoding certificate for %s: %s", dbCert.Name, err)
 			continue
 		}
 
 		cert, err := x509.ParseCertificate(certBlock.Bytes)
 		if err != nil {
-			shared.LogInfof("Error reading certificate for %s: %s", dbCert.Name, err)
+			logger.Infof("Error reading certificate for %s: %s", dbCert.Name, err)
 			continue
 		}
 		d.clientCerts = append(d.clientCerts, *cert)
 	}
 }
 
-func saveCert(d *Daemon, host string, cert *x509.Certificate) error {
-	baseCert := new(dbCertInfo)
-	baseCert.Fingerprint = certGenerateFingerprint(cert)
+func saveCert(dbObj *sql.DB, host string, cert *x509.Certificate) error {
+	baseCert := new(db.CertInfo)
+	baseCert.Fingerprint = shared.CertFingerprint(cert)
 	baseCert.Type = 1
 	baseCert.Name = host
 	baseCert.Certificate = string(
 		pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: cert.Raw}),
 	)
 
-	return dbCertSave(d.db, baseCert)
+	return db.CertSave(dbObj, baseCert)
 }
 
 func certificatesPost(d *Daemon, r *http.Request) Response {
 	// Parse the request
-	req := certificatesPostBody{}
+	req := api.CertificatesPost{}
 	if err := shared.ReadToJSON(r.Body, &req); err != nil {
 		return BadRequest(err)
 	}
 
 	// Access check
-	if !d.isTrustedClient(r) && d.PasswordCheck(req.Password) != nil {
+	secret := daemonConfig["core.trust_password"].Get()
+	if !util.IsTrustedClient(r, d.clientCerts) && util.PasswordCheck(secret, req.Password) != nil {
 		return Forbidden
 	}
 
@@ -141,21 +136,21 @@ func certificatesPost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("Can't use TLS data on non-TLS link"))
 	}
 
-	fingerprint := certGenerateFingerprint(cert)
+	fingerprint := shared.CertFingerprint(cert)
 	for _, existingCert := range d.clientCerts {
-		if fingerprint == certGenerateFingerprint(&existingCert) {
+		if fingerprint == shared.CertFingerprint(&existingCert) {
 			return BadRequest(fmt.Errorf("Certificate already in trust store"))
 		}
 	}
 
-	err := saveCert(d, name, cert)
+	err := saveCert(d.db, name, cert)
 	if err != nil {
 		return SmartError(err)
 	}
 
 	d.clientCerts = append(d.clientCerts, *cert)
 
-	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/certificates/%s", shared.APIVersion, fingerprint))
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/certificates/%s", version.APIVersion, fingerprint))
 }
 
 var certificatesCmd = Command{
@@ -171,7 +166,7 @@ var certificatesCmd = Command{
 func certificateFingerprintGet(d *Daemon, r *http.Request) Response {
 	fingerprint := mux.Vars(r)["fingerprint"]
 
-	cert, err := doCertificateGet(d, fingerprint)
+	cert, err := doCertificateGet(d.db, fingerprint)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -179,10 +174,10 @@ func certificateFingerprintGet(d *Daemon, r *http.Request) Response {
 	return SyncResponse(true, cert)
 }
 
-func doCertificateGet(d *Daemon, fingerprint string) (shared.CertInfo, error) {
-	resp := shared.CertInfo{}
+func doCertificateGet(dbObj *sql.DB, fingerprint string) (api.Certificate, error) {
+	resp := api.Certificate{}
 
-	dbCertInfo, err := dbCertGet(d.db, fingerprint)
+	dbCertInfo, err := db.CertGet(dbObj, fingerprint)
 	if err != nil {
 		return resp, err
 	}
@@ -202,12 +197,12 @@ func doCertificateGet(d *Daemon, fingerprint string) (shared.CertInfo, error) {
 func certificateFingerprintDelete(d *Daemon, r *http.Request) Response {
 	fingerprint := mux.Vars(r)["fingerprint"]
 
-	certInfo, err := dbCertGet(d.db, fingerprint)
+	certInfo, err := db.CertGet(d.db, fingerprint)
 	if err != nil {
 		return NotFound
 	}
 
-	err = dbCertDelete(d.db, certInfo.Fingerprint)
+	err = db.CertDelete(d.db, certInfo.Fingerprint)
 	if err != nil {
 		return SmartError(err)
 	}
