@@ -4,16 +4,36 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"os/exec"
 	"strconv"
 	"strings"
 	"time"
 
 	"gopkg.in/lxc/go-lxc.v2"
 
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/state"
+	"github.com/lxc/lxd/lxd/sys"
+	"github.com/lxc/lxd/lxd/types"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd/shared/idmap"
+	"github.com/lxc/lxd/shared/osarch"
 )
 
 // Helper functions
+
+// Returns the parent container name, snapshot name, and whether it actually was
+// a snapshot name.
+func containerGetParentAndSnapshotName(name string) (string, string, bool) {
+	fields := strings.SplitN(name, shared.SnapshotDelimiter, 2)
+	if len(fields) == 1 {
+		return name, "", false
+	}
+
+	return fields[0], fields[1], true
+}
+
 func containerPath(name string, isSnapshot bool) string {
 	if isSnapshot {
 		return shared.VarPath("snapshots", name)
@@ -168,6 +188,8 @@ func containerValidConfigKey(key string, value string) error {
 		return isBool(key, value)
 	case "security.nesting":
 		return isBool(key, value)
+	case "security.idmap.base":
+		return isUint32(key, value)
 	case "security.idmap.size":
 		return isUint32(key, value)
 	case "security.idmap.isolated":
@@ -200,6 +222,10 @@ func containerValidConfigKey(key string, value string) error {
 		if strings.HasSuffix(key, ".name") {
 			return nil
 		}
+
+		if strings.HasSuffix(key, ".host_name") {
+			return nil
+		}
 	}
 
 	if strings.HasPrefix(key, "environment.") {
@@ -210,9 +236,11 @@ func containerValidConfigKey(key string, value string) error {
 		return nil
 	}
 
-	return fmt.Errorf("Bad key: %s", key)
+	return fmt.Errorf("Unknown configuration key: %s", key)
 }
 
+var containerNetworkLimitKeys = []string{"limits.max", "limits.ingress", "limits.egress"}
+
 func containerValidDeviceConfigKey(t, k string) bool {
 	if k == "type" {
 		return true
@@ -289,7 +317,7 @@ func containerValidDeviceConfigKey(t, k string) bool {
 	}
 }
 
-func containerValidConfig(config map[string]string, profile bool, expanded bool) error {
+func containerValidConfig(os *sys.OS, config map[string]string, profile bool, expanded bool) error {
 	if config == nil {
 		return nil
 	}
@@ -305,15 +333,50 @@ func containerValidConfig(config map[string]string, profile bool, expanded bool)
 		}
 	}
 
+	if expanded && (config["security.privileged"] == "" || !shared.IsTrue(config["security.privileged"])) && os.IdmapSet == nil {
+		return fmt.Errorf("LXD doesn't have a uid/gid allocation. In this mode, only privileged containers are supported.")
+	}
+
 	return nil
 }
 
-func containerValidDevices(devices shared.Devices, profile bool, expanded bool) error {
+func isRootDiskDevice(device types.Device) bool {
+	if device["type"] == "disk" && device["path"] == "/" && device["source"] == "" {
+		return true
+	}
+
+	return false
+}
+
+func containerGetRootDiskDevice(devices types.Devices) (string, types.Device, error) {
+	var devName string
+	var dev types.Device
+
+	for n, d := range devices {
+		if isRootDiskDevice(d) {
+			if devName != "" {
+				return "", types.Device{}, fmt.Errorf("More than one root device found.")
+			}
+
+			devName = n
+			dev = d
+		}
+	}
+
+	if devName != "" {
+		return devName, dev, nil
+	}
+
+	return "", types.Device{}, fmt.Errorf("No root device could be found.")
+}
+
+func containerValidDevices(devices types.Devices, profile bool, expanded bool) error {
 	// Empty device list
 	if devices == nil {
 		return nil
 	}
 
+	var diskDevicePaths []string
 	// Check each device individually
 	for name, m := range devices {
 		if m["type"] == "" {
@@ -324,7 +387,7 @@ func containerValidDevices(devices shared.Devices, profile bool, expanded bool)
 			return fmt.Errorf("Invalid device type for device '%s'", name)
 		}
 
-		for k, _ := range m {
+		for k := range m {
 			if !containerValidDeviceConfigKey(m["type"], k) {
 				return fmt.Errorf("Invalid device configuration key for %s: %s", m["type"], k)
 			}
@@ -343,6 +406,12 @@ func containerValidDevices(devices shared.Devices, profile bool, expanded bool)
 				return fmt.Errorf("Missing parent for %s type nic.", m["nictype"])
 			}
 		} else if m["type"] == "disk" {
+			if !expanded && !shared.StringInSlice(m["path"], diskDevicePaths) {
+				diskDevicePaths = append(diskDevicePaths, m["path"])
+			} else if !expanded {
+				return fmt.Errorf("More than one disk device uses the same path: %s.", m["path"])
+			}
+
 			if m["path"] == "" {
 				return fmt.Errorf("Disk entry is missing the required \"path\" property.")
 			}
@@ -366,6 +435,25 @@ func containerValidDevices(devices shared.Devices, profile bool, expanded bool)
 			if m["path"] == "" {
 				return fmt.Errorf("Unix device entry is missing the required \"path\" property.")
 			}
+
+			if m["major"] == "" || m["minor"] == "" {
+				if !shared.PathExists(m["path"]) {
+					return fmt.Errorf("The device path doesn't exist on the host and major/minor wasn't specified.")
+				}
+
+				dType, _, _, err := deviceGetAttributes(m["path"])
+				if err != nil {
+					return err
+				}
+
+				if m["type"] == "unix-char" && dType != "c" {
+					return fmt.Errorf("Path specified for unix-char device is a block device.")
+				}
+
+				if m["type"] == "unix-block" && dType != "b" {
+					return fmt.Errorf("Path specified for unix-block device is a character device.")
+				}
+			}
 		} else if m["type"] == "none" {
 			continue
 		} else {
@@ -375,38 +463,15 @@ func containerValidDevices(devices shared.Devices, profile bool, expanded bool)
 
 	// Checks on the expanded config
 	if expanded {
-		foundRootfs := false
-		for _, m := range devices {
-			if m["type"] == "disk" && m["path"] == "/" {
-				foundRootfs = true
-			}
-		}
-
-		if !foundRootfs {
-			return fmt.Errorf("Container is lacking rootfs entry")
+		_, _, err := containerGetRootDiskDevice(devices)
+		if err != nil {
+			return err
 		}
 	}
 
 	return nil
 }
 
-// The container arguments
-type containerArgs struct {
-	// Don't set manually
-	Id int
-
-	Architecture int
-	BaseImage    string
-	Config       map[string]string
-	CreationDate time.Time
-	Ctype        containerType
-	Devices      shared.Devices
-	Ephemeral    bool
-	Name         string
-	Profiles     []string
-	Stateful     bool
-}
-
 // The container interface
 type container interface {
 	// Container actions
@@ -426,7 +491,7 @@ type container interface {
 
 	// Config handling
 	Rename(newName string) error
-	Update(newConfig containerArgs, userRequested bool) error
+	Update(newConfig db.ContainerArgs, userRequested bool) error
 
 	Delete() error
 	Export(w io.Writer, properties map[string]string) error
@@ -437,17 +502,28 @@ type container interface {
 	ConfigKeySet(key string, value string) error
 
 	// File handling
-	FilePull(srcpath string, dstpath string) (int, int, os.FileMode, error)
+	FilePull(srcpath string, dstpath string) (int64, int64, os.FileMode, error)
 	FileExists(path string) error
-	FilePush(srcpath string, dstpath string, uid int, gid int, mode int) error
+	FilePush(srcpath string, dstpath string, uid int64, gid int64, mode int) error
 	FileRemove(path string) error
 
-	// Command execution
-	Exec(command []string, env map[string]string, stdin *os.File, stdout *os.File, stderr *os.File) (int, error)
+	/* Command execution:
+		 * 1. passing in false for wait
+		 *    - equivalent to calling cmd.Run()
+		 * 2. passing in true for wait
+	         *    - start the command and return its PID in the first return
+	         *      argument and the PID of the attached process in the second
+	         *      argument. It's the callers responsibility to wait on the
+	         *      command. (Note. The returned PID of the attached process can not
+	         *      be waited upon since it's a child of the lxd forkexec command
+	         *      (the PID returned in the first return argument). It can however
+	         *      be used to e.g. forward signals.)
+	*/
+	Exec(command []string, env map[string]string, stdin *os.File, stdout *os.File, stderr *os.File, wait bool) (*exec.Cmd, int, int, error)
 
 	// Status
 	Render() (interface{}, error)
-	RenderState() (*shared.ContainerState, error)
+	RenderState() (*api.ContainerState, error)
 	IsPrivileged() bool
 	IsRunning() bool
 	IsFrozen() bool
@@ -466,9 +542,9 @@ type container interface {
 	Architecture() int
 	CreationDate() time.Time
 	ExpandedConfig() map[string]string
-	ExpandedDevices() shared.Devices
+	ExpandedDevices() types.Devices
 	LocalConfig() map[string]string
-	LocalDevices() shared.Devices
+	LocalDevices() types.Devices
 	Profiles() []string
 	InitPID() int
 	State() string
@@ -485,23 +561,24 @@ type container interface {
 	StorageStart() error
 	StorageStop() error
 	Storage() storage
-	IdmapSet() *shared.IdmapSet
-	LastIdmapSet() (*shared.IdmapSet, error)
+	IdmapSet() (*idmap.IdmapSet, error)
+	LastIdmapSet() (*idmap.IdmapSet, error)
 	TemplateApply(trigger string) error
-	Daemon() *Daemon
+	StateObject() *state.State
 }
 
 // Loader functions
-func containerCreateAsEmpty(d *Daemon, args containerArgs) (container, error) {
+func containerCreateAsEmpty(d *Daemon, args db.ContainerArgs) (container, error) {
 	// Create the container
-	c, err := containerCreateInternal(d, args)
+	c, err := containerCreateInternal(d.State(), d.Storage, args)
 	if err != nil {
 		return nil, err
 	}
 
 	// Now create the empty storage
-	if err := c.Storage().ContainerCreate(c); err != nil {
-		c.Delete()
+	err = c.Storage().ContainerCreate(c)
+	if err != nil {
+		db.ContainerRemove(d.db, args.Name)
 		return nil, err
 	}
 
@@ -515,36 +592,43 @@ func containerCreateAsEmpty(d *Daemon, args containerArgs) (container, error) {
 	return c, nil
 }
 
-func containerCreateEmptySnapshot(d *Daemon, args containerArgs) (container, error) {
+func containerCreateEmptySnapshot(s *state.State, storage storage, args db.ContainerArgs) (container, error) {
 	// Create the snapshot
-	c, err := containerCreateInternal(d, args)
+	c, err := containerCreateInternal(s, storage, args)
 	if err != nil {
 		return nil, err
 	}
 
 	// Now create the empty snapshot
-	if err := c.Storage().ContainerSnapshotCreateEmpty(c); err != nil {
-		c.Delete()
+	err = c.Storage().ContainerSnapshotCreateEmpty(c)
+	if err != nil {
+		db.ContainerRemove(s.DB, args.Name)
 		return nil, err
 	}
 
 	return c, nil
 }
 
-func containerCreateFromImage(d *Daemon, args containerArgs, hash string) (container, error) {
+func containerCreateFromImage(s *state.State, storage storage, args db.ContainerArgs, hash string) (container, error) {
+	// Set the BaseImage field (regardless of previous value)
+	args.BaseImage = hash
+
 	// Create the container
-	c, err := containerCreateInternal(d, args)
+	c, err := containerCreateInternal(s, storage, args)
 	if err != nil {
 		return nil, err
 	}
 
-	if err := dbImageLastAccessUpdate(d.db, hash, time.Now().UTC()); err != nil {
+	err = db.ImageLastAccessUpdate(s.DB, hash, time.Now().UTC())
+	if err != nil {
+		db.ContainerRemove(s.DB, args.Name)
 		return nil, fmt.Errorf("Error updating image last use date: %s", err)
 	}
 
 	// Now create the storage from an image
-	if err := c.Storage().ContainerCreateFromImage(c, hash); err != nil {
-		c.Delete()
+	err = c.Storage().ContainerCreateFromImage(c, hash)
+	if err != nil {
+		db.ContainerRemove(s.DB, args.Name)
 		return nil, err
 	}
 
@@ -558,9 +642,9 @@ func containerCreateFromImage(d *Daemon, args containerArgs, hash string) (conta
 	return c, nil
 }
 
-func containerCreateAsCopy(d *Daemon, args containerArgs, sourceContainer container) (container, error) {
-	// Create the container
-	c, err := containerCreateInternal(d, args)
+func containerCreateAsCopy(s *state.State, storage storage, args db.ContainerArgs, sourceContainer container) (container, error) {
+	// Create the container.
+	c, err := containerCreateInternal(s, storage, args)
 	if err != nil {
 		return nil, err
 	}
@@ -581,19 +665,20 @@ func containerCreateAsCopy(d *Daemon, args containerArgs, sourceContainer contai
 	return c, nil
 }
 
-func containerCreateAsSnapshot(d *Daemon, args containerArgs, sourceContainer container) (container, error) {
+func containerCreateAsSnapshot(s *state.State, storage storage, args db.ContainerArgs, sourceContainer container) (container, error) {
 	// Deal with state
 	if args.Stateful {
 		if !sourceContainer.IsRunning() {
-			return nil, fmt.Errorf("Container not running, cannot do stateful snapshot")
+			return nil, fmt.Errorf("Unable to create a stateful snapshot. The container isn't running.")
 		}
 
-		if err := findCriu("snapshot"); err != nil {
-			return nil, err
+		_, err := exec.LookPath("criu")
+		if err != nil {
+			return nil, fmt.Errorf("Unable to create a stateful snapshot. CRIU isn't installed.")
 		}
 
 		stateDir := sourceContainer.StatePath()
-		err := os.MkdirAll(stateDir, 0700)
+		err = os.MkdirAll(stateDir, 0700)
 		if err != nil {
 			return nil, err
 		}
@@ -616,14 +701,15 @@ func containerCreateAsSnapshot(d *Daemon, args containerArgs, sourceContainer co
 	}
 
 	// Create the snapshot
-	c, err := containerCreateInternal(d, args)
+	c, err := containerCreateInternal(s, storage, args)
 	if err != nil {
 		return nil, err
 	}
 
 	// Clone the container
-	if err := sourceContainer.Storage().ContainerSnapshotCreate(c, sourceContainer); err != nil {
-		c.Delete()
+	err = sourceContainer.Storage().ContainerSnapshotCreate(c, sourceContainer)
+	if err != nil {
+		db.ContainerRemove(s.DB, args.Name)
 		return nil, err
 	}
 
@@ -635,7 +721,7 @@ func containerCreateAsSnapshot(d *Daemon, args containerArgs, sourceContainer co
 	return c, nil
 }
 
-func containerCreateInternal(d *Daemon, args containerArgs) (container, error) {
+func containerCreateInternal(s *state.State, storage storage, args db.ContainerArgs) (container, error) {
 	// Set default values
 	if args.Profiles == nil {
 		args.Profiles = []string{"default"}
@@ -650,15 +736,15 @@ func containerCreateInternal(d *Daemon, args containerArgs) (container, error) {
 	}
 
 	if args.Devices == nil {
-		args.Devices = shared.Devices{}
+		args.Devices = types.Devices{}
 	}
 
 	if args.Architecture == 0 {
-		args.Architecture = d.architectures[0]
+		args.Architecture = s.OS.Architectures[0]
 	}
 
 	// Validate container name
-	if args.Ctype == cTypeRegular {
+	if args.Ctype == db.CTypeRegular {
 		err := containerValidName(args.Name)
 		if err != nil {
 			return nil, err
@@ -666,7 +752,7 @@ func containerCreateInternal(d *Daemon, args containerArgs) (container, error) {
 	}
 
 	// Validate container config
-	err := containerValidConfig(args.Config, false, false)
+	err := containerValidConfig(s.OS, args.Config, false, false)
 	if err != nil {
 		return nil, err
 	}
@@ -678,13 +764,17 @@ func containerCreateInternal(d *Daemon, args containerArgs) (container, error) {
 	}
 
 	// Validate architecture
-	_, err = shared.ArchitectureName(args.Architecture)
+	_, err = osarch.ArchitectureName(args.Architecture)
 	if err != nil {
 		return nil, err
 	}
 
+	if !shared.IntInSlice(args.Architecture, s.OS.Architectures) {
+		return nil, fmt.Errorf("Requested architecture isn't supported by this host")
+	}
+
 	// Validate profiles
-	profiles, err := dbProfiles(d.db)
+	profiles, err := db.Profiles(s.DB)
 	if err != nil {
 		return nil, err
 	}
@@ -695,73 +785,81 @@ func containerCreateInternal(d *Daemon, args containerArgs) (container, error) {
 		}
 	}
 
-	path := containerPath(args.Name, args.Ctype == cTypeSnapshot)
-	if shared.PathExists(path) {
-		if shared.IsSnapshot(args.Name) {
-			return nil, fmt.Errorf("Snapshot '%s' already exists", args.Name)
+	// Create the container entry
+	id, err := db.ContainerCreate(s.DB, args)
+	if err != nil {
+		if err == db.DbErrAlreadyDefined {
+			thing := "Container"
+			if shared.IsSnapshot(args.Name) {
+				thing = "Snapshot"
+			}
+			return nil, fmt.Errorf("%s '%s' already exists", thing, args.Name)
 		}
-		return nil, fmt.Errorf("The container already exists")
+		return nil, err
 	}
 
 	// Wipe any existing log for this container name
 	os.RemoveAll(shared.LogPath(args.Name))
 
-	// Create the container entry
-	id, err := dbContainerCreate(d.db, args)
-	if err != nil {
-		return nil, err
-	}
 	args.Id = id
 
 	// Read the timestamp from the database
-	dbArgs, err := dbContainerGet(d.db, args.Name)
+	dbArgs, err := db.ContainerGet(s.DB, args.Name)
 	if err != nil {
+		db.ContainerRemove(s.DB, args.Name)
 		return nil, err
 	}
 	args.CreationDate = dbArgs.CreationDate
 
-	return containerLXCCreate(d, args)
+	// Setup the container struct and finish creation (storage and idmap)
+	c, err := containerLXCCreate(s, storage, args)
+	if err != nil {
+		db.ContainerRemove(s.DB, args.Name)
+		return nil, err
+	}
+
+	return c, nil
 }
 
 func containerConfigureInternal(c container) error {
 	// Find the root device
-	for _, m := range c.ExpandedDevices() {
-		if m["type"] != "disk" || m["path"] != "/" || m["size"] == "" {
-			continue
-		}
+	_, rootDiskDevice, err := containerGetRootDiskDevice(c.ExpandedDevices())
+	if err != nil {
+		return err
+	}
 
-		size, err := shared.ParseByteSizeString(m["size"])
+	if rootDiskDevice["size"] != "" {
+		size, err := shared.ParseByteSizeString(rootDiskDevice["size"])
 		if err != nil {
 			return err
 		}
 
+		// Storage is guaranteed to be ready.
 		err = c.Storage().ContainerSetQuota(c, size)
 		if err != nil {
 			return err
 		}
-
-		break
 	}
 
 	return nil
 }
 
-func containerLoadById(d *Daemon, id int) (container, error) {
+func containerLoadById(s *state.State, storage storage, id int) (container, error) {
 	// Get the DB record
-	name, err := dbContainerName(d.db, id)
+	name, err := db.ContainerName(s.DB, id)
 	if err != nil {
 		return nil, err
 	}
 
-	return containerLoadByName(d, name)
+	return containerLoadByName(s, storage, name)
 }
 
-func containerLoadByName(d *Daemon, name string) (container, error) {
+func containerLoadByName(s *state.State, storage storage, name string) (container, error) {
 	// Get the DB record
-	args, err := dbContainerGet(d.db, name)
+	args, err := db.ContainerGet(s.DB, name)
 	if err != nil {
 		return nil, err
 	}
 
-	return containerLXCLoad(d, args)
+	return containerLXCLoad(s, storage, args)
 }
