@@ -6,34 +6,29 @@ import (
 	"io/ioutil"
 	"net/http"
 	"os"
+	"os/exec"
 	"strconv"
 	"strings"
 	"sync"
+	"syscall"
 
 	"github.com/gorilla/mux"
 	"github.com/gorilla/websocket"
 
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd/shared/logger"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
-type commandPostContent struct {
-	Command     []string          `json:"command"`
-	WaitForWS   bool              `json:"wait-for-websocket"`
-	Interactive bool              `json:"interactive"`
-	Environment map[string]string `json:"environment"`
-	Width       int               `json:"width"`
-	Height      int               `json:"height"`
-}
-
 type execWs struct {
 	command   []string
 	container container
 	env       map[string]string
 
-	rootUid          int
-	rootGid          int
+	rootUid          int64
+	rootGid          int64
 	conns            map[int]*websocket.Conn
 	connsLock        sync.Mutex
 	allConnected     chan bool
@@ -137,11 +132,14 @@ func (s *execWs) Do(op *operation) error {
 	}
 
 	controlExit := make(chan bool)
+	attachedChildIsBorn := make(chan int)
+	attachedChildIsDead := make(chan bool, 1)
 	var wgEOF sync.WaitGroup
 
 	if s.interactive {
 		wgEOF.Add(1)
 		go func() {
+			attachedChildPid := <-attachedChildIsBorn
 			select {
 			case <-s.controlConnected:
 				break
@@ -151,66 +149,104 @@ func (s *execWs) Do(op *operation) error {
 			}
 
 			for {
-				mt, r, err := s.conns[-1].NextReader()
+				s.connsLock.Lock()
+				conn := s.conns[-1]
+				s.connsLock.Unlock()
+
+				mt, r, err := conn.NextReader()
 				if mt == websocket.CloseMessage {
 					break
 				}
 
 				if err != nil {
-					shared.LogDebugf("Got error getting next reader %s", err)
-					break
+					logger.Debugf("Got error getting next reader %s", err)
+					er, ok := err.(*websocket.CloseError)
+					if !ok {
+						break
+					}
+
+					if er.Code != websocket.CloseAbnormalClosure {
+						break
+					}
+
+					// If an abnormal closure occurred, kill the attached process.
+					err := syscall.Kill(attachedChildPid, syscall.SIGKILL)
+					if err != nil {
+						logger.Debugf("Failed to send SIGKILL to pid %d.", attachedChildPid)
+					} else {
+						logger.Debugf("Sent SIGKILL to pid %d.", attachedChildPid)
+					}
+					return
 				}
 
 				buf, err := ioutil.ReadAll(r)
 				if err != nil {
-					shared.LogDebugf("Failed to read message %s", err)
+					logger.Debugf("Failed to read message %s", err)
 					break
 				}
 
-				command := shared.ContainerExecControl{}
+				command := api.ContainerExecControl{}
 
 				if err := json.Unmarshal(buf, &command); err != nil {
-					shared.LogDebugf("Failed to unmarshal control socket command: %s", err)
+					logger.Debugf("Failed to unmarshal control socket command: %s", err)
 					continue
 				}
 
 				if command.Command == "window-resize" {
 					winchWidth, err := strconv.Atoi(command.Args["width"])
 					if err != nil {
-						shared.LogDebugf("Unable to extract window width: %s", err)
+						logger.Debugf("Unable to extract window width: %s", err)
 						continue
 					}
 
 					winchHeight, err := strconv.Atoi(command.Args["height"])
 					if err != nil {
-						shared.LogDebugf("Unable to extract window height: %s", err)
+						logger.Debugf("Unable to extract window height: %s", err)
 						continue
 					}
 
 					err = shared.SetSize(int(ptys[0].Fd()), winchWidth, winchHeight)
 					if err != nil {
-						shared.LogDebugf("Failed to set window size to: %dx%d", winchWidth, winchHeight)
+						logger.Debugf("Failed to set window size to: %dx%d", winchWidth, winchHeight)
 						continue
 					}
 				}
 			}
 		}()
+
 		go func() {
-			readDone, writeDone := shared.WebsocketMirror(s.conns[0], ptys[0], ptys[0])
+			s.connsLock.Lock()
+			conn := s.conns[0]
+			s.connsLock.Unlock()
+
+			logger.Debugf("Starting to mirror websocket")
+			readDone, writeDone := shared.WebsocketExecMirror(conn, ptys[0], ptys[0], attachedChildIsDead, int(ptys[0].Fd()))
+
 			<-readDone
 			<-writeDone
-			s.conns[0].Close()
+			logger.Debugf("Finished to mirror websocket")
+
+			conn.Close()
 			wgEOF.Done()
 		}()
+
 	} else {
 		wgEOF.Add(len(ttys) - 1)
 		for i := 0; i < len(ttys); i++ {
 			go func(i int) {
 				if i == 0 {
-					<-shared.WebsocketRecvStream(ttys[i], s.conns[i])
+					s.connsLock.Lock()
+					conn := s.conns[i]
+					s.connsLock.Unlock()
+
+					<-shared.WebsocketRecvStream(ttys[i], conn)
 					ttys[i].Close()
 				} else {
-					<-shared.WebsocketSendStream(s.conns[i], ptys[i], -1)
+					s.connsLock.Lock()
+					conn := s.conns[i]
+					s.connsLock.Unlock()
+
+					<-shared.WebsocketSendStream(conn, ptys[i], -1)
 					ptys[i].Close()
 					wgEOF.Done()
 				}
@@ -218,38 +254,73 @@ func (s *execWs) Do(op *operation) error {
 		}
 	}
 
-	cmdResult, cmdErr := s.container.Exec(s.command, s.env, stdin, stdout, stderr)
-
-	for _, tty := range ttys {
-		tty.Close()
-	}
-
-	if s.conns[-1] == nil {
-		if s.interactive {
-			controlExit <- true
+	finisher := func(cmdResult int, cmdErr error) error {
+		for _, tty := range ttys {
+			tty.Close()
 		}
-	} else {
-		s.conns[-1].Close()
+
+		s.connsLock.Lock()
+		conn := s.conns[-1]
+		s.connsLock.Unlock()
+
+		if conn == nil {
+			if s.interactive {
+				controlExit <- true
+			}
+		} else {
+			conn.Close()
+		}
+
+		attachedChildIsDead <- true
+
+		wgEOF.Wait()
+
+		for _, pty := range ptys {
+			pty.Close()
+		}
+
+		metadata := shared.Jmap{"return": cmdResult}
+		err = op.UpdateMetadata(metadata)
+		if err != nil {
+			return err
+		}
+
+		return cmdErr
 	}
 
-	wgEOF.Wait()
-
-	for _, pty := range ptys {
-		pty.Close()
-	}
-
-	metadata := shared.Jmap{"return": cmdResult}
-	err = op.UpdateMetadata(metadata)
+	cmd, _, attachedPid, err := s.container.Exec(s.command, s.env, stdin, stdout, stderr, false)
 	if err != nil {
 		return err
 	}
 
-	return cmdErr
+	if s.interactive {
+		attachedChildIsBorn <- attachedPid
+	}
+
+	err = cmd.Wait()
+	if err == nil {
+		return finisher(0, nil)
+	}
+
+	exitErr, ok := err.(*exec.ExitError)
+	if ok {
+		status, ok := exitErr.Sys().(syscall.WaitStatus)
+		if ok {
+			return finisher(status.ExitStatus(), nil)
+		}
+
+		if status.Signaled() {
+			// COMMENT(brauner): 128 + n == Fatal error signal "n"
+			return finisher(128+int(status.Signal()), nil)
+		}
+	}
+
+	return finisher(-1, nil)
 }
 
 func containerExecPost(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
-	c, err := containerLoadByName(d, name)
+	c, err := containerLoadByName(d.State(), d.Storage, name)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -262,7 +333,7 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("Container is frozen."))
 	}
 
-	post := commandPostContent{}
+	post := api.ContainerExecPost{}
 	buf, err := ioutil.ReadAll(r.Body)
 	if err != nil {
 		return BadRequest(err)
@@ -286,6 +357,7 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 		}
 	}
 
+	// Set default value for PATH
 	_, ok := env["PATH"]
 	if !ok {
 		env["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
@@ -294,13 +366,37 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 		}
 	}
 
+	// Set default value for HOME
+	_, ok = env["HOME"]
+	if !ok {
+		env["HOME"] = "/root"
+	}
+
+	// Set default value for USER
+	_, ok = env["USER"]
+	if !ok {
+		env["USER"] = "root"
+	}
+
+	// Set default value for USER
+	_, ok = env["LANG"]
+	if !ok {
+		env["LANG"] = "C.UTF-8"
+	}
+
 	if post.WaitForWS {
 		ws := &execWs{}
 		ws.fds = map[int]string{}
-		idmapset := c.IdmapSet()
+
+		idmapset, err := c.IdmapSet()
+		if err != nil {
+			return InternalError(err)
+		}
+
 		if idmapset != nil {
 			ws.rootUid, ws.rootGid = idmapset.ShiftIntoNs(0, 0)
 		}
+
 		ws.conns = map[int]*websocket.Conn{}
 		ws.conns[-1] = nil
 		ws.conns[0] = nil
@@ -337,11 +433,12 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 	}
 
 	run := func(op *operation) error {
-		cmdResult, cmdErr := c.Exec(post.Command, env, nil, nil, nil)
+		_, cmdResult, _, cmdErr := c.Exec(post.Command, env, nil, nil, nil, true)
 		metadata := shared.Jmap{"return": cmdResult}
+
 		err = op.UpdateMetadata(metadata)
 		if err != nil {
-			shared.LogError("error updating metadata for cmd", log.Ctx{"err": err, "cmd": post.Command})
+			logger.Error("error updating metadata for cmd", log.Ctx{"err": err, "cmd": post.Command})
 		}
 		return cmdErr
 	}
