@@ -6,16 +6,14 @@ import (
 	"net/http"
 
 	"github.com/gorilla/mux"
-)
 
-type containerPostBody struct {
-	Migration bool   `json:"migration"`
-	Name      string `json:"name"`
-}
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/shared/api"
+)
 
 func containerPost(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
-	c, err := containerLoadByName(d, name)
+	c, err := containerLoadByName(d.State(), d.Storage, name)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -25,8 +23,9 @@ func containerPost(d *Daemon, r *http.Request) Response {
 		return InternalError(err)
 	}
 
-	body := containerPostBody{}
-	if err := json.Unmarshal(buf, &body); err != nil {
+	body := api.ContainerPost{}
+	err = json.Unmarshal(buf, &body)
+	if err != nil {
 		return BadRequest(err)
 	}
 
@@ -47,6 +46,12 @@ func containerPost(d *Daemon, r *http.Request) Response {
 		return OperationResponse(op)
 	}
 
+	// Check that the name isn't already in use
+	id, _ := db.ContainerId(d.db, body.Name)
+	if id > 0 {
+		return Conflict
+	}
+
 	run := func(*operation) error {
 		return c.Rename(body.Name)
 	}
