@@ -7,37 +7,34 @@ import (
 	"net/http"
 
 	"github.com/gorilla/mux"
+
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/osarch"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
-type containerPutReq struct {
-	Architecture string            `json:"architecture"`
-	Config       map[string]string `json:"config"`
-	Devices      shared.Devices    `json:"devices"`
-	Ephemeral    bool              `json:"ephemeral"`
-	Profiles     []string          `json:"profiles"`
-	Restore      string            `json:"restore"`
-}
-
 /*
  * Update configuration, or, if 'restore:snapshot-name' is present, restore
  * the named snapshot
  */
 func containerPut(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
-	c, err := containerLoadByName(d, name)
+	c, err := containerLoadByName(d.State(), d.Storage, name)
 	if err != nil {
 		return NotFound
 	}
 
-	configRaw := containerPutReq{}
+	configRaw := api.ContainerPut{}
 	if err := json.NewDecoder(r.Body).Decode(&configRaw); err != nil {
 		return BadRequest(err)
 	}
 
-	architecture, err := shared.ArchitectureId(configRaw.Architecture)
+	architecture, err := osarch.ArchitectureId(configRaw.Architecture)
 	if err != nil {
 		architecture = 0
 	}
@@ -47,7 +44,7 @@ func containerPut(d *Daemon, r *http.Request) Response {
 	if configRaw.Restore == "" {
 		// Update container configuration
 		do = func(op *operation) error {
-			args := containerArgs{
+			args := db.ContainerArgs{
 				Architecture: architecture,
 				Config:       configRaw.Config,
 				Devices:      configRaw.Devices,
@@ -65,7 +62,7 @@ func containerPut(d *Daemon, r *http.Request) Response {
 	} else {
 		// Snapshot Restore
 		do = func(op *operation) error {
-			return containerSnapRestore(d, name, configRaw.Restore)
+			return containerSnapRestore(d.State(), d.Storage, name, configRaw.Restore)
 		}
 	}
 
@@ -80,21 +77,21 @@ func containerPut(d *Daemon, r *http.Request) Response {
 	return OperationResponse(op)
 }
 
-func containerSnapRestore(d *Daemon, name string, snap string) error {
+func containerSnapRestore(s *state.State, storage storage, name string, snap string) error {
 	// normalize snapshot name
 	if !shared.IsSnapshot(snap) {
 		snap = name + shared.SnapshotDelimiter + snap
 	}
 
-	shared.LogInfo(
+	logger.Info(
 		"RESTORE => Restoring snapshot",
 		log.Ctx{
 			"snapshot":  snap,
 			"container": name})
 
-	c, err := containerLoadByName(d, name)
+	c, err := containerLoadByName(s, storage, name)
 	if err != nil {
-		shared.LogError(
+		logger.Error(
 			"RESTORE => loadcontainerLXD() failed",
 			log.Ctx{
 				"container": name,
@@ -102,7 +99,7 @@ func containerSnapRestore(d *Daemon, name string, snap string) error {
 		return err
 	}
 
-	source, err := containerLoadByName(d, snap)
+	source, err := containerLoadByName(s, storage, snap)
 	if err != nil {
 		switch err {
 		case sql.ErrNoRows:
