@@ -8,19 +8,14 @@ import (
 
 	"github.com/gorilla/mux"
 
+	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 )
 
-type containerStatePutReq struct {
-	Action   string `json:"action"`
-	Timeout  int    `json:"timeout"`
-	Force    bool   `json:"force"`
-	Stateful bool   `json:"stateful"`
-}
-
 func containerState(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
-	c, err := containerLoadByName(d, name)
+	c, err := containerLoadByName(d.State(), d.Storage, name)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -36,7 +31,7 @@ func containerState(d *Daemon, r *http.Request) Response {
 func containerStatePut(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
 
-	raw := containerStatePutReq{}
+	raw := api.ContainerStatePut{}
 
 	// We default to -1 (i.e. no timeout) here instead of 0 (instant
 	// timeout).
@@ -49,7 +44,7 @@ func containerStatePut(d *Daemon, r *http.Request) Response {
 	// Don't mess with containers while in setup mode
 	<-d.readyChan
 
-	c, err := containerLoadByName(d, name)
+	c, err := containerLoadByName(d.State(), d.Storage, name)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -105,7 +100,7 @@ func containerStatePut(d *Daemon, r *http.Request) Response {
 
 			if ephemeral {
 				// Unset ephemeral flag
-				args := containerArgs{
+				args := db.ContainerArgs{
 					Architecture: c.Architecture(),
 					Config:       c.LocalConfig(),
 					Devices:      c.LocalDevices(),
