@@ -2,18 +2,28 @@ package main
 
 import (
 	"fmt"
+	"testing"
 
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/types"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd/shared/idmap"
+	"github.com/stretchr/testify/suite"
 )
 
-func (suite *lxdTestSuite) TestContainer_ProfilesDefault() {
-	args := containerArgs{
-		Ctype:     cTypeRegular,
+type containerTestSuite struct {
+	lxdTestSuite
+}
+
+func (suite *containerTestSuite) TestContainer_ProfilesDefault() {
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeRegular,
 		Ephemeral: false,
 		Name:      "testFoo",
 	}
 
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 	defer c.Delete()
 
@@ -29,28 +39,28 @@ func (suite *lxdTestSuite) TestContainer_ProfilesDefault() {
 		"First profile should be the default profile.")
 }
 
-func (suite *lxdTestSuite) TestContainer_ProfilesMulti() {
+func (suite *containerTestSuite) TestContainer_ProfilesMulti() {
 	// Create an unprivileged profile
-	_, err := dbProfileCreate(
+	_, err := db.ProfileCreate(
 		suite.d.db,
 		"unprivileged",
 		"unprivileged",
 		map[string]string{"security.privileged": "true"},
-		shared.Devices{})
+		types.Devices{})
 
 	suite.Req.Nil(err, "Failed to create the unprivileged profile.")
 	defer func() {
-		dbProfileDelete(suite.d.db, "unprivileged")
+		db.ProfileDelete(suite.d.db, "unprivileged")
 	}()
 
-	args := containerArgs{
-		Ctype:     cTypeRegular,
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeRegular,
 		Ephemeral: false,
 		Profiles:  []string{"default", "unprivileged"},
 		Name:      "testFoo",
 	}
 
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 	defer c.Delete()
 
@@ -65,20 +75,20 @@ func (suite *lxdTestSuite) TestContainer_ProfilesMulti() {
 		"The container is not privileged (didn't apply the unprivileged profile?).")
 }
 
-func (suite *lxdTestSuite) TestContainer_ProfilesOverwriteDefaultNic() {
-	args := containerArgs{
-		Ctype:     cTypeRegular,
+func (suite *containerTestSuite) TestContainer_ProfilesOverwriteDefaultNic() {
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeRegular,
 		Ephemeral: false,
 		Config:    map[string]string{"security.privileged": "true"},
-		Devices: shared.Devices{
-			"eth0": shared.Device{
+		Devices: types.Devices{
+			"eth0": types.Device{
 				"type":    "nic",
 				"nictype": "bridged",
 				"parent":  "unknownbr0"}},
 		Name: "testFoo",
 	}
 
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 
 	suite.True(c.IsPrivileged(), "This container should be privileged.")
@@ -86,7 +96,7 @@ func (suite *lxdTestSuite) TestContainer_ProfilesOverwriteDefaultNic() {
 	out, err := c.Render()
 	suite.Req.Nil(err)
 
-	state := out.(*shared.ContainerInfo)
+	state := out.(*api.Container)
 	defer c.Delete()
 
 	suite.Equal(
@@ -95,13 +105,13 @@ func (suite *lxdTestSuite) TestContainer_ProfilesOverwriteDefaultNic() {
 		"Container config doesn't overwrite profile config.")
 }
 
-func (suite *lxdTestSuite) TestContainer_LoadFromDB() {
-	args := containerArgs{
-		Ctype:     cTypeRegular,
+func (suite *containerTestSuite) TestContainer_LoadFromDB() {
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeRegular,
 		Ephemeral: false,
 		Config:    map[string]string{"security.privileged": "true"},
-		Devices: shared.Devices{
-			"eth0": shared.Device{
+		Devices: types.Devices{
+			"eth0": types.Device{
 				"type":    "nic",
 				"nictype": "bridged",
 				"parent":  "unknownbr0"}},
@@ -109,12 +119,12 @@ func (suite *lxdTestSuite) TestContainer_LoadFromDB() {
 	}
 
 	// Create the container
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 	defer c.Delete()
 
 	// Load the container and trigger initLXC()
-	c2, err := containerLoadByName(suite.d, "testFoo")
+	c2, err := containerLoadByName(suite.d.State(), suite.d.Storage, "testFoo")
 	c2.IsRunning()
 	suite.Req.Nil(err)
 
@@ -124,15 +134,15 @@ func (suite *lxdTestSuite) TestContainer_LoadFromDB() {
 		"The loaded container isn't excactly the same as the created one.")
 }
 
-func (suite *lxdTestSuite) TestContainer_Path_Regular() {
+func (suite *containerTestSuite) TestContainer_Path_Regular() {
 	// Regular
-	args := containerArgs{
-		Ctype:     cTypeRegular,
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeRegular,
 		Ephemeral: false,
 		Name:      "testFoo",
 	}
 
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 	defer c.Delete()
 
@@ -141,15 +151,15 @@ func (suite *lxdTestSuite) TestContainer_Path_Regular() {
 	suite.Req.Equal(shared.VarPath("containers", "testFoo2"), containerPath("testFoo2", false))
 }
 
-func (suite *lxdTestSuite) TestContainer_Path_Snapshot() {
+func (suite *containerTestSuite) TestContainer_Path_Snapshot() {
 	// Snapshot
-	args := containerArgs{
-		Ctype:     cTypeSnapshot,
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeSnapshot,
 		Ephemeral: false,
 		Name:      "test/snap0",
 	}
 
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 	defer c.Delete()
 
@@ -162,29 +172,29 @@ func (suite *lxdTestSuite) TestContainer_Path_Snapshot() {
 		containerPath("test/snap1", true))
 }
 
-func (suite *lxdTestSuite) TestContainer_LogPath() {
-	args := containerArgs{
-		Ctype:     cTypeRegular,
+func (suite *containerTestSuite) TestContainer_LogPath() {
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeRegular,
 		Ephemeral: false,
 		Name:      "testFoo",
 	}
 
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 	defer c.Delete()
 
 	suite.Req.Equal(shared.VarPath("logs", "testFoo"), c.LogPath())
 }
 
-func (suite *lxdTestSuite) TestContainer_IsPrivileged_Privileged() {
-	args := containerArgs{
-		Ctype:     cTypeRegular,
+func (suite *containerTestSuite) TestContainer_IsPrivileged_Privileged() {
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeRegular,
 		Ephemeral: false,
 		Config:    map[string]string{"security.privileged": "true"},
 		Name:      "testFoo",
 	}
 
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 	defer c.Delete()
 
@@ -192,15 +202,15 @@ func (suite *lxdTestSuite) TestContainer_IsPrivileged_Privileged() {
 	suite.Req.Nil(c.Delete(), "Failed to delete the container.")
 }
 
-func (suite *lxdTestSuite) TestContainer_IsPrivileged_Unprivileged() {
-	args := containerArgs{
-		Ctype:     cTypeRegular,
+func (suite *containerTestSuite) TestContainer_IsPrivileged_Unprivileged() {
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeRegular,
 		Ephemeral: false,
 		Config:    map[string]string{"security.privileged": "false"},
 		Name:      "testFoo",
 	}
 
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 	defer c.Delete()
 
@@ -208,14 +218,14 @@ func (suite *lxdTestSuite) TestContainer_IsPrivileged_Unprivileged() {
 	suite.Req.Nil(c.Delete(), "Failed to delete the container.")
 }
 
-func (suite *lxdTestSuite) TestContainer_Rename() {
-	args := containerArgs{
-		Ctype:     cTypeRegular,
+func (suite *containerTestSuite) TestContainer_Rename() {
+	args := db.ContainerArgs{
+		Ctype:     db.CTypeRegular,
 		Ephemeral: false,
 		Name:      "testFoo",
 	}
 
-	c, err := containerCreateInternal(suite.d, args)
+	c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, args)
 	suite.Req.Nil(err)
 	defer c.Delete()
 
@@ -223,9 +233,9 @@ func (suite *lxdTestSuite) TestContainer_Rename() {
 	suite.Req.Equal(shared.VarPath("containers", "testFoo2"), c.Path())
 }
 
-func (suite *lxdTestSuite) TestContainer_findIdmap_isolated() {
-	c1, err := containerCreateInternal(suite.d, containerArgs{
-		Ctype: cTypeRegular,
+func (suite *containerTestSuite) TestContainer_findIdmap_isolated() {
+	c1, err := containerCreateInternal(suite.d.State(), suite.d.Storage, db.ContainerArgs{
+		Ctype: db.CTypeRegular,
 		Name:  "isol-1",
 		Config: map[string]string{
 			"security.idmap.isolated": "true",
@@ -234,8 +244,8 @@ func (suite *lxdTestSuite) TestContainer_findIdmap_isolated() {
 	suite.Req.Nil(err)
 	defer c1.Delete()
 
-	c2, err := containerCreateInternal(suite.d, containerArgs{
-		Ctype: cTypeRegular,
+	c2, err := containerCreateInternal(suite.d.State(), suite.d.Storage, db.ContainerArgs{
+		Ctype: db.CTypeRegular,
 		Name:  "isol-2",
 		Config: map[string]string{
 			"security.idmap.isolated": "true",
@@ -249,24 +259,24 @@ func (suite *lxdTestSuite) TestContainer_findIdmap_isolated() {
 	map2, err := c2.(*containerLXC).NextIdmapSet()
 	suite.Req.Nil(err)
 
-	host := suite.d.IdmapSet.Idmap[0]
+	host := suite.d.os.IdmapSet.Idmap[0]
 
 	for i := 0; i < 2; i++ {
 		suite.Req.Equal(host.Hostid+65536, map1.Idmap[i].Hostid, "hostids don't match %d", i)
-		suite.Req.Equal(0, map1.Idmap[i].Nsid, "nsid nonzero")
-		suite.Req.Equal(65536, map1.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(0), map1.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(65536), map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
 	for i := 0; i < 2; i++ {
 		suite.Req.Equal(host.Hostid+65536*2, map2.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(0, map2.Idmap[i].Nsid, "nsid nonzero")
-		suite.Req.Equal(65536, map2.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(0), map2.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(65536), map2.Idmap[i].Maprange, "incorrect maprange")
 	}
 }
 
-func (suite *lxdTestSuite) TestContainer_findIdmap_mixed() {
-	c1, err := containerCreateInternal(suite.d, containerArgs{
-		Ctype: cTypeRegular,
+func (suite *containerTestSuite) TestContainer_findIdmap_mixed() {
+	c1, err := containerCreateInternal(suite.d.State(), suite.d.Storage, db.ContainerArgs{
+		Ctype: db.CTypeRegular,
 		Name:  "isol-1",
 		Config: map[string]string{
 			"security.idmap.isolated": "false",
@@ -275,8 +285,8 @@ func (suite *lxdTestSuite) TestContainer_findIdmap_mixed() {
 	suite.Req.Nil(err)
 	defer c1.Delete()
 
-	c2, err := containerCreateInternal(suite.d, containerArgs{
-		Ctype: cTypeRegular,
+	c2, err := containerCreateInternal(suite.d.State(), suite.d.Storage, db.ContainerArgs{
+		Ctype: db.CTypeRegular,
 		Name:  "isol-2",
 		Config: map[string]string{
 			"security.idmap.isolated": "true",
@@ -290,24 +300,24 @@ func (suite *lxdTestSuite) TestContainer_findIdmap_mixed() {
 	map2, err := c2.(*containerLXC).NextIdmapSet()
 	suite.Req.Nil(err)
 
-	host := suite.d.IdmapSet.Idmap[0]
+	host := suite.d.os.IdmapSet.Idmap[0]
 
 	for i := 0; i < 2; i++ {
 		suite.Req.Equal(host.Hostid, map1.Idmap[i].Hostid, "hostids don't match %d", i)
-		suite.Req.Equal(0, map1.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(0), map1.Idmap[i].Nsid, "nsid nonzero")
 		suite.Req.Equal(host.Maprange, map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
 	for i := 0; i < 2; i++ {
 		suite.Req.Equal(host.Hostid+65536, map2.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(0, map2.Idmap[i].Nsid, "nsid nonzero")
-		suite.Req.Equal(65536, map2.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(0), map2.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(65536), map2.Idmap[i].Maprange, "incorrect maprange")
 	}
 }
 
-func (suite *lxdTestSuite) TestContainer_findIdmap_raw() {
-	c1, err := containerCreateInternal(suite.d, containerArgs{
-		Ctype: cTypeRegular,
+func (suite *containerTestSuite) TestContainer_findIdmap_raw() {
+	c1, err := containerCreateInternal(suite.d.State(), suite.d.Storage, db.ContainerArgs{
+		Ctype: db.CTypeRegular,
 		Name:  "isol-1",
 		Config: map[string]string{
 			"security.idmap.isolated": "false",
@@ -320,33 +330,33 @@ func (suite *lxdTestSuite) TestContainer_findIdmap_raw() {
 	map1, err := c1.(*containerLXC).NextIdmapSet()
 	suite.Req.Nil(err)
 
-	host := suite.d.IdmapSet.Idmap[0]
+	host := suite.d.os.IdmapSet.Idmap[0]
 
 	for _, i := range []int{0, 3} {
 		suite.Req.Equal(host.Hostid, map1.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(0, map1.Idmap[i].Nsid, "nsid nonzero")
-		suite.Req.Equal(1000, map1.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(0), map1.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(1000), map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
 	for _, i := range []int{1, 4} {
-		suite.Req.Equal(1000, map1.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(1000, map1.Idmap[i].Nsid, "invalid nsid")
-		suite.Req.Equal(1, map1.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(1000), map1.Idmap[i].Hostid, "hostids don't match")
+		suite.Req.Equal(int64(1000), map1.Idmap[i].Nsid, "invalid nsid")
+		suite.Req.Equal(int64(1), map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
 	for _, i := range []int{2, 5} {
 		suite.Req.Equal(host.Hostid+1001, map1.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(1001, map1.Idmap[i].Nsid, "invalid nsid")
+		suite.Req.Equal(int64(1001), map1.Idmap[i].Nsid, "invalid nsid")
 		suite.Req.Equal(host.Maprange-1000-1, map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 }
 
-func (suite *lxdTestSuite) TestContainer_findIdmap_maxed() {
-	maps := []*shared.IdmapSet{}
+func (suite *containerTestSuite) TestContainer_findIdmap_maxed() {
+	maps := []*idmap.IdmapSet{}
 
 	for i := 0; i < 7; i++ {
-		c, err := containerCreateInternal(suite.d, containerArgs{
-			Ctype: cTypeRegular,
+		c, err := containerCreateInternal(suite.d.State(), suite.d.Storage, db.ContainerArgs{
+			Ctype: db.CTypeRegular,
 			Name:  fmt.Sprintf("isol-%d", i),
 			Config: map[string]string{
 				"security.idmap.isolated": "true",
@@ -381,3 +391,7 @@ func (suite *lxdTestSuite) TestContainer_findIdmap_maxed() {
 		}
 	}
 }
+
+func TestContainerTestSuite(t *testing.T) {
+	suite.Run(t, new(containerTestSuite))
+}
