@@ -5,8 +5,6 @@ import (
 	"crypto/tls"
 	"crypto/x509"
 	"database/sql"
-	"encoding/hex"
-	"encoding/pem"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -22,25 +20,30 @@ import (
 	"syscall"
 	"time"
 
-	"golang.org/x/crypto/scrypt"
-
 	"github.com/gorilla/mux"
 	_ "github.com/mattn/go-sqlite3"
 	"github.com/syndtr/gocapability/capability"
 	"gopkg.in/tomb.v2"
 
-	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/state"
+	"github.com/lxc/lxd/lxd/sys"
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/logging"
+	"github.com/lxc/lxd/shared/version"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 // AppArmor
-var aaAdmin = true
-var aaAvailable = true
+var aaAvailable = false
+var aaAdmin = false
 var aaConfined = false
 var aaStacking = false
+var aaStacked = false
 
 // CGroup
 var cgBlkioController = false
@@ -62,13 +65,10 @@ type Socket struct {
 
 // A Daemon can respond to requests from a shared client.
 type Daemon struct {
-	architectures       []int
-	BackingFs           string
 	clientCerts         []x509.Certificate
+	os                  *sys.OS
 	db                  *sql.DB
 	group               string
-	IdmapSet            *shared.IdmapSet
-	lxcpath             string
 	mux                 *mux.Router
 	tomb                tomb.Tomb
 	readyChan           chan bool
@@ -83,17 +83,20 @@ type Daemon struct {
 
 	devlxd *net.UnixListener
 
-	MockMode  bool
 	SetupMode bool
 
-	imagesDownloading     map[string]chan bool
-	imagesDownloadingLock sync.RWMutex
-
 	tlsConfig *tls.Config
 
 	proxy func(req *http.Request) (*url.URL, error)
 }
 
+// NewDaemon returns a new, un-initialized Daemon object.
+func NewDaemon() *Daemon {
+	return &Daemon{
+		os: sys.NewOS(),
+	}
+}
+
 // Command is the basic structure for every API call.
 type Command struct {
 	name          string
@@ -105,142 +108,15 @@ type Command struct {
 	delete        func(d *Daemon, r *http.Request) Response
 }
 
-func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, error) {
-	var err error
-
-	var cert *x509.Certificate
-	if certificate != "" {
-		certBlock, _ := pem.Decode([]byte(certificate))
-		if certBlock == nil {
-			return nil, fmt.Errorf("Invalid certificate")
-		}
-
-		cert, err = x509.ParseCertificate(certBlock.Bytes)
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	tlsConfig, err := shared.GetTLSConfig("", "", cert)
-	if err != nil {
-		return nil, err
-	}
-
-	tr := &http.Transport{
-		TLSClientConfig:   tlsConfig,
-		Dial:              shared.RFC3493Dialer,
-		Proxy:             d.proxy,
-		DisableKeepAlives: true,
-	}
-
-	myhttp := http.Client{
-		Transport: tr,
-	}
-
-	req, err := http.NewRequest("GET", url, nil)
-	if err != nil {
-		return nil, err
-	}
-
-	req.Header.Set("User-Agent", shared.UserAgent)
-
-	r, err := myhttp.Do(req)
-	if err != nil {
-		return nil, err
-	}
-
-	resp, err := lxd.ParseResponse(r)
-	if err != nil {
-		return nil, err
-	}
-
-	if resp.Type != lxd.Sync {
-		return nil, fmt.Errorf("unexpected non-sync response")
-	}
-
-	return resp, nil
-}
-
-func (d *Daemon) httpGetFile(url string, certificate string) (*http.Response, error) {
-	var err error
-
-	var cert *x509.Certificate
-	if certificate != "" {
-		certBlock, _ := pem.Decode([]byte(certificate))
-		if certBlock == nil {
-			return nil, fmt.Errorf("Invalid certificate")
-		}
-
-		cert, err = x509.ParseCertificate(certBlock.Bytes)
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	tlsConfig, err := shared.GetTLSConfig("", "", cert)
-	if err != nil {
-		return nil, err
-	}
-
-	tr := &http.Transport{
-		TLSClientConfig:   tlsConfig,
-		Dial:              shared.RFC3493Dialer,
-		Proxy:             d.proxy,
-		DisableKeepAlives: true,
-	}
-	myhttp := http.Client{
-		Transport: tr,
-	}
-
-	req, err := http.NewRequest("GET", url, nil)
-	if err != nil {
-		return nil, err
-	}
-
-	req.Header.Set("User-Agent", shared.UserAgent)
-
-	raw, err := myhttp.Do(req)
-	if err != nil {
-		return nil, err
-	}
-
-	if raw.StatusCode != 200 {
-		_, err := lxd.HoistResponse(raw, lxd.Error)
-		if err != nil {
-			return nil, err
-		}
-
-		return nil, fmt.Errorf("non-200 status with no error response?")
-	}
-
-	return raw, nil
-}
-
 func readMyCert() (string, string, error) {
 	certf := shared.VarPath("server.crt")
 	keyf := shared.VarPath("server.key")
-	shared.LogDebug("Looking for existing certificates", log.Ctx{"cert": certf, "key": keyf})
+	logger.Debug("Looking for existing certificates", log.Ctx{"cert": certf, "key": keyf})
 	err := shared.FindOrGenCert(certf, keyf, false)
 
 	return certf, keyf, err
 }
 
-func (d *Daemon) isTrustedClient(r *http.Request) bool {
-	if r.RemoteAddr == "@" {
-		// Unix socket
-		return true
-	}
-	if r.TLS == nil {
-		return false
-	}
-	for i := range r.TLS.PeerCertificates {
-		if d.CheckTrustState(*r.TLS.PeerCertificates[i]) {
-			return true
-		}
-	}
-	return false
-}
-
 func isJSONRequest(r *http.Request) bool {
 	for k, vs := range r.Header {
 		if strings.ToLower(k) == "content-type" &&
@@ -252,14 +128,9 @@ func isJSONRequest(r *http.Request) bool {
 	return false
 }
 
-func (d *Daemon) isRecursionRequest(r *http.Request) bool {
-	recursionStr := r.FormValue("recursion")
-	recursion, err := strconv.Atoi(recursionStr)
-	if err != nil {
-		return false
-	}
-
-	return recursion == 1
+// State creates a new State instance liked to our internal db and os.
+func (d *Daemon) State() *state.State {
+	return state.NewState(d.db, d.os)
 }
 
 func (d *Daemon) createCmd(version string, c Command) {
@@ -273,20 +144,20 @@ func (d *Daemon) createCmd(version string, c Command) {
 	d.mux.HandleFunc(uri, func(w http.ResponseWriter, r *http.Request) {
 		w.Header().Set("Content-Type", "application/json")
 
-		if d.isTrustedClient(r) {
-			shared.LogDebug(
+		if util.IsTrustedClient(r, d.clientCerts) {
+			logger.Debug(
 				"handling",
 				log.Ctx{"method": r.Method, "url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else if r.Method == "GET" && c.untrustedGet {
-			shared.LogDebug(
+			logger.Debug(
 				"allowing untrusted GET",
 				log.Ctx{"url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else if r.Method == "POST" && c.untrustedPost {
-			shared.LogDebug(
+			logger.Debug(
 				"allowing untrusted POST",
 				log.Ctx{"url": r.URL.RequestURI(), "ip": r.RemoteAddr})
 		} else {
-			shared.LogWarn(
+			logger.Warn(
 				"rejecting request from untrusted client",
 				log.Ctx{"ip": r.RemoteAddr})
 			Forbidden.Render(w)
@@ -333,7 +204,7 @@ func (d *Daemon) createCmd(version string, c Command) {
 		if err := resp.Render(w); err != nil {
 			err := InternalError(err).Render(w)
 			if err != nil {
-				shared.LogErrorf("Failed writing error for error, giving up")
+				logger.Errorf("Failed writing error for error, giving up")
 			}
 		}
 
@@ -351,74 +222,33 @@ func (d *Daemon) createCmd(version string, c Command) {
 	})
 }
 
-func (d *Daemon) SetupStorageDriver() error {
-	var err error
-
-	lvmVgName := daemonConfig["storage.lvm_vg_name"].Get()
-	zfsPoolName := daemonConfig["storage.zfs_pool_name"].Get()
-
-	if lvmVgName != "" {
-		d.Storage, err = newStorage(d, storageTypeLvm)
-		if err != nil {
-			shared.LogErrorf("Could not initialize storage type LVM: %s - falling back to dir", err)
-		} else {
-			return nil
-		}
-	} else if zfsPoolName != "" {
-		d.Storage, err = newStorage(d, storageTypeZfs)
-		if err != nil {
-			shared.LogErrorf("Could not initialize storage type ZFS: %s - falling back to dir", err)
-		} else {
-			return nil
-		}
-	} else if d.BackingFs == "btrfs" {
-		d.Storage, err = newStorage(d, storageTypeBtrfs)
-		if err != nil {
-			shared.LogErrorf("Could not initialize storage type btrfs: %s - falling back to dir", err)
-		} else {
-			return nil
-		}
-	}
-
-	d.Storage, err = newStorage(d, storageTypeDir)
-
-	return err
-}
-
 // have we setup shared mounts?
 var sharedMounted bool
 var sharedMountsLock sync.Mutex
 
 func setupSharedMounts() error {
+	// Check if we already went through this
 	if sharedMounted {
 		return nil
 	}
 
+	// Get a lock to prevent races
 	sharedMountsLock.Lock()
 	defer sharedMountsLock.Unlock()
 
-	if sharedMounted {
-		return nil
-	}
-
+	// Check if already setup
 	path := shared.VarPath("shmounts")
-
-	isShared, err := shared.IsOnSharedMount(path)
-	if err != nil {
-		return err
-	}
-
-	if isShared {
-		// / may already be ms-shared, or shmounts may have
-		// been mounted by a previous lxd run
+	if shared.IsMountPoint(path) {
 		sharedMounted = true
 		return nil
 	}
 
-	if err := syscall.Mount(path, path, "none", syscall.MS_BIND, ""); err != nil {
+	// Mount a new tmpfs
+	if err := syscall.Mount("tmpfs", path, "tmpfs", 0, "size=100k,mode=0711"); err != nil {
 		return err
 	}
 
+	// Mark as MS_SHARED and MS_REC
 	var flags uintptr = syscall.MS_SHARED | syscall.MS_REC
 	if err := syscall.Mount(path, path, "none", flags, ""); err != nil {
 		return err
@@ -428,66 +258,6 @@ func setupSharedMounts() error {
 	return nil
 }
 
-func (d *Daemon) ListenAddresses() ([]string, error) {
-	addresses := make([]string, 0)
-
-	value := daemonConfig["core.https_address"].Get()
-	if value == "" {
-		return addresses, nil
-	}
-
-	localHost, localPort, err := net.SplitHostPort(value)
-	if err != nil {
-		localHost = value
-		localPort = shared.DefaultPort
-	}
-
-	if localHost == "0.0.0.0" || localHost == "::" || localHost == "[::]" {
-		ifaces, err := net.Interfaces()
-		if err != nil {
-			return addresses, err
-		}
-
-		for _, i := range ifaces {
-			addrs, err := i.Addrs()
-			if err != nil {
-				continue
-			}
-
-			for _, addr := range addrs {
-				var ip net.IP
-				switch v := addr.(type) {
-				case *net.IPNet:
-					ip = v.IP
-				case *net.IPAddr:
-					ip = v.IP
-				}
-
-				if !ip.IsGlobalUnicast() {
-					continue
-				}
-
-				if ip.To4() == nil {
-					if localHost == "0.0.0.0" {
-						continue
-					}
-					addresses = append(addresses, fmt.Sprintf("[%s]:%s", ip, localPort))
-				} else {
-					addresses = append(addresses, fmt.Sprintf("%s:%s", ip, localPort))
-				}
-			}
-		}
-	} else {
-		if strings.Contains(localHost, ":") {
-			addresses = append(addresses, fmt.Sprintf("[%s]:%s", localHost, localPort))
-		} else {
-			addresses = append(addresses, fmt.Sprintf("%s:%s", localHost, localPort))
-		}
-	}
-
-	return addresses, nil
-}
-
 func (d *Daemon) UpdateHTTPsPort(newAddress string) error {
 	oldAddress := daemonConfig["core.https_address"].Get()
 
@@ -524,7 +294,7 @@ func (d *Daemon) UpdateHTTPsPort(newAddress string) error {
 			return fmt.Errorf("cannot listen on https socket: %v", err)
 		}
 
-		d.tomb.Go(func() error { return http.Serve(tcpl, d.mux) })
+		d.tomb.Go(func() error { return http.Serve(tcpl, &lxdHttpServer{d.mux, d}) })
 		d.TCPSocket = &Socket{Socket: tcpl, CloseOnExit: true}
 	}
 
@@ -544,8 +314,6 @@ func haveMacAdmin() bool {
 
 func (d *Daemon) Init() error {
 	/* Initialize some variables */
-	d.imagesDownloading = map[string]chan bool{}
-
 	d.readyChan = make(chan bool)
 	d.shutdownChan = make(chan bool)
 
@@ -557,182 +325,160 @@ func (d *Daemon) Init() error {
 	}
 
 	/* Setup logging if that wasn't done before */
-	if shared.Log == nil {
-		shared.Log, err = logging.GetLogger("", "", true, true, nil)
+	if logger.Log == nil {
+		logger.Log, err = logging.GetLogger("", "", true, true, nil)
 		if err != nil {
 			return err
 		}
 	}
 
 	/* Print welcome message */
-	if d.MockMode {
-		shared.LogInfo("LXD is starting in mock mode",
+	if d.os.MockMode {
+		logger.Info(fmt.Sprintf("LXD %s is starting in mock mode", version.Version),
 			log.Ctx{"path": shared.VarPath("")})
 	} else if d.SetupMode {
-		shared.LogInfo("LXD is starting in setup mode",
+		logger.Info(fmt.Sprintf("LXD %s is starting in setup mode", version.Version),
 			log.Ctx{"path": shared.VarPath("")})
 	} else {
-		shared.LogInfo("LXD is starting in normal mode",
+		logger.Info(fmt.Sprintf("LXD %s is starting in normal mode", version.Version),
 			log.Ctx{"path": shared.VarPath("")})
 	}
 
 	/* Detect user namespaces */
 	runningInUserns = shared.RunningInUserNS()
 
-	/* Detect AppArmor support */
-	if aaAvailable && os.Getenv("LXD_SECURITY_APPARMOR") == "false" {
-		aaAvailable = false
-		aaAdmin = false
-		shared.LogWarnf("AppArmor support has been manually disabled")
-	}
-
-	if aaAvailable && !shared.IsDir("/sys/kernel/security/apparmor") {
-		aaAvailable = false
-		aaAdmin = false
-		shared.LogWarnf("AppArmor support has been disabled because of lack of kernel support")
-	}
-
+	/* Detect AppArmor availability */
 	_, err = exec.LookPath("apparmor_parser")
-	if aaAvailable && err != nil {
-		aaAvailable = false
-		aaAdmin = false
-		shared.LogWarnf("AppArmor support has been disabled because 'apparmor_parser' couldn't be found")
+	if os.Getenv("LXD_SECURITY_APPARMOR") == "false" {
+		logger.Warnf("AppArmor support has been manually disabled")
+	} else if !shared.IsDir("/sys/kernel/security/apparmor") {
+		logger.Warnf("AppArmor support has been disabled because of lack of kernel support")
+	} else if err != nil {
+		logger.Warnf("AppArmor support has been disabled because 'apparmor_parser' couldn't be found")
+	} else {
+		aaAvailable = true
+	}
+
+	/* Detect AppArmor stacking support */
+	aaCanStack := func() bool {
+		contentBytes, err := ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/stack")
+		if err != nil {
+			return false
+		}
+
+		if string(contentBytes) != "yes\n" {
+			return false
+		}
+
+		contentBytes, err = ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/version")
+		if err != nil {
+			return false
+		}
+
+		content := string(contentBytes)
+
+		parts := strings.Split(strings.TrimSpace(content), ".")
+
+		if len(parts) == 0 {
+			logger.Warn("unknown apparmor domain version", log.Ctx{"version": content})
+			return false
+		}
+
+		major, err := strconv.Atoi(parts[0])
+		if err != nil {
+			logger.Warn("unknown apparmor domain version", log.Ctx{"version": content})
+			return false
+		}
+
+		minor := 0
+		if len(parts) == 2 {
+			minor, err = strconv.Atoi(parts[1])
+			if err != nil {
+				logger.Warn("unknown apparmor domain version", log.Ctx{"version": content})
+				return false
+			}
+		}
+
+		return major >= 1 && minor >= 2
+	}
+
+	aaStacking = aaCanStack()
+
+	/* Detect existing AppArmor stack */
+	if shared.PathExists("/sys/kernel/security/apparmor/.ns_stacked") {
+		contentBytes, err := ioutil.ReadFile("/sys/kernel/security/apparmor/.ns_stacked")
+		if err == nil && string(contentBytes) == "yes\n" {
+			aaStacked = true
+		}
 	}
 
 	/* Detect AppArmor admin support */
-	if aaAdmin && !haveMacAdmin() {
-		aaAdmin = false
-		shared.LogWarnf("Per-container AppArmor profiles are disabled because the mac_admin capability is missing.")
-	}
-
-	if aaAdmin && runningInUserns {
-		aaAdmin = false
-		shared.LogWarnf("Per-container AppArmor profiles are disabled because LXD is running in an unprivileged container.")
+	if !haveMacAdmin() {
+		if aaAvailable {
+			logger.Warnf("Per-container AppArmor profiles are disabled because the mac_admin capability is missing.")
+		}
+	} else if runningInUserns && !aaStacked {
+		if aaAvailable {
+			logger.Warnf("Per-container AppArmor profiles are disabled because LXD is running in an unprivileged container without stacking.")
+		}
+	} else {
+		aaAdmin = true
 	}
 
 	/* Detect AppArmor confinment */
-	if !aaConfined {
-		profile := aaProfile()
-		if profile != "unconfined" && profile != "" {
-			aaConfined = true
-			shared.LogWarnf("Per-container AppArmor profiles are disabled because LXD is already protected by AppArmor.")
+	profile := aaProfile()
+	if profile != "unconfined" && profile != "" {
+		if aaAvailable {
+			logger.Warnf("Per-container AppArmor profiles are disabled because LXD is already protected by AppArmor.")
 		}
-	}
-
-	if aaAvailable {
-		canStack := func() bool {
-			contentBytes, err := ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/stack")
-			if err != nil {
-				return false
-			}
-
-			if string(contentBytes) != "yes\n" {
-				return false
-			}
-
-			contentBytes, err = ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/version")
-			if err != nil {
-				return false
-			}
-
-			content := string(contentBytes)
-
-			parts := strings.Split(strings.TrimSpace(content), ".")
-
-			if len(parts) == 0 {
-				shared.LogWarn("unknown apparmor domain version", log.Ctx{"version": content})
-				return false
-			}
-
-			major, err := strconv.Atoi(parts[0])
-			if err != nil {
-				shared.LogWarn("unknown apparmor domain version", log.Ctx{"version": content})
-				return false
-			}
-
-			minor := 0
-			if len(parts) == 2 {
-				minor, err = strconv.Atoi(parts[1])
-				if err != nil {
-					shared.LogWarn("unknown apparmor domain version", log.Ctx{"version": content})
-					return false
-				}
-			}
-
-			return major >= 1 && minor >= 2
-		}
-
-		aaStacking = canStack()
+		aaConfined = true
 	}
 
 	/* Detect CGroup support */
 	cgBlkioController = shared.PathExists("/sys/fs/cgroup/blkio/")
 	if !cgBlkioController {
-		shared.LogWarnf("Couldn't find the CGroup blkio controller, I/O limits will be ignored.")
+		logger.Warnf("Couldn't find the CGroup blkio controller, I/O limits will be ignored.")
 	}
 
 	cgCpuController = shared.PathExists("/sys/fs/cgroup/cpu/")
 	if !cgCpuController {
-		shared.LogWarnf("Couldn't find the CGroup CPU controller, CPU time limits will be ignored.")
+		logger.Warnf("Couldn't find the CGroup CPU controller, CPU time limits will be ignored.")
 	}
 
 	cgCpusetController = shared.PathExists("/sys/fs/cgroup/cpuset/")
 	if !cgCpusetController {
-		shared.LogWarnf("Couldn't find the CGroup CPUset controller, CPU pinning will be ignored.")
+		logger.Warnf("Couldn't find the CGroup CPUset controller, CPU pinning will be ignored.")
 	}
 
 	cgDevicesController = shared.PathExists("/sys/fs/cgroup/devices/")
 	if !cgDevicesController {
-		shared.LogWarnf("Couldn't find the CGroup devices controller, device access control won't work.")
+		logger.Warnf("Couldn't find the CGroup devices controller, device access control won't work.")
 	}
 
 	cgMemoryController = shared.PathExists("/sys/fs/cgroup/memory/")
 	if !cgMemoryController {
-		shared.LogWarnf("Couldn't find the CGroup memory controller, memory limits will be ignored.")
+		logger.Warnf("Couldn't find the CGroup memory controller, memory limits will be ignored.")
 	}
 
 	cgNetPrioController = shared.PathExists("/sys/fs/cgroup/net_prio/")
 	if !cgNetPrioController {
-		shared.LogWarnf("Couldn't find the CGroup network class controller, network limits will be ignored.")
+		logger.Warnf("Couldn't find the CGroup network class controller, network limits will be ignored.")
 	}
 
 	cgPidsController = shared.PathExists("/sys/fs/cgroup/pids/")
 	if !cgPidsController {
-		shared.LogWarnf("Couldn't find the CGroup pids controller, process limits will be ignored.")
+		logger.Warnf("Couldn't find the CGroup pids controller, process limits will be ignored.")
 	}
 
 	cgSwapAccounting = shared.PathExists("/sys/fs/cgroup/memory/memory.memsw.limit_in_bytes")
 	if !cgSwapAccounting {
-		shared.LogWarnf("CGroup memory swap accounting is disabled, swap limits will be ignored.")
+		logger.Warnf("CGroup memory swap accounting is disabled, swap limits will be ignored.")
 	}
 
-	/* Get the list of supported architectures */
-	var architectures = []int{}
-
-	architectureName, err := shared.ArchitectureGetLocal()
-	if err != nil {
-		return err
-	}
-
-	architecture, err := shared.ArchitectureId(architectureName)
-	if err != nil {
-		return err
-	}
-	architectures = append(architectures, architecture)
-
-	personalities, err := shared.ArchitecturePersonalities(architecture)
-	if err != nil {
-		return err
-	}
-	for _, personality := range personalities {
-		architectures = append(architectures, personality)
-	}
-	d.architectures = architectures
-
-	/* Set container path */
-	d.lxcpath = shared.VarPath("containers")
-
 	/* Make sure all our directories are available */
+	if err := os.MkdirAll(shared.VarPath(), 0711); err != nil {
+		return err
+	}
 	if err := os.MkdirAll(shared.CachePath(), 0700); err != nil {
 		return err
 	}
@@ -761,22 +507,10 @@ func (d *Daemon) Init() error {
 		return err
 	}
 
-	/* Detect the filesystem */
-	d.BackingFs, err = filesystemDetect(d.lxcpath)
+	/* Initialize the operating system facade */
+	err = d.os.Init()
 	if err != nil {
-		shared.LogError("Error detecting backing fs", log.Ctx{"err": err})
-	}
-
-	/* Read the uid/gid allocation */
-	d.IdmapSet, err = shared.DefaultIdmapSet()
-	if err != nil {
-		shared.LogWarn("Error reading idmap", log.Ctx{"err": err.Error()})
-		shared.LogWarnf("Only privileged containers will be able to run")
-	} else {
-		shared.LogInfof("Default uid/gid map:")
-		for _, lxcmap := range d.IdmapSet.ToLxcString() {
-			shared.LogInfof(strings.TrimRight(" - "+lxcmap, "\n"))
-		}
+		return err
 	}
 
 	/* Initialize the database */
@@ -791,9 +525,9 @@ func (d *Daemon) Init() error {
 		return err
 	}
 
-	if !d.MockMode {
+	if !d.os.MockMode {
 		/* Setup the storage driver */
-		err = d.SetupStorageDriver()
+		err = SetupStorageDriver(d)
 		if err != nil {
 			return fmt.Errorf("Failed to setup storage: %s", err)
 		}
@@ -815,14 +549,14 @@ func (d *Daemon) Init() error {
 	go func() {
 		t := time.NewTicker(24 * time.Hour)
 		for {
-			shared.LogInfof("Expiring log files")
+			logger.Infof("Expiring log files")
 
-			err := d.ExpireLogs()
+			err := ExpireLogs(d.db)
 			if err != nil {
-				shared.LogError("Failed to expire logs", log.Ctx{"err": err})
+				logger.Error("Failed to expire logs", log.Ctx{"err": err})
 			}
 
-			shared.LogInfof("Done expiring log files")
+			logger.Infof("Done expiring log files")
 			<-t.C
 		}
 	}()
@@ -834,16 +568,32 @@ func (d *Daemon) Init() error {
 		daemonConfig["core.proxy_ignore_hosts"].Get(),
 	)
 
+	/* Setup some mounts (nice to have) */
+	if !d.os.MockMode {
+		// Attempt to mount the shmounts tmpfs
+		setupSharedMounts()
+
+		// Attempt to Mount the devlxd tmpfs
+		if !shared.IsMountPoint(shared.VarPath("devlxd")) {
+			syscall.Mount("tmpfs", shared.VarPath("devlxd"), "tmpfs", 0, "size=100k,mode=0755")
+		}
+	}
+
 	/* Setup /dev/lxd */
-	shared.LogInfof("Starting /dev/lxd handler")
+	logger.Infof("Starting /dev/lxd handler")
 	d.devlxd, err = createAndBindDevLxd()
 	if err != nil {
 		return err
 	}
 
-	if !d.MockMode {
+	d.tomb.Go(func() error {
+		server := devLxdServer(d)
+		return server.Serve(d.devlxd)
+	})
+
+	if !d.os.MockMode {
 		/* Start the scheduler */
-		go deviceEventListener(d)
+		go deviceEventListener(d.State(), d.Storage)
 
 		/* Setup the TLS authentication */
 		certf, keyf, err := readMyCert()
@@ -857,12 +607,17 @@ func (d *Daemon) Init() error {
 		}
 
 		tlsConfig := &tls.Config{
-			InsecureSkipVerify: true,
-			ClientAuth:         tls.RequestClientCert,
-			Certificates:       []tls.Certificate{cert},
-			MinVersion:         tls.VersionTLS12,
-			MaxVersion:         tls.VersionTLS12,
+			ClientAuth:   tls.RequestClientCert,
+			Certificates: []tls.Certificate{cert},
+			MinVersion:   tls.VersionTLS12,
+			MaxVersion:   tls.VersionTLS12,
 			CipherSuites: []uint16{
+				tls.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
+				tls.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
+				tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
+				tls.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
+				tls.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
+				tls.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
 				tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
 				tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA},
 			PreferServerCipherSuites: true,
@@ -892,14 +647,15 @@ func (d *Daemon) Init() error {
 	}
 
 	d.mux.NotFoundHandler = http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-		shared.LogInfo("Sending top level 404", log.Ctx{"url": r.URL})
+		logger.Info("Sending top level 404", log.Ctx{"url": r.URL})
 		w.Header().Set("Content-Type", "application/json")
 		NotFound.Render(w)
 	})
 
-	listeners := d.GetListeners()
+	// Prepare the list of listeners
+	listeners := util.GetListeners()
 	if len(listeners) > 0 {
-		shared.LogInfof("LXD is socket activated")
+		logger.Infof("LXD is socket activated")
 
 		for _, listener := range listeners {
 			if shared.PathExists(listener.Addr().String()) {
@@ -910,16 +666,16 @@ func (d *Daemon) Init() error {
 			}
 		}
 	} else {
-		shared.LogInfof("LXD isn't socket activated")
+		logger.Infof("LXD isn't socket activated")
 
 		localSocketPath := shared.VarPath("unix.socket")
 
 		// If the socket exists, let's try to connect to it and see if there's
 		// a lxd running.
 		if shared.PathExists(localSocketPath) {
-			_, err := lxd.NewClient(&lxd.DefaultConfig, "local")
+			_, err := lxd.ConnectLXDUnix("", nil)
 			if err != nil {
-				shared.LogDebugf("Detected stale unix socket, deleting")
+				logger.Debugf("Detected stale unix socket, deleting")
 				// Connecting failed, so let's delete the socket and
 				// listen on it ourselves.
 				err = os.Remove(localSocketPath)
@@ -971,36 +727,30 @@ func (d *Daemon) Init() error {
 
 		tcpl, err := tls.Listen("tcp", listenAddr, d.tlsConfig)
 		if err != nil {
-			shared.LogError("cannot listen on https socket, skipping...", log.Ctx{"err": err})
+			logger.Error("cannot listen on https socket, skipping...", log.Ctx{"err": err})
 		} else {
 			if d.TCPSocket != nil {
-				shared.LogInfof("Replacing inherited TCP socket with configured one")
+				logger.Infof("Replacing inherited TCP socket with configured one")
 				d.TCPSocket.Socket.Close()
 			}
 			d.TCPSocket = &Socket{Socket: tcpl, CloseOnExit: true}
 		}
 	}
 
-	d.tomb.Go(func() error {
-		shared.LogInfof("REST API daemon:")
-		if d.UnixSocket != nil {
-			shared.LogInfo(" - binding Unix socket", log.Ctx{"socket": d.UnixSocket.Socket.Addr()})
-			d.tomb.Go(func() error { return http.Serve(d.UnixSocket.Socket, &lxdHttpServer{d.mux, d}) })
-		}
+	// Bind the REST API
+	logger.Infof("REST API daemon:")
+	if d.UnixSocket != nil {
+		logger.Info(" - binding Unix socket", log.Ctx{"socket": d.UnixSocket.Socket.Addr()})
+		d.tomb.Go(func() error { return http.Serve(d.UnixSocket.Socket, &lxdHttpServer{d.mux, d}) })
+	}
 
-		if d.TCPSocket != nil {
-			shared.LogInfo(" - binding TCP socket", log.Ctx{"socket": d.TCPSocket.Socket.Addr()})
-			d.tomb.Go(func() error { return http.Serve(d.TCPSocket.Socket, &lxdHttpServer{d.mux, d}) })
-		}
+	if d.TCPSocket != nil {
+		logger.Info(" - binding TCP socket", log.Ctx{"socket": d.TCPSocket.Socket.Addr()})
+		d.tomb.Go(func() error { return http.Serve(d.TCPSocket.Socket, &lxdHttpServer{d.mux, d}) })
+	}
 
-		d.tomb.Go(func() error {
-			server := devLxdServer(d)
-			return server.Serve(d.devlxd)
-		})
-		return nil
-	})
-
-	if !d.MockMode && !d.SetupMode {
+	// Run the post initialization actions
+	if !d.os.MockMode && !d.SetupMode {
 		err := d.Ready()
 		if err != nil {
 			return err
@@ -1014,12 +764,10 @@ func (d *Daemon) Ready() error {
 	/* Prune images */
 	d.pruneChan = make(chan bool)
 	go func() {
-		pruneExpiredImages(d)
 		for {
 			timer := time.NewTimer(24 * time.Hour)
-			timeChan := timer.C
 			select {
-			case <-timeChan:
+			case <-timer.C:
 				/* run once per day */
 				pruneExpiredImages(d)
 			case <-d.pruneChan:
@@ -1030,6 +778,9 @@ func (d *Daemon) Ready() error {
 		}
 	}()
 
+	// Do an initial pruning run before we start updating images
+	pruneExpiredImages(d)
+
 	/* Auto-update images */
 	d.resetAutoUpdateChan = make(chan bool)
 	go func() {
@@ -1044,10 +795,9 @@ func (d *Daemon) Ready() error {
 			interval := daemonConfig["images.auto_update_interval"].GetInt64()
 			if interval > 0 {
 				timer := time.NewTimer(time.Duration(interval) * time.Hour)
-				timeChan := timer.C
 
 				select {
-				case <-timeChan:
+				case <-timer.C:
 					autoUpdateImages(d)
 				case <-d.resetAutoUpdateChan:
 					timer.Stop()
@@ -1061,38 +811,37 @@ func (d *Daemon) Ready() error {
 		}
 	}()
 
+	/* Auto-update instance types */
+	go func() {
+		// Background update
+		for {
+			instanceRefreshTypes(d)
+			time.Sleep(24 * time.Hour)
+		}
+	}()
+
+	s := d.State()
+
 	/* Restore containers */
-	go containersRestart(d)
+	containersRestart(s, d.Storage)
 
 	/* Re-balance in case things changed while LXD was down */
-	deviceTaskBalance(d)
+	deviceTaskBalance(s, d.Storage)
 
 	close(d.readyChan)
 
 	return nil
 }
 
-// CheckTrustState returns True if the client is trusted else false.
-func (d *Daemon) CheckTrustState(cert x509.Certificate) bool {
-	for k, v := range d.clientCerts {
-		if bytes.Compare(cert.Raw, v.Raw) == 0 {
-			shared.LogDebug("Found cert", log.Ctx{"k": k})
-			return true
-		}
-		shared.Log.Debug("Client cert != key", log.Ctx{"k": k})
-	}
-	return false
-}
-
 func (d *Daemon) numRunningContainers() (int, error) {
-	results, err := dbContainersList(d.db, cTypeRegular)
+	results, err := db.ContainersList(d.db, db.CTypeRegular)
 	if err != nil {
 		return 0, err
 	}
 
 	count := 0
 	for _, r := range results {
-		container, err := containerLoadByName(d, r)
+		container, err := containerLoadByName(d.State(), d.Storage, r)
 		if err != nil {
 			continue
 		}
@@ -1112,43 +861,44 @@ func (d *Daemon) Stop() error {
 	forceStop := false
 
 	d.tomb.Kill(errStop)
-	shared.LogInfof("Stopping REST API handler:")
+	logger.Infof("Stopping REST API handler:")
 	for _, socket := range []*Socket{d.TCPSocket, d.UnixSocket} {
 		if socket == nil {
 			continue
 		}
 
 		if socket.CloseOnExit {
-			shared.LogInfo(" - closing socket", log.Ctx{"socket": socket.Socket.Addr()})
+			logger.Info(" - closing socket", log.Ctx{"socket": socket.Socket.Addr()})
 			socket.Socket.Close()
 		} else {
-			shared.LogInfo(" - skipping socket-activated socket", log.Ctx{"socket": socket.Socket.Addr()})
+			logger.Info(" - skipping socket-activated socket", log.Ctx{"socket": socket.Socket.Addr()})
 			forceStop = true
 		}
 	}
 
-	if n, err := d.numRunningContainers(); err != nil || n == 0 {
-		shared.LogInfof("Unmounting shmounts")
+	logger.Infof("Stopping /dev/lxd handler")
+	d.devlxd.Close()
+	logger.Infof("Stopped /dev/lxd handler")
 
+	if n, err := d.numRunningContainers(); err != nil || n == 0 {
+		logger.Infof("Unmounting temporary filesystems")
+
+		syscall.Unmount(shared.VarPath("devlxd"), syscall.MNT_DETACH)
 		syscall.Unmount(shared.VarPath("shmounts"), syscall.MNT_DETACH)
 
-		shared.LogInfof("Done unmounting shmounts")
+		logger.Infof("Done unmounting temporary filesystems")
 	} else {
-		shared.LogDebugf("Not unmounting shmounts (containers are still running)")
+		logger.Debugf("Not unmounting temporary filesystems (containers are still running)")
 	}
 
-	shared.LogInfof("Closing the database")
+	logger.Infof("Closing the database")
 	d.db.Close()
 
-	shared.LogInfof("Stopping /dev/lxd handler")
-	d.devlxd.Close()
-	shared.LogInfof("Stopped /dev/lxd handler")
-
-	shared.LogInfof("Saving simplestreams cache")
+	logger.Infof("Saving simplestreams cache")
 	imageSaveStreamCache()
-	shared.LogInfof("Saved simplestreams cache")
+	logger.Infof("Saved simplestreams cache")
 
-	if d.MockMode || forceStop {
+	if d.os.MockMode || forceStop {
 		return nil
 	}
 
@@ -1160,152 +910,6 @@ func (d *Daemon) Stop() error {
 	return err
 }
 
-func (d *Daemon) PasswordCheck(password string) error {
-	value := daemonConfig["core.trust_password"].Get()
-
-	// No password set
-	if value == "" {
-		return fmt.Errorf("No password is set")
-	}
-
-	// Compare the password
-	buff, err := hex.DecodeString(value)
-	if err != nil {
-		return err
-	}
-
-	salt := buff[0:32]
-	hash, err := scrypt.Key([]byte(password), salt, 1<<14, 8, 1, 64)
-	if err != nil {
-		return err
-	}
-
-	if !bytes.Equal(hash, buff[32:]) {
-		return fmt.Errorf("Bad password provided")
-	}
-
-	return nil
-}
-
-func (d *Daemon) ExpireLogs() error {
-	entries, err := ioutil.ReadDir(shared.LogPath())
-	if err != nil {
-		return err
-	}
-
-	result, err := dbContainersList(d.db, cTypeRegular)
-	if err != nil {
-		return err
-	}
-
-	newestFile := func(path string, dir os.FileInfo) time.Time {
-		newest := dir.ModTime()
-
-		entries, err := ioutil.ReadDir(path)
-		if err != nil {
-			return newest
-		}
-
-		for _, entry := range entries {
-			if entry.ModTime().After(newest) {
-				newest = entry.ModTime()
-			}
-		}
-
-		return newest
-	}
-
-	for _, entry := range entries {
-		// Check if the container still exists
-		if shared.StringInSlice(entry.Name(), result) {
-			// Remove any log file which wasn't modified in the past 48 hours
-			logs, err := ioutil.ReadDir(shared.LogPath(entry.Name()))
-			if err != nil {
-				return err
-			}
-
-			for _, logfile := range logs {
-				path := shared.LogPath(entry.Name(), logfile.Name())
-
-				// Always keep the LXC config
-				if logfile.Name() == "lxc.conf" {
-					continue
-				}
-
-				// Deal with directories (snapshots)
-				if logfile.IsDir() {
-					newest := newestFile(path, logfile)
-					if time.Since(newest).Hours() >= 48 {
-						os.RemoveAll(path)
-						if err != nil {
-							return err
-						}
-					}
-
-					continue
-				}
-
-				// Individual files
-				if time.Since(logfile.ModTime()).Hours() >= 48 {
-					err := os.Remove(path)
-					if err != nil {
-						return err
-					}
-				}
-			}
-		} else {
-			// Empty directory if unchanged in the past 24 hours
-			path := shared.LogPath(entry.Name())
-			newest := newestFile(path, entry)
-			if time.Since(newest).Hours() >= 24 {
-				err := os.RemoveAll(path)
-				if err != nil {
-					return err
-				}
-			}
-		}
-	}
-
-	return nil
-}
-
-func (d *Daemon) GetListeners() []net.Listener {
-	defer func() {
-		os.Unsetenv("LISTEN_PID")
-		os.Unsetenv("LISTEN_FDS")
-	}()
-
-	pid, err := strconv.Atoi(os.Getenv("LISTEN_PID"))
-	if err != nil {
-		return nil
-	}
-
-	if pid != os.Getpid() {
-		return nil
-	}
-
-	fds, err := strconv.Atoi(os.Getenv("LISTEN_FDS"))
-	if err != nil {
-		return nil
-	}
-
-	listeners := []net.Listener{}
-
-	for i := 3; i < 3+fds; i++ {
-		syscall.CloseOnExec(i)
-
-		file := os.NewFile(uintptr(i), fmt.Sprintf("inherited-fd%d", i))
-		listener, err := net.FileListener(file)
-		if err != nil {
-			continue
-		}
-
-		listeners = append(listeners, listener)
-	}
-
-	return listeners
-}
-
 type lxdHttpServer struct {
 	r *mux.Router
 	d *Daemon
@@ -1336,3 +940,50 @@ func (s *lxdHttpServer) ServeHTTP(rw http.ResponseWriter, req *http.Request) {
 	// Call the original server
 	s.r.ServeHTTP(rw, req)
 }
+
+// Create a database connection and perform any updates needed.
+func initializeDbObject(d *Daemon, path string) error {
+	var err error
+
+	// Open the database. If the file doesn't exist it is created.
+	d.db, err = db.OpenDb(path)
+	if err != nil {
+		return err
+	}
+
+	// Create the DB if it doesn't exist.
+	err = db.CreateDb(d.db, patchesGetNames())
+	if err != nil {
+		return fmt.Errorf("Error creating database: %s", err)
+	}
+
+	// Detect LXD downgrades
+	if db.GetSchema(d.db) > db.GetLatestSchema() {
+		return fmt.Errorf("The database schema is more recent than LXD's schema.")
+	}
+
+	// Apply any database update.
+	//
+	// NOTE: we use the legacyPatches parameter to run a few
+	// legacy non-db updates that were in place before the
+	// patches mechanism was introduced in lxd/patches.go. The
+	// rest of non-db patches will be applied separately via
+	// patchesApplyAll. See PR #3322 for more details.
+	legacy := map[int]*db.LegacyPatch{}
+	for i, patch := range legacyPatches {
+		legacy[i] = &db.LegacyPatch{
+			Hook: func() error {
+				return patch(d)
+			},
+		}
+	}
+	for _, i := range legacyPatchesNeedingDB {
+		legacy[i].NeedsDB = true
+	}
+	err = db.UpdatesApplyAll(d.db, true, legacy)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
