@@ -14,7 +14,9 @@ import (
 	"golang.org/x/crypto/scrypt"
 	log "gopkg.in/inconshreveable/log15.v2"
 
+	dbapi "github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/logger"
 )
 
 var daemonConfigLock sync.Mutex
@@ -49,8 +51,17 @@ func (k *daemonConfigKey) name() string {
 }
 
 func (k *daemonConfigKey) Validate(d *Daemon, value string) error {
-	// No need to validate when unsetting
+	// Handle unsetting
 	if value == "" {
+		value = k.defaultValue
+
+		if k.validator != nil {
+			err := k.validator(d, k.name(), value)
+			if err != nil {
+				return err
+			}
+		}
+
 		return nil
 	}
 
@@ -117,11 +128,16 @@ func (k *daemonConfigKey) Set(d *Daemon, value string) error {
 	k.currentValue = value
 	daemonConfigLock.Unlock()
 
-	err = dbConfigValueSet(d.db, name, value)
+	err = dbapi.ConfigValueSet(d.db, name, value)
 	if err != nil {
 		return err
 	}
 
+	// Run the trigger (if any)
+	if k.trigger != nil {
+		k.trigger(d, k.name(), value)
+	}
+
 	return nil
 }
 
@@ -164,29 +180,29 @@ func (k *daemonConfigKey) GetInt64() int64 {
 func daemonConfigInit(db *sql.DB) error {
 	// Set all the keys
 	daemonConfig = map[string]*daemonConfigKey{
-		"core.https_address":         &daemonConfigKey{valueType: "string", setter: daemonConfigSetAddress},
-		"core.https_allowed_headers": &daemonConfigKey{valueType: "string"},
-		"core.https_allowed_methods": &daemonConfigKey{valueType: "string"},
-		"core.https_allowed_origin":  &daemonConfigKey{valueType: "string"},
-		"core.proxy_http":            &daemonConfigKey{valueType: "string", setter: daemonConfigSetProxy},
-		"core.proxy_https":           &daemonConfigKey{valueType: "string", setter: daemonConfigSetProxy},
-		"core.proxy_ignore_hosts":    &daemonConfigKey{valueType: "string", setter: daemonConfigSetProxy},
-		"core.trust_password":        &daemonConfigKey{valueType: "string", hiddenValue: true, setter: daemonConfigSetPassword},
+		"core.https_address":         {valueType: "string", setter: daemonConfigSetAddress},
+		"core.https_allowed_headers": {valueType: "string"},
+		"core.https_allowed_methods": {valueType: "string"},
+		"core.https_allowed_origin":  {valueType: "string"},
+		"core.proxy_http":            {valueType: "string", setter: daemonConfigSetProxy},
+		"core.proxy_https":           {valueType: "string", setter: daemonConfigSetProxy},
+		"core.proxy_ignore_hosts":    {valueType: "string", setter: daemonConfigSetProxy},
+		"core.trust_password":        {valueType: "string", hiddenValue: true, setter: daemonConfigSetPassword},
 
-		"images.auto_update_cached":    &daemonConfigKey{valueType: "bool", defaultValue: "true"},
-		"images.auto_update_interval":  &daemonConfigKey{valueType: "int", defaultValue: "6"},
-		"images.compression_algorithm": &daemonConfigKey{valueType: "string", validator: daemonConfigValidateCompression, defaultValue: "gzip"},
-		"images.remote_cache_expiry":   &daemonConfigKey{valueType: "int", defaultValue: "10", trigger: daemonConfigTriggerExpiry},
+		"images.auto_update_cached":    {valueType: "bool", defaultValue: "true"},
+		"images.auto_update_interval":  {valueType: "int", defaultValue: "6", trigger: daemonConfigTriggerAutoUpdateInterval},
+		"images.compression_algorithm": {valueType: "string", validator: daemonConfigValidateCompression, defaultValue: "gzip"},
+		"images.remote_cache_expiry":   {valueType: "int", defaultValue: "10", trigger: daemonConfigTriggerExpiry},
 
-		"storage.lvm_fstype":        &daemonConfigKey{valueType: "string", defaultValue: "ext4", validValues: []string{"ext4", "xfs"}},
-		"storage.lvm_thinpool_name": &daemonConfigKey{valueType: "string", defaultValue: "LXDPool", validator: storageLVMValidateThinPoolName},
-		"storage.lvm_vg_name":       &daemonConfigKey{valueType: "string", validator: storageLVMValidateVolumeGroupName, setter: daemonConfigSetStorage},
-		"storage.lvm_volume_size":   &daemonConfigKey{valueType: "string", defaultValue: "10GiB"},
-		"storage.zfs_pool_name":     &daemonConfigKey{valueType: "string", validator: storageZFSValidatePoolName, setter: daemonConfigSetStorage},
+		"storage.lvm_fstype":        {valueType: "string", defaultValue: "ext4", validValues: []string{"ext4", "xfs"}},
+		"storage.lvm_thinpool_name": {valueType: "string", defaultValue: "LXDPool", validator: storageLVMValidateThinPoolName},
+		"storage.lvm_vg_name":       {valueType: "string", validator: storageLVMValidateVolumeGroupName, setter: daemonConfigSetStorage},
+		"storage.lvm_volume_size":   {valueType: "string", defaultValue: "10GiB"},
+		"storage.zfs_pool_name":     {valueType: "string", validator: storageZFSValidatePoolName, setter: daemonConfigSetStorage},
 	}
 
 	// Load the values from the DB
-	dbValues, err := dbConfigValuesGet(db)
+	dbValues, err := dbapi.ConfigValuesGet(db)
 	if err != nil {
 		return err
 	}
@@ -195,7 +211,8 @@ func daemonConfigInit(db *sql.DB) error {
 	for k, v := range dbValues {
 		_, ok := daemonConfig[k]
 		if !ok {
-			shared.LogError("Found invalid configuration key in database", log.Ctx{"key": k})
+			logger.Error("Found unknown configuration key in database", log.Ctx{"key": k})
+			continue
 		}
 
 		daemonConfig[k].currentValue = v
@@ -261,7 +278,7 @@ func daemonConfigSetStorage(d *Daemon, key string, value string) (string, error)
 	}()
 
 	// Update the current storage driver
-	err := d.SetupStorageDriver()
+	err := SetupStorageDriver(d)
 	if err != nil {
 		return "", err
 	}
@@ -298,7 +315,7 @@ func daemonConfigSetProxy(d *Daemon, key string, value string) (string, error) {
 
 	// Clear the simplestreams cache as it's tied to the old proxy config
 	imageStreamCacheLock.Lock()
-	for k, _ := range imageStreamCache {
+	for k := range imageStreamCache {
 		delete(imageStreamCache, k)
 	}
 	imageStreamCacheLock.Unlock()
@@ -311,6 +328,11 @@ func daemonConfigTriggerExpiry(d *Daemon, key string, value string) {
 	d.pruneChan <- true
 }
 
+func daemonConfigTriggerAutoUpdateInterval(d *Daemon, key string, value string) {
+	// Reset the auto-update interval loop
+	d.resetAutoUpdateChan <- true
+}
+
 func daemonConfigValidateCompression(d *Daemon, key string, value string) error {
 	if value == "none" {
 		return nil
