@@ -6,9 +6,9 @@ import (
 	"crypto/rand"
 	"encoding/hex"
 	"fmt"
+	"io/ioutil"
 	"math/big"
 	"os"
-	"os/exec"
 	"path"
 	"path/filepath"
 	"sort"
@@ -18,7 +18,11 @@ import (
 
 	_ "github.com/mattn/go-sqlite3"
 
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/state"
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/logger"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
@@ -168,7 +172,7 @@ func parseCpuset(cpu string) ([]int, error) {
 	return cpus, nil
 }
 
-func deviceTaskBalance(d *Daemon) {
+func deviceTaskBalance(s *state.State, storage storage) {
 	min := func(x, y int) int {
 		if x < y {
 			return x
@@ -187,30 +191,67 @@ func deviceTaskBalance(d *Daemon) {
 		// Older kernel - use cpuset.cpus
 		effectiveCpus, err = cGroupGet("cpuset", "/", "cpuset.cpus")
 		if err != nil {
-			shared.LogErrorf("Error reading host's cpuset.cpus")
+			logger.Errorf("Error reading host's cpuset.cpus")
 			return
 		}
 	}
+
+	effectiveCpusInt, err := parseCpuset(effectiveCpus)
+	if err != nil {
+		logger.Errorf("Error parsing effective CPU set")
+		return
+	}
+
+	isolatedCpusInt := []int{}
+	if shared.PathExists("/sys/devices/system/cpu/isolated") {
+		buf, err := ioutil.ReadFile("/sys/devices/system/cpu/isolated")
+		if err != nil {
+			logger.Errorf("Error reading host's isolated cpu")
+			return
+		}
+
+		// File might exist even though there are no isolated cpus.
+		isolatedCpus := strings.TrimSpace(string(buf))
+		if isolatedCpus != "" {
+			isolatedCpusInt, err = parseCpuset(isolatedCpus)
+			if err != nil {
+				logger.Errorf("Error parsing isolated CPU set: %s", string(isolatedCpus))
+				return
+			}
+		}
+	}
+
+	effectiveCpusSlice := []string{}
+	for _, id := range effectiveCpusInt {
+		if shared.IntInSlice(id, isolatedCpusInt) {
+			continue
+		}
+
+		effectiveCpusSlice = append(effectiveCpusSlice, fmt.Sprintf("%d", id))
+	}
+
+	effectiveCpus = strings.Join(effectiveCpusSlice, ",")
+
 	err = cGroupSet("cpuset", "/lxc", "cpuset.cpus", effectiveCpus)
 	if err != nil && shared.PathExists("/sys/fs/cgroup/cpuset/lxc") {
-		shared.LogWarn("Error setting lxd's cpuset.cpus", log.Ctx{"err": err})
+		logger.Warn("Error setting lxd's cpuset.cpus", log.Ctx{"err": err})
 	}
 	cpus, err := parseCpuset(effectiveCpus)
 	if err != nil {
-		shared.LogError("Error parsing host's cpu set", log.Ctx{"cpuset": effectiveCpus, "err": err})
+		logger.Error("Error parsing host's cpu set", log.Ctx{"cpuset": effectiveCpus, "err": err})
 		return
 	}
 
 	// Iterate through the containers
-	containers, err := dbContainersList(d.db, cTypeRegular)
+	containers, err := db.ContainersList(s.DB, db.CTypeRegular)
 	if err != nil {
-		shared.LogError("problem loading containers list", log.Ctx{"err": err})
+		logger.Error("problem loading containers list", log.Ctx{"err": err})
 		return
 	}
 	fixedContainers := map[int][]container{}
 	balancedContainers := map[container]int{}
 	for _, name := range containers {
-		c, err := containerLoadByName(d, name)
+		c, err := containerLoadByName(s, storage, name)
 		if err != nil {
 			continue
 		}
@@ -268,7 +309,7 @@ func deviceTaskBalance(d *Daemon) {
 	for cpu, ctns := range fixedContainers {
 		c, ok := usage[cpu]
 		if !ok {
-			shared.LogErrorf("Internal error: container using unavailable cpu")
+			logger.Errorf("Internal error: container using unavailable cpu")
 			continue
 		}
 		id := c.strId
@@ -317,25 +358,25 @@ func deviceTaskBalance(d *Daemon) {
 		sort.Strings(set)
 		err := ctn.CGroupSet("cpuset.cpus", strings.Join(set, ","))
 		if err != nil {
-			shared.LogError("balance: Unable to set cpuset", log.Ctx{"name": ctn.Name(), "err": err, "value": strings.Join(set, ",")})
+			logger.Error("balance: Unable to set cpuset", log.Ctx{"name": ctn.Name(), "err": err, "value": strings.Join(set, ",")})
 		}
 	}
 }
 
-func deviceNetworkPriority(d *Daemon, netif string) {
+func deviceNetworkPriority(s *state.State, storage storage, netif string) {
 	// Don't bother running when CGroup support isn't there
 	if !cgNetPrioController {
 		return
 	}
 
-	containers, err := dbContainersList(d.db, cTypeRegular)
+	containers, err := db.ContainersList(s.DB, db.CTypeRegular)
 	if err != nil {
 		return
 	}
 
 	for _, name := range containers {
 		// Get the container struct
-		c, err := containerLoadByName(d, name)
+		c, err := containerLoadByName(s, storage, name)
 		if err != nil {
 			continue
 		}
@@ -358,10 +399,10 @@ func deviceNetworkPriority(d *Daemon, netif string) {
 	return
 }
 
-func deviceEventListener(d *Daemon) {
+func deviceEventListener(s *state.State, storage storage) {
 	chNetlinkCPU, chNetlinkNetwork, err := deviceNetlinkListener()
 	if err != nil {
-		shared.LogErrorf("scheduler: couldn't setup netlink listener")
+		logger.Errorf("scheduler: couldn't setup netlink listener")
 		return
 	}
 
@@ -369,7 +410,7 @@ func deviceEventListener(d *Daemon) {
 		select {
 		case e := <-chNetlinkCPU:
 			if len(e) != 2 {
-				shared.LogErrorf("Scheduler: received an invalid cpu hotplug event")
+				logger.Errorf("Scheduler: received an invalid cpu hotplug event")
 				continue
 			}
 
@@ -377,11 +418,11 @@ func deviceEventListener(d *Daemon) {
 				continue
 			}
 
-			shared.LogDebugf("Scheduler: cpu: %s is now %s: re-balancing", e[0], e[1])
-			deviceTaskBalance(d)
+			logger.Debugf("Scheduler: cpu: %s is now %s: re-balancing", e[0], e[1])
+			deviceTaskBalance(s, storage)
 		case e := <-chNetlinkNetwork:
 			if len(e) != 2 {
-				shared.LogErrorf("Scheduler: received an invalid network hotplug event")
+				logger.Errorf("Scheduler: received an invalid network hotplug event")
 				continue
 			}
 
@@ -389,11 +430,11 @@ func deviceEventListener(d *Daemon) {
 				continue
 			}
 
-			shared.LogDebugf("Scheduler: network: %s has been added: updating network priorities", e[0])
-			deviceNetworkPriority(d, e[0])
+			logger.Debugf("Scheduler: network: %s has been added: updating network priorities", e[0])
+			deviceNetworkPriority(s, storage, e[0])
 		case e := <-deviceSchedRebalance:
 			if len(e) != 3 {
-				shared.LogErrorf("Scheduler: received an invalid rebalance event")
+				logger.Errorf("Scheduler: received an invalid rebalance event")
 				continue
 			}
 
@@ -401,8 +442,8 @@ func deviceEventListener(d *Daemon) {
 				continue
 			}
 
-			shared.LogDebugf("Scheduler: %s %s %s: re-balancing", e[0], e[1], e[2])
-			deviceTaskBalance(d)
+			logger.Debugf("Scheduler: %s %s %s: re-balancing", e[0], e[1], e[2])
+			deviceTaskBalance(s, storage)
 		}
 	}
 }
@@ -467,8 +508,8 @@ func deviceGetAttributes(path string) (string, int, int, error) {
 	}
 
 	// Return the device information
-	major := int(stat.Rdev / 256)
-	minor := int(stat.Rdev % 256)
+	major := shared.Major(stat.Rdev)
+	minor := shared.Minor(stat.Rdev)
 	return dType, major, minor, nil
 }
 
@@ -498,7 +539,8 @@ func deviceNextVeth() string {
 }
 
 func deviceRemoveInterface(nic string) error {
-	return exec.Command("ip", "link", "del", nic).Run()
+	_, err := shared.RunCommand("ip", "link", "del", "dev", nic)
+	return err
 }
 
 func deviceMountDisk(srcPath string, dstPath string, readonly bool, recursive bool) error {
@@ -529,6 +571,14 @@ func deviceMountDisk(srcPath string, dstPath string, readonly bool, recursive bo
 		return fmt.Errorf("Unable to mount %s at %s: %s", srcPath, dstPath, err)
 	}
 
+	// Remount bind mounts in readonly mode if requested
+	if readonly == true && flags&syscall.MS_BIND == syscall.MS_BIND {
+		flags = syscall.MS_RDONLY | syscall.MS_BIND | syscall.MS_REMOUNT
+		if err = syscall.Mount("", dstPath, fstype, uintptr(flags), ""); err != nil {
+			return fmt.Errorf("Unable to mount %s in readonly mode: %s", dstPath, err)
+		}
+	}
+
 	flags = syscall.MS_REC | syscall.MS_SLAVE
 	if err = syscall.Mount("", dstPath, "", uintptr(flags), ""); err != nil {
 		return fmt.Errorf("unable to make mount %s private: %s", dstPath, err)
@@ -686,19 +736,19 @@ func deviceGetParentBlocks(path string) ([]string, error) {
 
 	// Deal with per-filesystem oddities. We don't care about failures here
 	// because any non-special filesystem => directory backend.
-	fs, _ := filesystemDetect(expPath)
+	fs, _ := util.FilesystemDetect(expPath)
 
 	if fs == "zfs" && shared.PathExists("/dev/zfs") {
 		// Accessible zfs filesystems
 		poolName := strings.Split(device[1], "/")[0]
 
-		output, err := exec.Command("zpool", "status", poolName).CombinedOutput()
+		output, err := shared.RunCommand("zpool", "status", "-P", "-L", poolName)
 		if err != nil {
 			return nil, fmt.Errorf("Failed to query zfs filesystem information for %s: %s", device[1], output)
 		}
 
 		header := true
-		for _, line := range strings.Split(string(output), "\n") {
+		for _, line := range strings.Split(output, "\n") {
 			fields := strings.Fields(line)
 			if len(fields) < 5 {
 				continue
@@ -727,12 +777,6 @@ func deviceGetParentBlocks(path string) ([]string, error) {
 						devices = append(devices, dev)
 					}
 				}
-			} else if deviceIsBlockdev(fmt.Sprintf("/dev/%s", fields[0])) {
-				path = fmt.Sprintf("/dev/%s", fields[0])
-			} else if deviceIsBlockdev(fmt.Sprintf("/dev/disk/by-id/%s", fields[0])) {
-				path = fmt.Sprintf("/dev/disk/by-id/%s", fields[0])
-			} else if deviceIsBlockdev(fmt.Sprintf("/dev/mapper/%s", fields[0])) {
-				path = fmt.Sprintf("/dev/mapper/%s", fields[0])
 			} else {
 				continue
 			}
@@ -752,12 +796,12 @@ func deviceGetParentBlocks(path string) ([]string, error) {
 		}
 	} else if fs == "btrfs" && shared.PathExists(device[1]) {
 		// Accessible btrfs filesystems
-		output, err := exec.Command("btrfs", "filesystem", "show", device[1]).CombinedOutput()
+		output, err := shared.RunCommand("btrfs", "filesystem", "show", device[1])
 		if err != nil {
 			return nil, fmt.Errorf("Failed to query btrfs filesystem information for %s: %s", device[1], output)
 		}
 
-		for _, line := range strings.Split(string(output), "\n") {
+		for _, line := range strings.Split(output, "\n") {
 			fields := strings.Fields(line)
 			if len(fields) == 0 || fields[0] != "devid" {
 				continue
