@@ -15,7 +15,11 @@ import (
 
 	"github.com/gorilla/mux"
 
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/version"
 )
 
 type devLxdResponse struct {
@@ -42,7 +46,7 @@ type devLxdHandler struct {
 
 var configGet = devLxdHandler{"/1.0/config", func(c container, r *http.Request) *devLxdResponse {
 	filtered := []string{}
-	for k, _ := range c.ExpandedConfig() {
+	for k := range c.ExpandedConfig() {
 		if strings.HasPrefix(k, "user.") {
 			filtered = append(filtered, fmt.Sprintf("/1.0/config/%s", k))
 		}
@@ -70,11 +74,11 @@ var metadataGet = devLxdHandler{"/1.0/meta-data", func(c container, r *http.Requ
 }}
 
 var handlers = []devLxdHandler{
-	devLxdHandler{"/", func(c container, r *http.Request) *devLxdResponse {
+	{"/", func(c container, r *http.Request) *devLxdResponse {
 		return okResponse([]string{"/1.0"}, "json")
 	}},
-	devLxdHandler{"/1.0", func(c container, r *http.Request) *devLxdResponse {
-		return okResponse(shared.Jmap{"api_version": shared.APIVersion}, "json")
+	{"/1.0", func(c container, r *http.Request) *devLxdResponse {
+		return okResponse(shared.Jmap{"api_version": version.APIVersion}, "json")
 	}},
 	configGet,
 	configKeyGet,
@@ -116,7 +120,7 @@ func hoistReq(f func(container, *http.Request) *devLxdResponse, d *Daemon) func(
 			http.Error(w, fmt.Sprintf("%s", resp.content), resp.code)
 		} else if resp.ctype == "json" {
 			w.Header().Set("Content-Type", "application/json")
-			WriteJSON(w, resp.content)
+			util.WriteJSON(w, resp.content, debug)
 		} else {
 			w.Header().Set("Content-Type", "application/octet-stream")
 			fmt.Fprintf(w, resp.content.(string))
@@ -217,7 +221,7 @@ func (m *ConnPidMapper) ConnStateHandler(conn net.Conn, state http.ConnState) {
 	case http.StateNew:
 		cred, err := getCred(unixConn)
 		if err != nil {
-			shared.Debugf("Error getting ucred for conn %s", err)
+			logger.Debugf("Error getting ucred for conn %s", err)
 		} else {
 			m.m[unixConn] = cred
 		}
@@ -238,7 +242,7 @@ func (m *ConnPidMapper) ConnStateHandler(conn net.Conn, state http.ConnState) {
 	case http.StateClosed:
 		delete(m.m, unixConn)
 	default:
-		shared.Debugf("Unknown state for connection %s", state)
+		logger.Debugf("Unknown state for connection %s", state)
 	}
 }
 
@@ -246,16 +250,38 @@ func (m *ConnPidMapper) ConnStateHandler(conn net.Conn, state http.ConnState) {
  * I also don't see that golang exports an API to get at the underlying FD, but
  * we need it to get at SO_PEERCRED, so let's grab it.
  */
-func extractUnderlyingFd(unixConnPtr *net.UnixConn) int {
+func extractUnderlyingFd(unixConnPtr *net.UnixConn) (int, error) {
 	conn := reflect.Indirect(reflect.ValueOf(unixConnPtr))
+
 	netFdPtr := conn.FieldByName("fd")
+	if !netFdPtr.IsValid() {
+		return -1, fmt.Errorf("Unable to extract fd from net.UnixConn")
+	}
 	netFd := reflect.Indirect(netFdPtr)
+
 	fd := netFd.FieldByName("sysfd")
-	return int(fd.Int())
+	if !fd.IsValid() {
+		// Try under the new name
+		pfdPtr := netFd.FieldByName("pfd")
+		if !pfdPtr.IsValid() {
+			return -1, fmt.Errorf("Unable to extract pfd from netFD")
+		}
+		pfd := reflect.Indirect(pfdPtr)
+
+		fd = pfd.FieldByName("Sysfd")
+		if !fd.IsValid() {
+			return -1, fmt.Errorf("Unable to extract Sysfd from poll.FD")
+		}
+	}
+
+	return int(fd.Int()), nil
 }
 
 func getCred(conn *net.UnixConn) (*ucred, error) {
-	fd := extractUnderlyingFd(conn)
+	fd, err := extractUnderlyingFd(conn)
+	if err != nil {
+		return nil, err
+	}
 
 	uid, gid, pid, err := getUcred(fd)
 	if err != nil {
@@ -268,7 +294,7 @@ func getCred(conn *net.UnixConn) (*ucred, error) {
 /*
  * As near as I can tell, there is no nice way of extracting an underlying
  * net.Conn (or in our case, net.UnixConn) from an http.Request or
- * ResponseWriter without hijacking it [1]. Since we want to send and recieve
+ * ResponseWriter without hijacking it [1]. Since we want to send and receive
  * unix creds to figure out which container this request came from, we need to
  * do this.
  *
@@ -300,7 +326,7 @@ func findContainerForPid(pid int32, d *Daemon) (container, error) {
 	 *    an lxc monitor process and extract its name from there.
 	 *
 	 * 2. If this fails, it may be that someone did an `lxc exec foo bash`,
-	 *    so the process isn't actually a decendant of the container's
+	 *    so the process isn't actually a descendant of the container's
 	 *    init. In this case we just look through all the containers until
 	 *    we find an init with a matching pid namespace. This is probably
 	 *    uncommon, so hopefully the slowness won't hurt us.
@@ -319,7 +345,7 @@ func findContainerForPid(pid int32, d *Daemon) (container, error) {
 			parts := strings.Split(string(cmdline), " ")
 			name := strings.TrimSuffix(parts[len(parts)-1], "\x00")
 
-			return containerLoadByName(d, name)
+			return containerLoadByName(d.State(), d.Storage, name)
 		}
 
 		status, err := ioutil.ReadFile(fmt.Sprintf("/proc/%d/status", pid))
@@ -347,13 +373,13 @@ func findContainerForPid(pid int32, d *Daemon) (container, error) {
 		return nil, err
 	}
 
-	containers, err := dbContainersList(d.db, cTypeRegular)
+	containers, err := db.ContainersList(d.db, db.CTypeRegular)
 	if err != nil {
 		return nil, err
 	}
 
 	for _, container := range containers {
-		c, err := containerLoadByName(d, container)
+		c, err := containerLoadByName(d.State(), d.Storage, container)
 		if err != nil {
 			return nil, err
 		}
