@@ -11,8 +11,8 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"os/exec"
 	"path/filepath"
-	"strconv"
 	"strings"
 	"sync"
 
@@ -20,8 +20,9 @@ import (
 	"github.com/gorilla/websocket"
 	"gopkg.in/lxc/go-lxc.v2"
 
-	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/idmap"
+	"github.com/lxc/lxd/shared/logger"
 )
 
 type migrationFields struct {
@@ -136,7 +137,7 @@ func (c *migrationFields) controlChannel() <-chan MigrationControl {
 		msg := MigrationControl{}
 		err := c.recv(&msg)
 		if err != nil {
-			shared.LogDebugf("Got error reading migration control socket %s", err)
+			logger.Debugf("Got error reading migration control socket %s", err)
 			close(ch)
 			return
 		}
@@ -167,8 +168,9 @@ func NewMigrationSource(c container) (*migrationSourceWs, error) {
 	}
 
 	if c.IsRunning() {
-		if err := findCriu("source"); err != nil {
-			return nil, err
+		_, err := exec.LookPath("criu")
+		if err != nil {
+			return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the source server.")
 		}
 
 		ret.live = true
@@ -210,8 +212,9 @@ func (s *migrationSourceWs) Connect(op *operation, r *http.Request, w http.Respo
 	case s.fsSecret:
 		conn = &s.fsConn
 	default:
-		/* If we didn't find the right secret, the user provided a bad one,
-		 * which 403, not 404, since this operation actually exists */
+		// If we didn't find the right secret, the user provided a bad
+		// one, which 403, not 404, since this operation actually
+		// exists.
 		return os.ErrPermission
 	}
 
@@ -242,7 +245,8 @@ fi
 	}
 	defer f.Close()
 
-	if err := f.Chmod(0500); err != nil {
+	err = f.Chmod(0500)
+	if err != nil {
 		return err
 	}
 
@@ -274,6 +278,7 @@ func snapshotToProtobuf(c container) *Snapshot {
 	isEphemeral := c.IsEphemeral()
 	arch := int32(c.Architecture())
 	stateful := c.IsStateful()
+
 	return &Snapshot{
 		Name:         &parts[len(parts)-1],
 		LocalConfig:  config,
@@ -302,15 +307,19 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 
 	idmaps := make([]*IDMapType, 0)
 
-	idmapset := s.container.IdmapSet()
+	idmapset, err := s.container.IdmapSet()
+	if err != nil {
+		return err
+	}
+
 	if idmapset != nil {
 		for _, ctnIdmap := range idmapset.Idmap {
 			idmap := IDMapType{
 				Isuid:    proto.Bool(ctnIdmap.Isuid),
 				Isgid:    proto.Bool(ctnIdmap.Isgid),
-				Hostid:   proto.Int(ctnIdmap.Hostid),
-				Nsid:     proto.Int(ctnIdmap.Nsid),
-				Maprange: proto.Int(ctnIdmap.Maprange),
+				Hostid:   proto.Int32(int32(ctnIdmap.Hostid)),
+				Nsid:     proto.Int32(int32(ctnIdmap.Nsid)),
+				Maprange: proto.Int32(int32(ctnIdmap.Maprange)),
 			}
 
 			idmaps = append(idmaps, &idmap)
@@ -331,6 +340,8 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		}
 	}
 
+	// The protocol says we have to send a header no matter what, so let's
+	// do that, but then immediately send an error.
 	myType := s.container.Storage().MigrationType()
 	header := MigrationHeader{
 		Fs:            &myType,
@@ -340,7 +351,8 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		Snapshots:     snapshots,
 	}
 
-	if err := s.send(&header); err != nil {
+	err = s.send(&header)
+	if err != nil {
 		s.sendControl(err)
 		return err
 	}
@@ -350,7 +362,8 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		return fsErr
 	}
 
-	if err := s.recv(&header); err != nil {
+	err = s.recv(&header)
+	if err != nil {
 		s.sendControl(err)
 		return err
 	}
@@ -375,7 +388,8 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		return err
 	}
 
-	if err := driver.SendWhileRunning(s.fsConn); err != nil {
+	err = driver.SendWhileRunning(s.fsConn)
+	if err != nil {
 		return abort(err)
 	}
 
@@ -455,7 +469,8 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 				return abort(err)
 			}
 
-			if err := writeActionScript(checkpointDir, actionScriptOp.url, actionScriptOpSecret); err != nil {
+			err = writeActionScript(checkpointDir, actionScriptOp.url, actionScriptOpSecret)
+			if err != nil {
 				os.RemoveAll(checkpointDir)
 				return abort(err)
 			}
@@ -477,11 +492,12 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 				return abort(err)
 			/* the dump finished, let's continue on to the restore */
 			case <-dumpDone:
-				shared.LogDebugf("Dump finished, continuing with restore...")
+				logger.Debugf("Dump finished, continuing with restore...")
 			}
 		} else {
 			defer os.RemoveAll(checkpointDir)
-			if err := s.container.Migrate(lxc.MIGRATE_DUMP, checkpointDir, "migration", true, false); err != nil {
+			err = s.container.Migrate(lxc.MIGRATE_DUMP, checkpointDir, "migration", true, false)
+			if err != nil {
 				return abort(err)
 			}
 		}
@@ -493,11 +509,14 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		 * no reason to do these in parallel. In the future when we're using
 		 * p.haul's protocol, it will make sense to do these in parallel.
 		 */
-		if err := RsyncSend(shared.AddSlash(checkpointDir), s.criuConn); err != nil {
+		ctName, _, _ := containerGetParentAndSnapshotName(s.container.Name())
+		err = RsyncSend(ctName, shared.AddSlash(checkpointDir), s.criuConn)
+		if err != nil {
 			return abort(err)
 		}
 
-		if err := driver.SendAfterCheckpoint(s.fsConn); err != nil {
+		err = driver.SendAfterCheckpoint(s.fsConn)
+		if err != nil {
 			return abort(err)
 		}
 	}
@@ -505,7 +524,8 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 	driver.Cleanup()
 
 	msg := MigrationControl{}
-	if err := s.recv(&msg); err != nil {
+	err = s.recv(&msg)
+	if err != nil {
 		s.disconnect()
 		return err
 	}
@@ -514,7 +534,7 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 		restoreSuccess <- *msg.Success
 		err := <-dumpSuccess
 		if err != nil {
-			shared.LogErrorf("dump failed after successful restore?: %q", err)
+			logger.Errorf("dump failed after successful restore?: %q", err)
 		}
 	}
 
@@ -561,8 +581,9 @@ func NewMigrationSink(args *MigrationSinkArgs) (*migrationSink, error) {
 	sink.src.criuSecret, ok = args.Secrets["criu"]
 	sink.src.live = ok
 
-	if err := findCriu("destination"); sink.src.live && err != nil {
-		return nil, err
+	_, err := exec.LookPath("criu")
+	if sink.src.live && err != nil {
+		return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the destination server.")
 	}
 
 	return &sink, nil
@@ -574,7 +595,12 @@ func (c *migrationSink) connectWithSecret(secret string) (*websocket.Conn, error
 	// The URL is a https URL to the operation, mangle to be a wss URL to the secret
 	wsUrl := fmt.Sprintf("wss://%s/websocket?%s", strings.TrimPrefix(c.url, "https://"), query.Encode())
 
-	return lxd.WebsocketDial(c.dialer, wsUrl)
+	conn, _, err := c.dialer.Dial(wsUrl, http.Header{})
+	if err != nil {
+		return nil, err
+	}
+
+	return conn, err
 }
 
 func (c *migrationSink) Do(migrateOp *operation) error {
@@ -630,7 +656,8 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 		resp.Fs = &myType
 	}
 
-	if err := c.src.send(&resp); err != nil {
+	err = c.src.send(&resp)
+	if err != nil {
 		c.src.container.Delete()
 		c.src.sendControl(err)
 		return err
@@ -639,16 +666,16 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 	restore := make(chan error)
 	go func(c *migrationSink) {
 		imagesDir := ""
-		srcIdmap := new(shared.IdmapSet)
+		srcIdmap := new(idmap.IdmapSet)
 
-		for _, idmap := range header.Idmap {
-			e := shared.IdmapEntry{
-				Isuid:    *idmap.Isuid,
-				Isgid:    *idmap.Isgid,
-				Nsid:     int(*idmap.Nsid),
-				Hostid:   int(*idmap.Hostid),
-				Maprange: int(*idmap.Maprange)}
-			srcIdmap.Idmap = shared.Extend(srcIdmap.Idmap, e)
+		for _, idmapSet := range header.Idmap {
+			e := idmap.IdmapEntry{
+				Isuid:    *idmapSet.Isuid,
+				Isgid:    *idmapSet.Isgid,
+				Nsid:     int64(*idmapSet.Nsid),
+				Hostid:   int64(*idmapSet.Hostid),
+				Maprange: int64(*idmapSet.Maprange)}
+			srcIdmap.Idmap = idmap.Extend(srcIdmap.Idmap, e)
 		}
 
 		/* We do the fs receive in parallel so we don't have to reason
@@ -675,12 +702,14 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 				snapshots = header.Snapshots
 			}
 
-			if err := mySink(c.src.live, c.src.container, header.Snapshots, c.src.fsConn, srcIdmap); err != nil {
+			err := mySink(c.src.live, c.src.container, snapshots, c.src.fsConn, srcIdmap)
+			if err != nil {
 				fsTransfer <- err
 				return
 			}
 
-			if err := ShiftIfNecessary(c.src.container, srcIdmap); err != nil {
+			err = ShiftIfNecessary(c.src.container, srcIdmap)
+			if err != nil {
 				fsTransfer <- err
 				return
 			}
@@ -698,7 +727,8 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 
 			defer os.RemoveAll(imagesDir)
 
-			if err := RsyncRecv(shared.AddSlash(imagesDir), c.src.criuConn); err != nil {
+			err = RsyncRecv(shared.AddSlash(imagesDir), c.src.criuConn)
+			if err != nil {
 				restore <- err
 				return
 			}
@@ -747,7 +777,7 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 				// The source can only tell us it failed (e.g. if
 				// checkpointing failed). We have to tell the source
 				// whether or not the restore was successful.
-				shared.LogDebugf("Unknown message %v from source", msg)
+				logger.Debugf("Unknown message %v from source", msg)
 				err = c.src.container.TemplateApply("copy")
 				if err != nil {
 					c.src.container.Delete()
@@ -757,46 +787,3 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 		}
 	}
 }
-
-/*
- * Similar to forkstart, this is called when lxd is invoked as:
- *
- *    lxd forkmigrate <container> <lxcpath> <path_to_config> <path_to_criu_images> <preserves_inodes>
- *
- * liblxc's restore() sets up the processes in such a way that the monitor ends
- * up being a child of the process that calls it, in our case lxd. However, we
- * really want the monitor to be daemonized, so we fork again. Additionally, we
- * want to fork for the same reasons we do forkstart (i.e. reduced memory
- * footprint when we fork tasks that will never free golang's memory, etc.)
- */
-func MigrateContainer(args []string) error {
-	if len(args) != 6 {
-		return fmt.Errorf("Bad arguments %q", args)
-	}
-
-	name := args[1]
-	lxcpath := args[2]
-	configPath := args[3]
-	imagesDir := args[4]
-	preservesInodes, err := strconv.ParseBool(args[5])
-
-	c, err := lxc.NewContainer(name, lxcpath)
-	if err != nil {
-		return err
-	}
-
-	if err := c.LoadConfigFile(configPath); err != nil {
-		return err
-	}
-
-	/* see https://github.com/golang/go/issues/13155, startContainer, and dc3a229 */
-	os.Stdin.Close()
-	os.Stdout.Close()
-	os.Stderr.Close()
-
-	return c.Migrate(lxc.MIGRATE_RESTORE, lxc.MigrateOptions{
-		Directory:       imagesDir,
-		Verbose:         true,
-		PreservesInodes: preservesInodes,
-	})
-}
