@@ -11,7 +11,12 @@ import (
 	"github.com/gorilla/mux"
 	"github.com/pborman/uuid"
 
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd/shared/cancel"
+	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/version"
 )
 
 var operationsLock sync.Mutex
@@ -38,12 +43,13 @@ type operation struct {
 	class     operationClass
 	createdAt time.Time
 	updatedAt time.Time
-	status    shared.StatusCode
+	status    api.StatusCode
 	url       string
 	resources map[string][]string
 	metadata  map[string]interface{}
 	err       string
 	readonly  bool
+	canceler  *cancel.Canceler
 
 	// Those functions are called at various points in the operation lifecycle
 	onRun     func(*operation) error
@@ -96,27 +102,27 @@ func (op *operation) done() {
 }
 
 func (op *operation) Run() (chan error, error) {
-	if op.status != shared.Pending {
+	if op.status != api.Pending {
 		return nil, fmt.Errorf("Only pending operations can be started")
 	}
 
 	chanRun := make(chan error, 1)
 
 	op.lock.Lock()
-	op.status = shared.Running
+	op.status = api.Running
 
 	if op.onRun != nil {
 		go func(op *operation, chanRun chan error) {
 			err := op.onRun(op)
 			if err != nil {
 				op.lock.Lock()
-				op.status = shared.Failure
+				op.status = api.Failure
 				op.err = SmartError(err).String()
 				op.lock.Unlock()
 				op.done()
 				chanRun <- err
 
-				shared.Debugf("Failure for %s operation: %s: %s", op.class.String(), op.id, err)
+				logger.Debugf("Failure for %s operation: %s: %s", op.class.String(), op.id, err)
 
 				_, md, _ := op.Render()
 				eventSend("operation", md)
@@ -124,13 +130,13 @@ func (op *operation) Run() (chan error, error) {
 			}
 
 			op.lock.Lock()
-			op.status = shared.Success
+			op.status = api.Success
 			op.lock.Unlock()
 			op.done()
 			chanRun <- nil
 
 			op.lock.Lock()
-			shared.Debugf("Success for %s operation: %s", op.class.String(), op.id)
+			logger.Debugf("Success for %s operation: %s", op.class.String(), op.id)
 			_, md, _ := op.Render()
 			eventSend("operation", md)
 			op.lock.Unlock()
@@ -138,7 +144,7 @@ func (op *operation) Run() (chan error, error) {
 	}
 	op.lock.Unlock()
 
-	shared.Debugf("Started %s operation: %s", op.class.String(), op.id)
+	logger.Debugf("Started %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
@@ -146,7 +152,7 @@ func (op *operation) Run() (chan error, error) {
 }
 
 func (op *operation) Cancel() (chan error, error) {
-	if op.status != shared.Running {
+	if op.status != api.Running {
 		return nil, fmt.Errorf("Only running operations can be cancelled")
 	}
 
@@ -158,11 +164,11 @@ func (op *operation) Cancel() (chan error, error) {
 
 	op.lock.Lock()
 	oldStatus := op.status
-	op.status = shared.Cancelling
+	op.status = api.Cancelling
 	op.lock.Unlock()
 
 	if op.onCancel != nil {
-		go func(op *operation, oldStatus shared.StatusCode, chanCancel chan error) {
+		go func(op *operation, oldStatus api.StatusCode, chanCancel chan error) {
 			err := op.onCancel(op)
 			if err != nil {
 				op.lock.Lock()
@@ -170,37 +176,44 @@ func (op *operation) Cancel() (chan error, error) {
 				op.lock.Unlock()
 				chanCancel <- err
 
-				shared.Debugf("Failed to cancel %s operation: %s: %s", op.class.String(), op.id, err)
+				logger.Debugf("Failed to cancel %s operation: %s: %s", op.class.String(), op.id, err)
 				_, md, _ := op.Render()
 				eventSend("operation", md)
 				return
 			}
 
 			op.lock.Lock()
-			op.status = shared.Cancelled
+			op.status = api.Cancelled
 			op.lock.Unlock()
 			op.done()
 			chanCancel <- nil
 
-			shared.Debugf("Cancelled %s operation: %s", op.class.String(), op.id)
+			logger.Debugf("Cancelled %s operation: %s", op.class.String(), op.id)
 			_, md, _ := op.Render()
 			eventSend("operation", md)
 		}(op, oldStatus, chanCancel)
 	}
 
-	shared.Debugf("Cancelling %s operation: %s", op.class.String(), op.id)
+	logger.Debugf("Cancelling %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
+	if op.canceler != nil {
+		err := op.canceler.Cancel()
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	if op.onCancel == nil {
 		op.lock.Lock()
-		op.status = shared.Cancelled
+		op.status = api.Cancelled
 		op.lock.Unlock()
 		op.done()
 		chanCancel <- nil
 	}
 
-	shared.Debugf("Cancelled %s operation: %s", op.class.String(), op.id)
+	logger.Debugf("Cancelled %s operation: %s", op.class.String(), op.id)
 	_, md, _ = op.Render()
 	eventSend("operation", md)
 
@@ -212,7 +225,7 @@ func (op *operation) Connect(r *http.Request, w http.ResponseWriter) (chan error
 		return nil, fmt.Errorf("Only websocket operations can be connected")
 	}
 
-	if op.status != shared.Running {
+	if op.status != api.Running {
 		return nil, fmt.Errorf("Only running operations can be connected")
 	}
 
@@ -225,26 +238,38 @@ func (op *operation) Connect(r *http.Request, w http.ResponseWriter) (chan error
 		if err != nil {
 			chanConnect <- err
 
-			shared.Debugf("Failed to handle %s operation: %s: %s", op.class.String(), op.id, err)
+			logger.Debugf("Failed to handle %s operation: %s: %s", op.class.String(), op.id, err)
 			return
 		}
 
 		chanConnect <- nil
 
-		shared.Debugf("Handled %s operation: %s", op.class.String(), op.id)
+		logger.Debugf("Handled %s operation: %s", op.class.String(), op.id)
 	}(op, chanConnect)
 	op.lock.Unlock()
 
-	shared.Debugf("Connected %s operation: %s", op.class.String(), op.id)
+	logger.Debugf("Connected %s operation: %s", op.class.String(), op.id)
 
 	return chanConnect, nil
 }
 
 func (op *operation) mayCancel() bool {
-	return op.onCancel != nil || op.class == operationClassToken
+	if op.class == operationClassToken {
+		return true
+	}
+
+	if op.onCancel != nil {
+		return true
+	}
+
+	if op.canceler != nil && op.canceler.Cancelable() {
+		return true
+	}
+
+	return false
 }
 
-func (op *operation) Render() (string, *shared.Operation, error) {
+func (op *operation) Render() (string, *api.Operation, error) {
 	// Setup the resource URLs
 	resources := op.resources
 	if resources != nil {
@@ -252,24 +277,22 @@ func (op *operation) Render() (string, *shared.Operation, error) {
 		for key, value := range resources {
 			var values []string
 			for _, c := range value {
-				values = append(values, fmt.Sprintf("/%s/%s/%s", shared.APIVersion, key, c))
+				values = append(values, fmt.Sprintf("/%s/%s/%s", version.APIVersion, key, c))
 			}
 			tmpResources[key] = values
 		}
 		resources = tmpResources
 	}
 
-	md := shared.Jmap(op.metadata)
-
-	return op.url, &shared.Operation{
-		Id:         op.id,
+	return op.url, &api.Operation{
+		ID:         op.id,
 		Class:      op.class.String(),
 		CreatedAt:  op.createdAt,
 		UpdatedAt:  op.updatedAt,
 		Status:     op.status.String(),
 		StatusCode: op.status,
 		Resources:  resources,
-		Metadata:   &md,
+		Metadata:   op.metadata,
 		MayCancel:  op.mayCancel(),
 		Err:        op.err,
 	}, nil
@@ -283,23 +306,19 @@ func (op *operation) WaitFinal(timeout int) (bool, error) {
 
 	// Wait indefinitely
 	if timeout == -1 {
-		for {
-			<-op.chanDone
-			return true, nil
-		}
+		<-op.chanDone
+		return true, nil
 	}
 
 	// Wait until timeout
 	if timeout > 0 {
 		timer := time.NewTimer(time.Duration(timeout) * time.Second)
-		for {
-			select {
-			case <-op.chanDone:
-				return false, nil
+		select {
+		case <-op.chanDone:
+			return false, nil
 
-			case <-timer.C:
-				return false, nil
-			}
+		case <-timer.C:
+			return false, nil
 		}
 	}
 
@@ -307,7 +326,7 @@ func (op *operation) WaitFinal(timeout int) (bool, error) {
 }
 
 func (op *operation) UpdateResources(opResources map[string][]string) error {
-	if op.status != shared.Pending && op.status != shared.Running {
+	if op.status != api.Pending && op.status != api.Running {
 		return fmt.Errorf("Only pending or running operations can be updated")
 	}
 
@@ -320,7 +339,7 @@ func (op *operation) UpdateResources(opResources map[string][]string) error {
 	op.resources = opResources
 	op.lock.Unlock()
 
-	shared.Debugf("Updated resources for %s operation: %s", op.class.String(), op.id)
+	logger.Debugf("Updated resources for %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
@@ -328,7 +347,7 @@ func (op *operation) UpdateResources(opResources map[string][]string) error {
 }
 
 func (op *operation) UpdateMetadata(opMetadata interface{}) error {
-	if op.status != shared.Pending && op.status != shared.Running {
+	if op.status != api.Pending && op.status != api.Running {
 		return fmt.Errorf("Only pending or running operations can be updated")
 	}
 
@@ -346,7 +365,7 @@ func (op *operation) UpdateMetadata(opMetadata interface{}) error {
 	op.metadata = newMetadata
 	op.lock.Unlock()
 
-	shared.Debugf("Updated metadata for %s operation: %s", op.class.String(), op.id)
+	logger.Debugf("Updated metadata for %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
@@ -364,8 +383,8 @@ func operationCreate(opClass operationClass, opResources map[string][]string, op
 	op.class = opClass
 	op.createdAt = time.Now()
 	op.updatedAt = op.createdAt
-	op.status = shared.Pending
-	op.url = fmt.Sprintf("/%s/operations/%s", shared.APIVersion, op.id)
+	op.status = api.Pending
+	op.url = fmt.Sprintf("/%s/operations/%s", version.APIVersion, op.id)
 	op.resources = opResources
 	op.chanDone = make(chan error)
 
@@ -401,7 +420,7 @@ func operationCreate(opClass operationClass, opResources map[string][]string, op
 	operations[op.id] = &op
 	operationsLock.Unlock()
 
-	shared.Debugf("New %s operation: %s", op.class.String(), op.id)
+	logger.Debugf("New %s operation: %s", op.class.String(), op.id)
 	_, md, _ := op.Render()
 	eventSend("operation", md)
 
@@ -431,7 +450,7 @@ func operationAPIGet(d *Daemon, r *http.Request) Response {
 
 	_, body, err := op.Render()
 	if err != nil {
-		return InternalError(err)
+		return SmartError(err)
 	}
 
 	return SyncResponse(true, body)
@@ -458,7 +477,7 @@ var operationCmd = Command{name: "operations/{id}", get: operationAPIGet, delete
 func operationsAPIGet(d *Daemon, r *http.Request) Response {
 	var md shared.Jmap
 
-	recursion := d.isRecursionRequest(r)
+	recursion := util.IsRecursionRequest(r)
 
 	md = shared.Jmap{}
 
@@ -471,7 +490,7 @@ func operationsAPIGet(d *Daemon, r *http.Request) Response {
 		_, ok := md[status]
 		if !ok {
 			if recursion {
-				md[status] = make([]*shared.Operation, 0)
+				md[status] = make([]*api.Operation, 0)
 			} else {
 				md[status] = make([]string, 0)
 			}
@@ -487,7 +506,7 @@ func operationsAPIGet(d *Daemon, r *http.Request) Response {
 			continue
 		}
 
-		md[status] = append(md[status].([]*shared.Operation), body)
+		md[status] = append(md[status].([]*api.Operation), body)
 	}
 
 	return SyncResponse(true, md)
@@ -514,7 +533,7 @@ func operationAPIWaitGet(d *Daemon, r *http.Request) Response {
 
 	_, body, err := op.Render()
 	if err != nil {
-		return InternalError(err)
+		return SmartError(err)
 	}
 
 	return SyncResponse(true, body)
@@ -543,7 +562,7 @@ func (r *operationWebSocket) String() string {
 		return fmt.Sprintf("error: %s", err)
 	}
 
-	return md.Id
+	return md.ID
 }
 
 func operationAPIWebsocketGet(d *Daemon, r *http.Request) Response {
