@@ -4,44 +4,42 @@ import (
 	"encoding/json"
 	"fmt"
 	"net/http"
-	"reflect"
 	"strings"
 
 	"github.com/gorilla/mux"
 	_ "github.com/mattn/go-sqlite3"
 
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/state"
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/version"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 /* This is used for both profiles post and profile put */
-type profilesPostReq struct {
-	Name        string            `json:"name"`
-	Config      map[string]string `json:"config"`
-	Description string            `json:"description"`
-	Devices     shared.Devices    `json:"devices"`
-}
-
 func profilesGet(d *Daemon, r *http.Request) Response {
-	results, err := dbProfiles(d.db)
+	results, err := db.Profiles(d.db)
 	if err != nil {
 		return SmartError(err)
 	}
 
-	recursion := d.isRecursionRequest(r)
+	recursion := util.IsRecursionRequest(r)
 
 	resultString := make([]string, len(results))
-	resultMap := make([]*shared.ProfileConfig, len(results))
+	resultMap := make([]*api.Profile, len(results))
 	i := 0
 	for _, name := range results {
 		if !recursion {
-			url := fmt.Sprintf("/%s/profiles/%s", shared.APIVersion, name)
+			url := fmt.Sprintf("/%s/profiles/%s", version.APIVersion, name)
 			resultString[i] = url
 		} else {
-			profile, err := doProfileGet(d, name)
+			profile, err := doProfileGet(d.State(), name)
 			if err != nil {
-				shared.LogError("Failed to get profile", log.Ctx{"profile": name})
+				logger.Error("Failed to get profile", log.Ctx{"profile": name})
 				continue
 			}
 			resultMap[i] = profile
@@ -57,7 +55,7 @@ func profilesGet(d *Daemon, r *http.Request) Response {
 }
 
 func profilesPost(d *Daemon, r *http.Request) Response {
-	req := profilesPostReq{}
+	req := api.ProfilesPost{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -67,7 +65,7 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("No name provided"))
 	}
 
-	_, profile, _ := dbProfileGet(d.db, req.Name)
+	_, profile, _ := db.ProfileGet(d.db, req.Name)
 	if profile != nil {
 		return BadRequest(fmt.Errorf("The profile already exists"))
 	}
@@ -80,7 +78,7 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("Invalid profile name '%s'", req.Name))
 	}
 
-	err := containerValidConfig(req.Config, true, false)
+	err := containerValidConfig(d.os, req.Config, true, false)
 	if err != nil {
 		return BadRequest(err)
 	}
@@ -91,13 +89,13 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 	}
 
 	// Update DB entry
-	_, err = dbProfileCreate(d.db, req.Name, req.Description, req.Config, req.Devices)
+	_, err = db.ProfileCreate(d.db, req.Name, req.Description, req.Config, req.Devices)
 	if err != nil {
-		return InternalError(
+		return SmartError(
 			fmt.Errorf("Error inserting %s into database: %s", req.Name, err))
 	}
 
-	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/profiles/%s", shared.APIVersion, req.Name))
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/profiles/%s", version.APIVersion, req.Name))
 }
 
 var profilesCmd = Command{
@@ -105,15 +103,15 @@ var profilesCmd = Command{
 	get:  profilesGet,
 	post: profilesPost}
 
-func doProfileGet(d *Daemon, name string) (*shared.ProfileConfig, error) {
-	_, profile, err := dbProfileGet(d.db, name)
+func doProfileGet(s *state.State, name string) (*api.Profile, error) {
+	_, profile, err := db.ProfileGet(s.DB, name)
 	return profile, err
 }
 
 func profileGet(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
 
-	resp, err := doProfileGet(d, name)
+	resp, err := doProfileGet(d.State(), name)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -121,18 +119,18 @@ func profileGet(d *Daemon, r *http.Request) Response {
 	return SyncResponse(true, resp)
 }
 
-func getContainersWithProfile(d *Daemon, profile string) []container {
+func getContainersWithProfile(s *state.State, storage storage, profile string) []container {
 	results := []container{}
 
-	output, err := dbProfileContainersGet(d.db, profile)
+	output, err := db.ProfileContainersGet(s.DB, profile)
 	if err != nil {
 		return results
 	}
 
 	for _, name := range output {
-		c, err := containerLoadByName(d, name)
+		c, err := containerLoadByName(s, storage, name)
 		if err != nil {
-			shared.LogError("Failed opening container", log.Ctx{"container": name})
+			logger.Error("Failed opening container", log.Ctx{"container": name})
 			continue
 		}
 		results = append(results, c)
@@ -142,110 +140,26 @@ func getContainersWithProfile(d *Daemon, profile string) []container {
 }
 
 func profilePut(d *Daemon, r *http.Request) Response {
+	// Get the profile
 	name := mux.Vars(r)["name"]
+	id, profile, err := db.ProfileGet(d.db, name)
+	if err != nil {
+		return SmartError(fmt.Errorf("Failed to retrieve profile='%s'", name))
+	}
 
-	req := profilesPostReq{}
+	req := api.ProfilePut{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
 
-	// Sanity checks
-	err := containerValidConfig(req.Config, true, false)
-	if err != nil {
-		return BadRequest(err)
-	}
-
-	err = containerValidDevices(req.Devices, true, false)
-	if err != nil {
-		return BadRequest(err)
-	}
-
-	// Get the container list
-	containers := getContainersWithProfile(d, name)
-
-	// Update the database
-	id, profile, err := dbProfileGet(d.db, name)
-	if err != nil {
-		return InternalError(fmt.Errorf("Failed to retrieve profile='%s'", name))
-	}
-
-	tx, err := dbBegin(d.db)
-	if err != nil {
-		return InternalError(err)
-	}
-
-	if profile.Description != req.Description {
-		err = dbProfileDescriptionUpdate(tx, id, req.Description)
-		if err != nil {
-			tx.Rollback()
-			return InternalError(err)
-		}
-	}
-
-	// Optimize for description-only changes
-	if reflect.DeepEqual(profile.Config, req.Config) && reflect.DeepEqual(profile.Devices, req.Devices) {
-		err = txCommit(tx)
-		if err != nil {
-			return InternalError(err)
-		}
-
-		return EmptySyncResponse
-	}
-
-	err = dbProfileConfigClear(tx, id)
-	if err != nil {
-		tx.Rollback()
-		return InternalError(err)
-	}
-
-	err = dbProfileConfigAdd(tx, id, req.Config)
-	if err != nil {
-		tx.Rollback()
-		return SmartError(err)
-	}
-
-	err = dbDevicesAdd(tx, "profile", id, req.Devices)
-	if err != nil {
-		tx.Rollback()
-		return SmartError(err)
-	}
-
-	err = txCommit(tx)
-	if err != nil {
-		return InternalError(err)
-	}
-
-	// Update all the containers using the profile. Must be done after txCommit due to DB lock.
-	failures := map[string]error{}
-	for _, c := range containers {
-		err = c.Update(containerArgs{
-			Architecture: c.Architecture(),
-			Ephemeral:    c.IsEphemeral(),
-			Config:       c.LocalConfig(),
-			Devices:      c.LocalDevices(),
-			Profiles:     c.Profiles()}, true)
-
-		if err != nil {
-			failures[c.Name()] = err
-		}
-	}
-
-	if len(failures) != 0 {
-		msg := "The following containers failed to update (profile change still saved):\n"
-		for cname, err := range failures {
-			msg += fmt.Sprintf(" - %s: %s\n", cname, err)
-		}
-		return InternalError(fmt.Errorf("%s", msg))
-	}
-
-	return EmptySyncResponse
+	return doProfileUpdate(d, name, id, profile, req)
 }
 
 // The handler for the post operation.
 func profilePost(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
 
-	req := profilesPostReq{}
+	req := api.ProfilePost{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -256,7 +170,7 @@ func profilePost(d *Daemon, r *http.Request) Response {
 	}
 
 	// Check that the name isn't already in use
-	id, _, _ := dbProfileGet(d.db, req.Name)
+	id, _, _ := db.ProfileGet(d.db, req.Name)
 	if id > 0 {
 		return Conflict
 	}
@@ -269,29 +183,29 @@ func profilePost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("Invalid profile name '%s'", req.Name))
 	}
 
-	err := dbProfileUpdate(d.db, name, req.Name)
+	err := db.ProfileUpdate(d.db, name, req.Name)
 	if err != nil {
-		return InternalError(err)
+		return SmartError(err)
 	}
 
-	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/profiles/%s", shared.APIVersion, req.Name))
+	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/profiles/%s", version.APIVersion, req.Name))
 }
 
 // The handler for the delete operation.
 func profileDelete(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
 
-	_, err := doProfileGet(d, name)
+	_, err := doProfileGet(d.State(), name)
 	if err != nil {
 		return SmartError(err)
 	}
 
-	clist := getContainersWithProfile(d, name)
+	clist := getContainersWithProfile(d.State(), d.Storage, name)
 	if len(clist) != 0 {
 		return BadRequest(fmt.Errorf("Profile is currently in use"))
 	}
 
-	err = dbProfileDelete(d.db, name)
+	err = db.ProfileDelete(d.db, name)
 	if err != nil {
 		return SmartError(err)
 	}
