@@ -8,56 +8,22 @@ import (
 	"path/filepath"
 	"reflect"
 	"syscall"
-	"time"
 
 	"github.com/gorilla/websocket"
 
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/state"
+	"github.com/lxc/lxd/lxd/types"
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd/shared/idmap"
+	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/logging"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
-/* Some interesting filesystems */
-const (
-	filesystemSuperMagicTmpfs = 0x01021994
-	filesystemSuperMagicExt4  = 0xEF53
-	filesystemSuperMagicXfs   = 0x58465342
-	filesystemSuperMagicNfs   = 0x6969
-	filesystemSuperMagicZfs   = 0x2fc12fc1
-)
-
-/*
- * filesystemDetect returns the filesystem on which
- * the passed-in path sits
- */
-func filesystemDetect(path string) (string, error) {
-	fs := syscall.Statfs_t{}
-
-	err := syscall.Statfs(path, &fs)
-	if err != nil {
-		return "", err
-	}
-
-	switch fs.Type {
-	case filesystemSuperMagicBtrfs:
-		return "btrfs", nil
-	case filesystemSuperMagicZfs:
-		return "zfs", nil
-	case filesystemSuperMagicTmpfs:
-		return "tmpfs", nil
-	case filesystemSuperMagicExt4:
-		return "ext4", nil
-	case filesystemSuperMagicXfs:
-		return "xfs", nil
-	case filesystemSuperMagicNfs:
-		return "nfs", nil
-	default:
-		shared.LogDebugf("Unknown backing filesystem type: 0x%x", fs.Type)
-		return string(fs.Type), nil
-	}
-}
-
 // storageRsyncCopy copies a directory using rsync (with the --devices option).
 func storageRsyncCopy(source string, dest string) (string, error) {
 	if err := os.MkdirAll(dest, 0755); err != nil {
@@ -69,19 +35,33 @@ func storageRsyncCopy(source string, dest string) (string, error) {
 		rsyncVerbosity = "-vi"
 	}
 
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"rsync",
 		"-a",
 		"-HAX",
+		"--sparse",
 		"--devices",
 		"--delete",
 		"--checksum",
 		"--numeric-ids",
 		rsyncVerbosity,
 		shared.AddSlash(source),
-		dest).CombinedOutput()
+		dest)
+	if err != nil {
+		runError, ok := err.(shared.RunError)
+		if ok {
+			exitError, ok := runError.Err.(*exec.ExitError)
+			if ok {
+				waitStatus := exitError.Sys().(syscall.WaitStatus)
+				if waitStatus.ExitStatus() == 24 {
+					return string(output), nil
+				}
+			}
+		}
+		return string(output), err
+	}
 
-	return string(output), err
+	return string(output), nil
 }
 
 // storageType defines the type of a storage
@@ -148,8 +128,8 @@ type storage interface {
 	ContainerCanRestore(container container, sourceContainer container) error
 	ContainerDelete(container container) error
 	ContainerCopy(container container, sourceContainer container) error
-	ContainerStart(container container) error
-	ContainerStop(container container) error
+	ContainerStart(name string, path string) error
+	ContainerStop(name string, path string) error
 	ContainerRename(container container, newName string) error
 	ContainerRestore(container container, sourceContainer container) error
 	ContainerSetQuota(container container, size int64) error
@@ -192,67 +172,78 @@ type storage interface {
 	// already present on the target instance as an exercise for the
 	// enterprising developer.
 	MigrationSource(container container) (MigrationStorageSourceDriver, error)
-	MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error
+	MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error
 }
 
 func newStorage(d *Daemon, sType storageType) (storage, error) {
 	var nilmap map[string]interface{}
-	return newStorageWithConfig(d, sType, nilmap)
+	return newStorageWithConfig(d.State(), d.Storage, sType, nilmap)
 }
 
-func newStorageWithConfig(d *Daemon, sType storageType, config map[string]interface{}) (storage, error) {
-	if d.MockMode {
-		return d.Storage, nil
+func newStorageWithConfig(s *state.State, st storage, sType storageType, config map[string]interface{}) (storage, error) {
+	if s.OS.MockMode {
+		return st, nil
 	}
 
-	var s storage
+	shared := storageShared{s: s}
+	var w storage
 
 	switch sType {
 	case storageTypeBtrfs:
-		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeBtrfs {
-			return d.Storage, nil
+		if st != nil && st.GetStorageType() == storageTypeBtrfs {
+			return st, nil
 		}
-
-		s = &storageLogWrapper{w: &storageBtrfs{d: d}}
+		btrfs := &storageBtrfs{storageShared: shared}
+		w = &storageLogWrapper{w: btrfs}
+		btrfs.storage = w
 	case storageTypeZfs:
-		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeZfs {
-			return d.Storage, nil
+		if st != nil && st.GetStorageType() == storageTypeZfs {
+			return st, nil
 		}
-
-		s = &storageLogWrapper{w: &storageZfs{d: d}}
+		zfs := &storageZfs{storageShared: shared}
+		w = &storageLogWrapper{w: zfs}
+		zfs.storage = w
 	case storageTypeLvm:
-		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeLvm {
-			return d.Storage, nil
+		if st != nil && st.GetStorageType() == storageTypeLvm {
+			return st, nil
 		}
-
-		s = &storageLogWrapper{w: &storageLvm{d: d}}
+		lvm := &storageLvm{storageShared: shared}
+		w = &storageLogWrapper{w: lvm}
+		lvm.storage = w
 	default:
-		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeDir {
-			return d.Storage, nil
+		if st != nil && st.GetStorageType() == storageTypeDir {
+			return st, nil
 		}
-
-		s = &storageLogWrapper{w: &storageDir{d: d}}
+		dir := &storageDir{storageShared: shared}
+		w = &storageLogWrapper{w: dir}
+		dir.storage = w
 	}
 
-	return s.Init(config)
+	return w.Init(config)
 }
 
-func storageForFilename(d *Daemon, filename string) (storage, error) {
+func storageForFilename(s *state.State, storage storage, filename string) (storage, error) {
 	var filesystem string
 	var err error
 
 	config := make(map[string]interface{})
 	storageType := storageTypeDir
 
-	if d.MockMode {
-		return newStorageWithConfig(d, storageTypeMock, config)
+	if s.OS.MockMode {
+		return newStorageWithConfig(s, storage, storageTypeMock, config)
 	}
 
 	if shared.PathExists(filename) {
-		filesystem, err = filesystemDetect(filename)
+		filesystem, err = util.FilesystemDetect(filename)
 		if err != nil {
 			return nil, fmt.Errorf("couldn't detect filesystem for '%s': %v", filename, err)
 		}
+
+		if filesystem == "btrfs" {
+			if !(*storageBtrfs).isSubvolume(nil, filename) {
+				filesystem = ""
+			}
+		}
 	}
 
 	if shared.PathExists(filename + ".lv") {
@@ -269,12 +260,12 @@ func storageForFilename(d *Daemon, filename string) (storage, error) {
 		storageType = storageTypeBtrfs
 	}
 
-	return newStorageWithConfig(d, storageType, config)
+	return newStorageWithConfig(s, storage, storageType, config)
 }
 
-func storageForImage(d *Daemon, imgInfo *shared.ImageInfo) (storage, error) {
+func storageForImage(s *state.State, storage storage, imgInfo *api.Image) (storage, error) {
 	imageFilename := shared.VarPath("images", imgInfo.Fingerprint)
-	return storageForFilename(d, imageFilename)
+	return storageForFilename(s, storage, imageFilename)
 }
 
 type storageShared struct {
@@ -282,12 +273,16 @@ type storageShared struct {
 	sTypeName    string
 	sTypeVersion string
 
-	log shared.Logger
+	s *state.State
+
+	storage storage
+
+	log logger.Logger
 }
 
 func (ss *storageShared) initShared() error {
 	ss.log = logging.AddContext(
-		shared.Log,
+		logger.Log,
 		log.Ctx{"driver": fmt.Sprintf("storage/%s", ss.sTypeName)},
 	)
 	return nil
@@ -309,18 +304,21 @@ func (ss *storageShared) shiftRootfs(c container) error {
 	dpath := c.Path()
 	rpath := c.RootfsPath()
 
-	shared.LogDebug("Shifting root filesystem",
+	logger.Debug("Shifting root filesystem",
 		log.Ctx{"container": c.Name(), "rootfs": rpath})
 
-	idmapset := c.IdmapSet()
+	idmapset, err := c.IdmapSet()
+	if err != nil {
+		return err
+	}
 
 	if idmapset == nil {
 		return fmt.Errorf("IdmapSet of container '%s' is nil", c.Name())
 	}
 
-	err := idmapset.ShiftRootfs(rpath)
+	err = idmapset.ShiftRootfs(rpath)
 	if err != nil {
-		shared.LogDebugf("Shift of rootfs %s failed: %s", rpath, err)
+		logger.Debugf("Shift of rootfs %s failed: %s", rpath, err)
 		return err
 	}
 
@@ -331,7 +329,10 @@ func (ss *storageShared) shiftRootfs(c container) error {
 }
 
 func (ss *storageShared) setUnprivUserAcl(c container, destPath string) error {
-	idmapset := c.IdmapSet()
+	idmapset, err := c.IdmapSet()
+	if err != nil {
+		return err
+	}
 
 	// Skip for privileged containers
 	if idmapset == nil {
@@ -349,9 +350,9 @@ func (ss *storageShared) setUnprivUserAcl(c container, destPath string) error {
 
 	// Attempt to set a POSIX ACL first. Fallback to chmod if the fs doesn't support it.
 	acl := fmt.Sprintf("%d:rx", uid)
-	_, err := exec.Command("setfacl", "-m", acl, destPath).CombinedOutput()
+	_, err = shared.RunCommand("setfacl", "-m", acl, destPath)
 	if err != nil {
-		_, err := exec.Command("chmod", "+x", destPath).CombinedOutput()
+		_, err := shared.RunCommand("chmod", "+x", destPath)
 		if err != nil {
 			return fmt.Errorf("Failed to chmod the container path.")
 		}
@@ -362,13 +363,13 @@ func (ss *storageShared) setUnprivUserAcl(c container, destPath string) error {
 
 type storageLogWrapper struct {
 	w   storage
-	log shared.Logger
+	log logger.Logger
 }
 
 func (lw *storageLogWrapper) Init(config map[string]interface{}) (storage, error) {
 	_, err := lw.w.Init(config)
 	lw.log = logging.AddContext(
-		shared.Log,
+		logger.Log,
 		log.Ctx{"driver": fmt.Sprintf("storage/%s", lw.w.GetStorageTypeName())},
 	)
 
@@ -430,14 +431,14 @@ func (lw *storageLogWrapper) ContainerCopy(
 	return lw.w.ContainerCopy(container, sourceContainer)
 }
 
-func (lw *storageLogWrapper) ContainerStart(container container) error {
-	lw.log.Debug("ContainerStart", log.Ctx{"container": container.Name()})
-	return lw.w.ContainerStart(container)
+func (lw *storageLogWrapper) ContainerStart(name string, path string) error {
+	lw.log.Debug("ContainerStart", log.Ctx{"container": name})
+	return lw.w.ContainerStart(name, path)
 }
 
-func (lw *storageLogWrapper) ContainerStop(container container) error {
-	lw.log.Debug("ContainerStop", log.Ctx{"container": container.Name()})
-	return lw.w.ContainerStop(container)
+func (lw *storageLogWrapper) ContainerStop(name string, path string) error {
+	lw.log.Debug("ContainerStop", log.Ctx{"container": name})
+	return lw.w.ContainerStop(name, path)
 }
 
 func (lw *storageLogWrapper) ContainerRename(
@@ -556,7 +557,7 @@ func (lw *storageLogWrapper) MigrationSource(container container) (MigrationStor
 	return lw.w.MigrationSource(container)
 }
 
-func (lw *storageLogWrapper) MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
+func (lw *storageLogWrapper) MigrationSink(live bool, container container, objects []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
 	objNames := []string{}
 	for _, obj := range objects {
 		objNames = append(objNames, obj.GetName())
@@ -572,10 +573,14 @@ func (lw *storageLogWrapper) MigrationSink(live bool, container container, objec
 	return lw.w.MigrationSink(live, container, objects, conn, srcIdmap)
 }
 
-func ShiftIfNecessary(container container, srcIdmap *shared.IdmapSet) error {
-	dstIdmap := container.IdmapSet()
+func ShiftIfNecessary(container container, srcIdmap *idmap.IdmapSet) error {
+	dstIdmap, err := container.IdmapSet()
+	if err != nil {
+		return err
+	}
+
 	if dstIdmap == nil {
-		dstIdmap = new(shared.IdmapSet)
+		dstIdmap = new(idmap.IdmapSet)
 	}
 
 	if !reflect.DeepEqual(srcIdmap, dstIdmap) {
@@ -609,6 +614,7 @@ func (s rsyncStorageSourceDriver) Snapshots() []container {
 }
 
 func (s rsyncStorageSourceDriver) SendWhileRunning(conn *websocket.Conn) error {
+	ctName, _, _ := containerGetParentAndSnapshotName(s.container.Name())
 	for _, send := range s.snapshots {
 		if err := send.StorageStart(); err != nil {
 			return err
@@ -616,17 +622,19 @@ func (s rsyncStorageSourceDriver) SendWhileRunning(conn *websocket.Conn) error {
 		defer send.StorageStop()
 
 		path := send.Path()
-		if err := RsyncSend(shared.AddSlash(path), conn); err != nil {
+		if err := RsyncSend(ctName, shared.AddSlash(path), conn); err != nil {
 			return err
 		}
 	}
 
-	return RsyncSend(shared.AddSlash(s.container.Path()), conn)
+	return RsyncSend(ctName, shared.AddSlash(s.container.Path()), conn)
 }
 
 func (s rsyncStorageSourceDriver) SendAfterCheckpoint(conn *websocket.Conn) error {
+	ctName, _, _ := containerGetParentAndSnapshotName(s.container.Name())
+
 	/* resync anything that changed between our first send and the checkpoint */
-	return RsyncSend(shared.AddSlash(s.container.Path()), conn)
+	return RsyncSend(ctName, shared.AddSlash(s.container.Path()), conn)
 }
 
 func (s rsyncStorageSourceDriver) Cleanup() {
@@ -642,14 +650,14 @@ func rsyncMigrationSource(container container) (MigrationStorageSourceDriver, er
 	return rsyncStorageSourceDriver{container, snapshots}, nil
 }
 
-func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) containerArgs {
+func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) db.ContainerArgs {
 	config := map[string]string{}
 
 	for _, ent := range snap.LocalConfig {
 		config[ent.GetKey()] = ent.GetValue()
 	}
 
-	devices := shared.Devices{}
+	devices := types.Devices{}
 	for _, ent := range snap.LocalDevices {
 		props := map[string]string{}
 		for _, prop := range ent.Config {
@@ -660,9 +668,9 @@ func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) conta
 	}
 
 	name := containerName + shared.SnapshotDelimiter + snap.GetName()
-	return containerArgs{
+	return db.ContainerArgs{
 		Name:         name,
-		Ctype:        cTypeSnapshot,
+		Ctype:        db.CTypeSnapshot,
 		Config:       config,
 		Profiles:     snap.Profiles,
 		Ephemeral:    snap.GetEphemeral(),
@@ -672,7 +680,7 @@ func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) conta
 	}
 }
 
-func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
+func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
 	isDirBackend := container.Storage().GetStorageType() == storageTypeDir
 
 	if isDirBackend {
@@ -684,7 +692,7 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 		}
 		for _, snap := range snapshots {
 			args := snapshotProtobufToContainerArgs(container.Name(), snap)
-			s, err := containerCreateEmptySnapshot(container.Daemon(), args)
+			s, err := containerCreateEmptySnapshot(container.StateObject(), container.Storage(), args)
 			if err != nil {
 				return err
 			}
@@ -717,7 +725,7 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 			}
 
 			args := snapshotProtobufToContainerArgs(container.Name(), snap)
-			_, err := containerCreateAsSnapshot(container.Daemon(), args, container)
+			_, err := containerCreateAsSnapshot(container.StateObject(), container.Storage(), args, container)
 			if err != nil {
 				return err
 			}
@@ -742,57 +750,36 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 	return nil
 }
 
-// Useful functions for unreliable backends
-func tryExec(name string, arg ...string) ([]byte, error) {
-	var err error
-	var output []byte
-
-	for i := 0; i < 20; i++ {
-		output, err = exec.Command(name, arg...).CombinedOutput()
-		if err == nil {
-			break
-		}
-
-		time.Sleep(500 * time.Millisecond)
-	}
-
-	return output, err
-}
-
-func tryMount(src string, dst string, fs string, flags uintptr, options string) error {
+func SetupStorageDriver(d *Daemon) error {
 	var err error
 
-	for i := 0; i < 20; i++ {
-		err = syscall.Mount(src, dst, fs, flags, options)
-		if err == nil {
-			break
+	lvmVgName := daemonConfig["storage.lvm_vg_name"].Get()
+	zfsPoolName := daemonConfig["storage.zfs_pool_name"].Get()
+
+	if lvmVgName != "" {
+		d.Storage, err = newStorage(d, storageTypeLvm)
+		if err != nil {
+			logger.Errorf("Could not initialize storage type LVM: %s - falling back to dir", err)
+		} else {
+			return nil
 		}
-
-		time.Sleep(500 * time.Millisecond)
-	}
-
-	if err != nil {
-		return err
-	}
-
-	return nil
-}
-
-func tryUnmount(path string, flags int) error {
-	var err error
-
-	for i := 0; i < 20; i++ {
-		err = syscall.Unmount(path, flags)
-		if err == nil {
-			break
+	} else if zfsPoolName != "" {
+		d.Storage, err = newStorage(d, storageTypeZfs)
+		if err != nil {
+			logger.Errorf("Could not initialize storage type ZFS: %s - falling back to dir", err)
+		} else {
+			return nil
+		}
+	} else if d.os.BackingFS == "btrfs" {
+		d.Storage, err = newStorage(d, storageTypeBtrfs)
+		if err != nil {
+			logger.Errorf("Could not initialize storage type btrfs: %s - falling back to dir", err)
+		} else {
+			return nil
 		}
-
-		time.Sleep(500 * time.Millisecond)
 	}
 
-	if err != nil {
-		return err
-	}
+	d.Storage, err = newStorage(d, storageTypeDir)
 
-	return nil
+	return err
 }
