@@ -1,6 +1,7 @@
 package main
 
 import (
+	"database/sql"
 	"fmt"
 	"io/ioutil"
 	"os"
@@ -12,15 +13,18 @@ import (
 
 	"github.com/gorilla/websocket"
 
+	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/idmap"
+	"github.com/lxc/lxd/shared/logger"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 func storageLVMCheckVolumeGroup(vgName string) error {
-	output, err := exec.Command("vgdisplay", "-s", vgName).CombinedOutput()
+	output, err := shared.RunCommand("vgdisplay", "-s", vgName)
 	if err != nil {
-		shared.LogDebug("vgdisplay failed to find vg", log.Ctx{"output": string(output)})
+		logger.Debug("vgdisplay failed to find vg", log.Ctx{"output": string(output)})
 		return fmt.Errorf("LVM volume group '%s' not found", vgName)
 	}
 
@@ -48,14 +52,14 @@ func storageLVMThinpoolExists(vgName string, poolName string) (bool, error) {
 	return false, fmt.Errorf("Pool named '%s' exists but is not a thin pool.", poolName)
 }
 
-func storageLVMGetThinPoolUsers(d *Daemon) ([]string, error) {
+func storageLVMGetThinPoolUsers(dbObj *sql.DB) ([]string, error) {
 	results := []string{}
 
 	if daemonConfig["storage.lvm_vg_name"].Get() == "" {
 		return results, nil
 	}
 
-	cNames, err := dbContainersList(d.db, cTypeRegular)
+	cNames, err := db.ContainersList(dbObj, db.CTypeRegular)
 	if err != nil {
 		return results, err
 	}
@@ -73,7 +77,7 @@ func storageLVMGetThinPoolUsers(d *Daemon) ([]string, error) {
 		}
 	}
 
-	imageNames, err := dbImagesGet(d.db, false)
+	imageNames, err := db.ImagesGet(dbObj, false)
 	if err != nil {
 		return results, err
 	}
@@ -89,7 +93,11 @@ func storageLVMGetThinPoolUsers(d *Daemon) ([]string, error) {
 }
 
 func storageLVMValidateThinPoolName(d *Daemon, key string, value string) error {
-	users, err := storageLVMGetThinPoolUsers(d)
+	return doStorageLVMValidateThinPoolName(d.db, key, value)
+}
+
+func doStorageLVMValidateThinPoolName(db *sql.DB, key string, value string) error {
+	users, err := storageLVMGetThinPoolUsers(db)
 	if err != nil {
 		return fmt.Errorf("Error checking if a pool is already in use: %v", err)
 	}
@@ -118,7 +126,7 @@ func storageLVMValidateThinPoolName(d *Daemon, key string, value string) error {
 }
 
 func storageLVMValidateVolumeGroupName(d *Daemon, key string, value string) error {
-	users, err := storageLVMGetThinPoolUsers(d)
+	users, err := storageLVMGetThinPoolUsers(d.db)
 	if err != nil {
 		return fmt.Errorf("Error checking if a pool is already in use: %v", err)
 	}
@@ -138,10 +146,10 @@ func storageLVMValidateVolumeGroupName(d *Daemon, key string, value string) erro
 }
 
 func xfsGenerateNewUUID(lvpath string) error {
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"xfs_admin",
 		"-U", "generate",
-		lvpath).CombinedOutput()
+		lvpath)
 	if err != nil {
 		return fmt.Errorf("Error generating new UUID: %v\noutput:'%s'", err, string(output))
 	}
@@ -155,7 +163,6 @@ func containerNameToLVName(containerName string) string {
 }
 
 type storageLvm struct {
-	d      *Daemon
 	vgName string
 
 	storageShared
@@ -168,7 +175,7 @@ func (s *storageLvm) Init(config map[string]interface{}) (storage, error) {
 		return s, err
 	}
 
-	output, err := exec.Command("lvm", "version").CombinedOutput()
+	output, err := shared.RunCommand("lvm", "version")
 	if err != nil {
 		return nil, fmt.Errorf("Error getting LVM version: %v\noutput:'%s'", err, string(output))
 	}
@@ -403,7 +410,7 @@ func (s *storageLvm) ContainerCopy(container container, sourceContainer containe
 			return err
 		}
 
-		if err := s.ContainerStart(container); err != nil {
+		if err := s.ContainerStart(container.Name(), container.Path()); err != nil {
 			s.log.Error("Error starting/mounting container", log.Ctx{"err": err, "container": container.Name()})
 			s.ContainerDelete(container)
 			return err
@@ -418,35 +425,35 @@ func (s *storageLvm) ContainerCopy(container container, sourceContainer containe
 			return fmt.Errorf("rsync failed: %s", string(output))
 		}
 
-		if err := s.ContainerStop(container); err != nil {
+		if err := s.ContainerStop(container.Name(), container.Path()); err != nil {
 			return err
 		}
 	}
 	return container.TemplateApply("copy")
 }
 
-func (s *storageLvm) ContainerStart(container container) error {
-	lvName := containerNameToLVName(container.Name())
+func (s *storageLvm) ContainerStart(name string, path string) error {
+	lvName := containerNameToLVName(name)
 	lvpath := fmt.Sprintf("/dev/%s/%s", s.vgName, lvName)
 	fstype := daemonConfig["storage.lvm_fstype"].Get()
 
-	err := tryMount(lvpath, container.Path(), fstype, 0, "discard")
+	err := tryMount(lvpath, path, fstype, 0, "discard")
 	if err != nil {
 		return fmt.Errorf(
 			"Error mounting snapshot LV path='%s': %v",
-			container.Path(),
+			path,
 			err)
 	}
 
 	return nil
 }
 
-func (s *storageLvm) ContainerStop(container container) error {
-	err := tryUnmount(container.Path(), 0)
+func (s *storageLvm) ContainerStop(name string, path string) error {
+	err := tryUnmount(path, 0)
 	if err != nil {
 		return fmt.Errorf(
 			"failed to unmount container path '%s'.\nError: %v",
-			container.Path(),
+			path,
 			err)
 	}
 
@@ -557,7 +564,7 @@ func (s *storageLvm) createSnapshotContainer(
 
 	srcName := containerNameToLVName(sourceContainer.Name())
 	destName := containerNameToLVName(snapshotContainer.Name())
-	shared.LogDebug(
+	logger.Debug(
 		"Creating snapshot",
 		log.Ctx{"srcName": srcName, "destName": destName})
 
@@ -649,7 +656,7 @@ func (s *storageLvm) ContainerSnapshotStart(container container) error {
 	srcName := containerNameToLVName(container.Name())
 	destName := containerNameToLVName(container.Name() + "/rw")
 
-	shared.LogDebug(
+	logger.Debug(
 		"Creating snapshot",
 		log.Ctx{"srcName": srcName, "destName": destName})
 
@@ -687,7 +694,7 @@ func (s *storageLvm) ContainerSnapshotStart(container container) error {
 }
 
 func (s *storageLvm) ContainerSnapshotStop(container container) error {
-	err := s.ContainerStop(container)
+	err := s.ContainerStop(container.Name(), container.Path())
 	if err != nil {
 		return err
 	}
@@ -732,11 +739,11 @@ func (s *storageLvm) ImageCreate(fingerprint string) error {
 	fstype := daemonConfig["storage.lvm_fstype"].Get()
 	err = tryMount(lvpath, tempLVMountPoint, fstype, 0, "discard")
 	if err != nil {
-		shared.LogInfof("Error mounting image LV for unpacking: %v", err)
+		logger.Infof("Error mounting image LV for unpacking: %v", err)
 		return fmt.Errorf("Error mounting image LV: %v", err)
 	}
 
-	unpackErr := unpackImage(s.d, finalName, tempLVMountPoint)
+	unpackErr := unpackImage(finalName, tempLVMountPoint, s.storage.GetStorageType())
 
 	err = tryUnmount(tempLVMountPoint, 0)
 	if err != nil {
@@ -784,16 +791,16 @@ func (s *storageLvm) createDefaultThinPool() (string, error) {
 	}
 
 	// Create the thin pool
-	var output []byte
+	var output string
 	if isRecent {
-		output, err = tryExec(
+		output, err = shared.TryRunCommand(
 			"lvcreate",
 			"--poolmetadatasize", "1G",
 			"-l", "100%FREE",
 			"--thinpool",
 			fmt.Sprintf("%s/%s", s.vgName, thinPoolName))
 	} else {
-		output, err = tryExec(
+		output, err = shared.TryRunCommand(
 			"lvcreate",
 			"--poolmetadatasize", "1G",
 			"-L", "1G",
@@ -815,7 +822,7 @@ func (s *storageLvm) createDefaultThinPool() (string, error) {
 
 	if !isRecent {
 		// Grow it to the maximum VG size (two step process required by old LVM)
-		output, err = tryExec(
+		output, err = shared.TryRunCommand(
 			"lvextend",
 			"--alloc", "anywhere",
 			"-l", "100%FREE",
@@ -853,7 +860,7 @@ func (s *storageLvm) createThinLV(lvname string) (string, error) {
 			return "", fmt.Errorf("Error creating LVM thin pool: %v", err)
 		}
 
-		err = storageLVMValidateThinPoolName(s.d, "", poolname)
+		err = doStorageLVMValidateThinPoolName(s.s.DB, "", poolname)
 		if err != nil {
 			s.log.Error("Setting thin pool name", log.Ctx{"err": err})
 			return "", fmt.Errorf("Error setting LVM thin pool config: %v", err)
@@ -862,7 +869,7 @@ func (s *storageLvm) createThinLV(lvname string) (string, error) {
 
 	lvSize := daemonConfig["storage.lvm_volume_size"].Get()
 
-	output, err := tryExec(
+	output, err := shared.TryRunCommand(
 		"lvcreate",
 		"--thin",
 		"-n", lvname,
@@ -878,12 +885,12 @@ func (s *storageLvm) createThinLV(lvname string) (string, error) {
 	fstype := daemonConfig["storage.lvm_fstype"].Get()
 	switch fstype {
 	case "xfs":
-		output, err = tryExec(
+		output, err = shared.TryRunCommand(
 			"mkfs.xfs",
 			lvpath)
 	default:
 		// default = ext4
-		output, err = tryExec(
+		output, err = shared.TryRunCommand(
 			"mkfs.ext4",
 			"-E", "nodiscard,lazy_itable_init=0,lazy_journal_init=0",
 			lvpath)
@@ -899,9 +906,9 @@ func (s *storageLvm) createThinLV(lvname string) (string, error) {
 
 func (s *storageLvm) removeLV(lvname string) error {
 	var err error
-	var output []byte
+	var output string
 
-	output, err = tryExec(
+	output, err = shared.TryRunCommand(
 		"lvremove", "-f", fmt.Sprintf("%s/%s", s.vgName, lvname))
 
 	if err != nil {
@@ -918,15 +925,16 @@ func (s *storageLvm) createSnapshotLV(lvname string, origlvname string, readonly
 	if err != nil {
 		return "", fmt.Errorf("Error checking LVM version: %v", err)
 	}
-	var output []byte
+
+	var output string
 	if isRecent {
-		output, err = tryExec(
+		output, err = shared.TryRunCommand(
 			"lvcreate",
 			"-kn",
 			"-n", lvname,
 			"-s", fmt.Sprintf("/dev/%s/%s", s.vgName, origlvname))
 	} else {
-		output, err = tryExec(
+		output, err = shared.TryRunCommand(
 			"lvcreate",
 			"-n", lvname,
 			"-s", fmt.Sprintf("/dev/%s/%s", s.vgName, origlvname))
@@ -939,9 +947,9 @@ func (s *storageLvm) createSnapshotLV(lvname string, origlvname string, readonly
 	snapshotFullName := fmt.Sprintf("/dev/%s/%s", s.vgName, lvname)
 
 	if readonly {
-		output, err = tryExec("lvchange", "-ay", "-pr", snapshotFullName)
+		output, err = shared.TryRunCommand("lvchange", "-ay", "-pr", snapshotFullName)
 	} else {
-		output, err = tryExec("lvchange", "-ay", snapshotFullName)
+		output, err = shared.TryRunCommand("lvchange", "-ay", snapshotFullName)
 	}
 
 	if err != nil {
@@ -956,7 +964,7 @@ func (s *storageLvm) isLVMContainer(container container) bool {
 }
 
 func (s *storageLvm) renameLV(oldName string, newName string) (string, error) {
-	output, err := tryExec("lvrename", s.vgName, oldName, newName)
+	output, err := shared.TryRunCommand("lvrename", s.vgName, oldName, newName)
 	return string(output), err
 }
 
@@ -972,6 +980,6 @@ func (s *storageLvm) MigrationSource(container container) (MigrationStorageSourc
 	return rsyncMigrationSource(container)
 }
 
-func (s *storageLvm) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
+func (s *storageLvm) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
 	return rsyncMigrationSink(live, container, snapshots, conn, srcIdmap)
 }
