@@ -12,14 +12,16 @@ import (
 
 	"github.com/gorilla/websocket"
 
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/idmap"
+	"github.com/lxc/lxd/shared/logger"
 
 	"github.com/pborman/uuid"
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 type storageZfs struct {
-	d       *Daemon
 	zfsPool string
 
 	storageShared
@@ -53,10 +55,10 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 	err = s.zfsCheckPool(s.zfsPool)
 	if err != nil {
 		if shared.PathExists(shared.VarPath("zfs.img")) {
-			_ = loadModule("zfs")
+			_ = util.LoadModule("zfs")
 
-			output, err := exec.Command("zpool", "import",
-				"-d", shared.VarPath(), s.zfsPool).CombinedOutput()
+			output, err := shared.RunCommand("zpool", "import",
+				"-d", shared.VarPath(), s.zfsPool)
 			if err != nil {
 				return s, fmt.Errorf("Unable to import the ZFS pool: %s", output)
 			}
@@ -65,7 +67,7 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 		}
 	}
 
-	output, err := exec.Command("zfs", "get", "version", "-H", "-o", "value", s.zfsPool).CombinedOutput()
+	output, err := shared.RunCommand("zfs", "get", "version", "-H", "-o", "value", s.zfsPool)
 	if err != nil {
 		return s, fmt.Errorf("The 'zfs' tool isn't working properly")
 	}
@@ -75,13 +77,13 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 		return s, fmt.Errorf("The 'zfs' tool isn't working properly")
 	}
 
-	output, err = exec.Command("zfs", "get", "mountpoint", "-H", "-o", "source", s.zfsPool).CombinedOutput()
+	output, err = shared.RunCommand("zfs", "get", "mountpoint", "-H", "-o", "source", s.zfsPool)
 	if err != nil {
 		return s, fmt.Errorf("Unable to query ZFS mountpoint")
 	}
 
 	if strings.TrimSpace(string(output)) != "local" {
-		err = shared.RunCommand("zfs", "set", "mountpoint=none", s.zfsPool)
+		_, err = shared.RunCommand("zfs", "set", "mountpoint=none", s.zfsPool)
 		if err != nil {
 			return s, err
 		}
@@ -91,8 +93,8 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 }
 
 // Things we don't need to care about
-func (s *storageZfs) ContainerStart(container container) error {
-	fs := fmt.Sprintf("containers/%s", container.Name())
+func (s *storageZfs) ContainerStart(name string, path string) error {
+	fs := fmt.Sprintf("containers/%s", name)
 
 	// Just in case the container filesystem got unmounted
 	if !shared.IsMountPoint(shared.VarPath(fs)) {
@@ -102,7 +104,7 @@ func (s *storageZfs) ContainerStart(container container) error {
 	return nil
 }
 
-func (s *storageZfs) ContainerStop(container container) error {
+func (s *storageZfs) ContainerStop(name string, path string) error {
 	return nil
 }
 
@@ -628,7 +630,7 @@ func (s *storageZfs) ImageCreate(fingerprint string) error {
 		return err
 	}
 
-	err = unpackImage(s.d, imagePath, subvol)
+	err = unpackImage(imagePath, subvol, s.storage.GetStorageType())
 	if err != nil {
 		return cleanup(err)
 	}
@@ -685,8 +687,8 @@ func (s *storageZfs) ImageDelete(fingerprint string) error {
 
 // Helper functions
 func (s *storageZfs) zfsCheckPool(pool string) error {
-	output, err := exec.Command(
-		"zfs", "get", "type", "-H", "-o", "value", pool).CombinedOutput()
+	output, err := shared.RunCommand(
+		"zfs", "get", "type", "-H", "-o", "value", pool)
 	if err != nil {
 		return fmt.Errorf(strings.Split(string(output), "\n")[0])
 	}
@@ -707,13 +709,13 @@ func (s *storageZfs) zfsClone(source string, name string, dest string, dotZfs bo
 		mountpoint += ".zfs"
 	}
 
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"zfs",
 		"clone",
 		"-p",
 		"-o", fmt.Sprintf("mountpoint=%s", mountpoint),
 		fmt.Sprintf("%s/%s@%s", s.zfsPool, source, name),
-		fmt.Sprintf("%s/%s", s.zfsPool, dest)).CombinedOutput()
+		fmt.Sprintf("%s/%s", s.zfsPool, dest))
 	if err != nil {
 		s.log.Error("zfs clone failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to clone the filesystem: %s", output)
@@ -740,13 +742,13 @@ func (s *storageZfs) zfsClone(source string, name string, dest string, dotZfs bo
 			mountpoint += ".zfs"
 		}
 
-		output, err := exec.Command(
+		output, err := shared.RunCommand(
 			"zfs",
 			"clone",
 			"-p",
 			"-o", fmt.Sprintf("mountpoint=%s", mountpoint),
 			fmt.Sprintf("%s/%s@%s", s.zfsPool, sub, name),
-			fmt.Sprintf("%s/%s", s.zfsPool, destSubvol)).CombinedOutput()
+			fmt.Sprintf("%s/%s", s.zfsPool, destSubvol))
 		if err != nil {
 			s.log.Error("zfs clone failed", log.Ctx{"output": string(output)})
 			return fmt.Errorf("Failed to clone the sub-volume: %s", output)
@@ -757,12 +759,12 @@ func (s *storageZfs) zfsClone(source string, name string, dest string, dotZfs bo
 }
 
 func (s *storageZfs) zfsCreate(path string) error {
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"zfs",
 		"create",
 		"-p",
 		"-o", fmt.Sprintf("mountpoint=%s.zfs", shared.VarPath(path)),
-		fmt.Sprintf("%s/%s", s.zfsPool, path)).CombinedOutput()
+		fmt.Sprintf("%s/%s", s.zfsPool, path))
 	if err != nil {
 		s.log.Error("zfs create failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to create ZFS filesystem: %s", output)
@@ -786,7 +788,7 @@ func (s *storageZfs) zfsDestroy(path string) error {
 	}
 
 	// Due to open fds or kernel refs, this may fail for a bit, give it 10s
-	output, err := tryExec(
+	output, err := shared.TryRunCommand(
 		"zfs",
 		"destroy",
 		"-r",
@@ -876,14 +878,14 @@ func (s *storageZfs) zfsExists(path string) bool {
 }
 
 func (s *storageZfs) zfsGet(path string, key string) (string, error) {
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"zfs",
 		"get",
 		"-H",
 		"-p",
 		"-o", "value",
 		key,
-		fmt.Sprintf("%s/%s", s.zfsPool, path)).CombinedOutput()
+		fmt.Sprintf("%s/%s", s.zfsPool, path))
 	if err != nil {
 		return string(output), fmt.Errorf("Failed to get ZFS config: %s", output)
 	}
@@ -893,15 +895,15 @@ func (s *storageZfs) zfsGet(path string, key string) (string, error) {
 
 func (s *storageZfs) zfsRename(source string, dest string) error {
 	var err error
-	var output []byte
+	var output string
 
 	for i := 0; i < 20; i++ {
-		output, err = exec.Command(
+		output, err = shared.RunCommand(
 			"zfs",
 			"rename",
 			"-p",
 			fmt.Sprintf("%s/%s", s.zfsPool, source),
-			fmt.Sprintf("%s/%s", s.zfsPool, dest)).CombinedOutput()
+			fmt.Sprintf("%s/%s", s.zfsPool, dest))
 
 		// Success
 		if err == nil {
@@ -922,11 +924,11 @@ func (s *storageZfs) zfsRename(source string, dest string) error {
 }
 
 func (s *storageZfs) zfsSet(path string, key string, value string) error {
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"zfs",
 		"set",
 		fmt.Sprintf("%s=%s", key, value),
-		fmt.Sprintf("%s/%s", s.zfsPool, path)).CombinedOutput()
+		fmt.Sprintf("%s/%s", s.zfsPool, path))
 	if err != nil {
 		s.log.Error("zfs set failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to set ZFS config: %s", output)
@@ -936,11 +938,11 @@ func (s *storageZfs) zfsSet(path string, key string, value string) error {
 }
 
 func (s *storageZfs) zfsSnapshotCreate(path string, name string) error {
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"zfs",
 		"snapshot",
 		"-r",
-		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name)).CombinedOutput()
+		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name))
 	if err != nil {
 		s.log.Error("zfs snapshot failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to create ZFS snapshot: %s", output)
@@ -950,11 +952,11 @@ func (s *storageZfs) zfsSnapshotCreate(path string, name string) error {
 }
 
 func (s *storageZfs) zfsSnapshotDestroy(path string, name string) error {
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"zfs",
 		"destroy",
 		"-r",
-		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name)).CombinedOutput()
+		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name))
 	if err != nil {
 		s.log.Error("zfs destroy failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to destroy ZFS snapshot: %s", output)
@@ -964,7 +966,7 @@ func (s *storageZfs) zfsSnapshotDestroy(path string, name string) error {
 }
 
 func (s *storageZfs) zfsSnapshotRestore(path string, name string) error {
-	output, err := tryExec(
+	output, err := shared.TryRunCommand(
 		"zfs",
 		"rollback",
 		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name))
@@ -988,7 +990,7 @@ func (s *storageZfs) zfsSnapshotRestore(path string, name string) error {
 			continue
 		}
 
-		output, err := tryExec(
+		output, err := shared.TryRunCommand(
 			"zfs",
 			"rollback",
 			fmt.Sprintf("%s/%s@%s", s.zfsPool, sub, name))
@@ -1002,12 +1004,12 @@ func (s *storageZfs) zfsSnapshotRestore(path string, name string) error {
 }
 
 func (s *storageZfs) zfsSnapshotRename(path string, oldName string, newName string) error {
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"zfs",
 		"rename",
 		"-r",
 		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, oldName),
-		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, newName)).CombinedOutput()
+		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, newName))
 	if err != nil {
 		s.log.Error("zfs snapshot rename failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to rename ZFS snapshot: %s", output)
@@ -1017,7 +1019,7 @@ func (s *storageZfs) zfsSnapshotRename(path string, oldName string, newName stri
 }
 
 func (s *storageZfs) zfsMount(path string) error {
-	output, err := tryExec(
+	output, err := shared.TryRunCommand(
 		"zfs",
 		"mount",
 		fmt.Sprintf("%s/%s", s.zfsPool, path))
@@ -1030,7 +1032,7 @@ func (s *storageZfs) zfsMount(path string) error {
 }
 
 func (s *storageZfs) zfsUnmount(path string) error {
-	output, err := tryExec(
+	output, err := shared.TryRunCommand(
 		"zfs",
 		"unmount",
 		fmt.Sprintf("%s/%s", s.zfsPool, path))
@@ -1049,13 +1051,13 @@ func (s *storageZfs) zfsListSubvolumes(path string) ([]string, error) {
 		fullPath = fmt.Sprintf("%s/%s", s.zfsPool, path)
 	}
 
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"zfs",
 		"list",
 		"-t", "filesystem",
 		"-o", "name",
 		"-H",
-		"-r", fullPath).CombinedOutput()
+		"-r", fullPath)
 	if err != nil {
 		s.log.Error("zfs list failed", log.Ctx{"output": string(output)})
 		return []string{}, fmt.Errorf("Failed to list ZFS filesystems: %s", output)
@@ -1084,7 +1086,7 @@ func (s *storageZfs) zfsListSnapshots(path string) ([]string, error) {
 		fullPath = fmt.Sprintf("%s/%s", s.zfsPool, path)
 	}
 
-	output, err := exec.Command(
+	output, err := shared.RunCommand(
 		"zfs",
 		"list",
 		"-t", "snapshot",
@@ -1092,7 +1094,7 @@ func (s *storageZfs) zfsListSnapshots(path string) ([]string, error) {
 		"-H",
 		"-d", "1",
 		"-s", "creation",
-		"-r", fullPath).CombinedOutput()
+		"-r", fullPath)
 	if err != nil {
 		s.log.Error("zfs list failed", log.Ctx{"output": string(output)})
 		return []string{}, fmt.Errorf("Failed to list ZFS snapshots: %s", output)
@@ -1184,17 +1186,17 @@ func storageZFSValidatePoolName(d *Daemon, key string, value string) error {
 		if err != nil {
 			return fmt.Errorf("Invalid ZFS pool: %v", err)
 		}
-	}
 
-	// Confirm that the new pool is empty
-	s.zfsPool = value
-	subvols, err := s.zfsListSubvolumes("")
-	if err != nil {
-		return err
-	}
+		// Confirm that the new pool is empty
+		s.zfsPool = value
+		subvols, err := s.zfsListSubvolumes("")
+		if err != nil {
+			return err
+		}
 
-	if len(subvols) > 0 {
-		return fmt.Errorf("Provided ZFS pool (or dataset) isn't empty")
+		if len(subvols) > 0 {
+			return fmt.Errorf("Provided ZFS pool (or dataset) isn't empty")
+		}
 	}
 
 	// Confirm the old pool isn't in use anymore
@@ -1255,12 +1257,12 @@ func (s *zfsMigrationSourceDriver) send(conn *websocket.Conn, zfsName string, zf
 
 	output, err := ioutil.ReadAll(stderr)
 	if err != nil {
-		shared.LogError("problem reading zfs send stderr", log.Ctx{"err": err})
+		logger.Error("problem reading zfs send stderr", log.Ctx{"err": err})
 	}
 
 	err = cmd.Wait()
 	if err != nil {
-		shared.LogError("problem with zfs send", log.Ctx{"output": string(output)})
+		logger.Error("problem with zfs send", log.Ctx{"output": string(output)})
 	}
 
 	return err
@@ -1366,7 +1368,7 @@ func (s *storageZfs) MigrationSource(ct container) (MigrationStorageSourceDriver
 		}
 
 		lxdName := fmt.Sprintf("%s%s%s", ct.Name(), shared.SnapshotDelimiter, snap[len("snapshot-"):])
-		snapshot, err := containerLoadByName(s.d, lxdName)
+		snapshot, err := containerLoadByName(s.s, s.storage, lxdName)
 		if err != nil {
 			return nil, err
 		}
@@ -1378,7 +1380,7 @@ func (s *storageZfs) MigrationSource(ct container) (MigrationStorageSourceDriver
 	return &driver, nil
 }
 
-func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
+func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
 	zfsRecv := func(zfsName string) error {
 		zfsFsName := fmt.Sprintf("%s/%s", s.zfsPool, zfsName)
 		args := []string{"receive", "-F", "-u", zfsFsName}
@@ -1402,12 +1404,12 @@ func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*
 
 		output, err := ioutil.ReadAll(stderr)
 		if err != nil {
-			shared.LogDebug("problem reading zfs recv stderr %s", log.Ctx{"err": err})
+			logger.Debug("problem reading zfs recv stderr %s", log.Ctx{"err": err})
 		}
 
 		err = cmd.Wait()
 		if err != nil {
-			shared.LogError("problem with zfs recv", log.Ctx{"output": string(output)})
+			logger.Error("problem with zfs recv", log.Ctx{"output": string(output)})
 		}
 		return err
 	}
@@ -1426,7 +1428,7 @@ func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*
 
 	for _, snap := range snapshots {
 		args := snapshotProtobufToContainerArgs(container.Name(), snap)
-		_, err := containerCreateEmptySnapshot(container.Daemon(), args)
+		_, err := containerCreateEmptySnapshot(container.StateObject(), container.Storage(), args)
 		if err != nil {
 			return err
 		}
@@ -1451,7 +1453,7 @@ func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*
 		/* clean up our migration-send snapshots that we got from recv. */
 		zfsSnapshots, err := s.zfsListSnapshots(fmt.Sprintf("containers/%s", container.Name()))
 		if err != nil {
-			shared.LogError("failed listing snapshots post migration", log.Ctx{"err": err})
+			logger.Error("failed listing snapshots post migration", log.Ctx{"err": err})
 			return
 		}
 
