@@ -1,15 +1,11 @@
-#!/bin/sh
-
 btrfs_setup() {
+  # shellcheck disable=2039
   local LXD_DIR
+
   LXD_DIR=$1
 
   echo "==> Setting up btrfs backend in ${LXD_DIR}"
 
-  if ! which btrfs >/dev/null 2>&1; then
-    echo "Couldn't find the btrfs binary"; false
-  fi
-
   truncate -s 100G "${TEST_DIR}/$(basename "${LXD_DIR}").btrfs"
   mkfs.btrfs "${TEST_DIR}/$(basename "${LXD_DIR}").btrfs"
 
@@ -17,14 +13,18 @@ btrfs_setup() {
 }
 
 btrfs_configure() {
+  # shellcheck disable=2039
   local LXD_DIR
+
   LXD_DIR=$1
 
   echo "==> Configuring btrfs backend in ${LXD_DIR}"
 }
 
 btrfs_teardown() {
+  # shellcheck disable=2039
   local LXD_DIR
+
   LXD_DIR=$1
 
   echo "==> Tearing down btrfs backend in ${LXD_DIR}"
