@@ -1,15 +1,11 @@
-#!/bin/sh
-
 lvm_setup() {
+  # shellcheck disable=2039
   local LXD_DIR
+
   LXD_DIR=$1
 
   echo "==> Setting up lvm backend in ${LXD_DIR}"
 
-  if ! which lvm >/dev/null 2>&1; then
-    echo "Couldn't find the lvm binary"; false
-  fi
-
   truncate -s 4G "${TEST_DIR}/$(basename "${LXD_DIR}").lvm"
   pvloopdev=$(losetup --show -f "${TEST_DIR}/$(basename "${LXD_DIR}").lvm")
   if [ ! -e "${pvloopdev}" ]; then
@@ -23,7 +19,9 @@ lvm_setup() {
 }
 
 lvm_configure() {
+  # shellcheck disable=2039
   local LXD_DIR
+
   LXD_DIR=$1
 
   echo "==> Configuring lvm backend in ${LXD_DIR}"
@@ -33,7 +31,9 @@ lvm_configure() {
 }
 
 lvm_teardown() {
+  # shellcheck disable=2039
   local LXD_DIR
+
   LXD_DIR=$1
 
   echo "==> Tearing down lvm backend in ${LXD_DIR}"
