@@ -1,4 +1,3 @@
-#!/bin/sh
 ensure_removed() {
   bad=0
   lxc exec foo -- stat /dev/ttyS0 && bad=1
@@ -12,13 +11,13 @@ dounixdevtest() {
     lxc start foo
     lxc config device add foo tty unix-char "$@"
     lxc exec foo -- stat /dev/ttyS0
-    lxc exec foo reboot
+    lxc restart foo
     lxc exec foo -- stat /dev/ttyS0
     lxc restart foo --force
     lxc exec foo -- stat /dev/ttyS0
     lxc config device remove foo tty
     ensure_removed "was not hot-removed"
-    lxc exec foo reboot
+    lxc restart foo
     ensure_removed "removed device re-appeared after container reboot"
     lxc restart foo --force
     ensure_removed "removed device re-appaared after lxc reboot"
@@ -70,7 +69,7 @@ testloopmounts() {
   lxc exec foo stat /mnt/hello
   lxc config device remove foo mnt
   ensure_fs_unmounted "fs should have been hot-unmounted"
-  lxc exec foo reboot
+  lxc restart foo
   ensure_fs_unmounted "removed fs re-appeared after reboot"
   lxc restart foo --force
   ensure_fs_unmounted "removed fs re-appeared after restart"
@@ -79,6 +78,24 @@ testloopmounts() {
   sed -i "\|^${lpath}|d" "${TEST_DIR}/loops"
 }
 
+test_mount_order() {
+  mkdir -p "${TEST_DIR}/order/empty"
+  mkdir -p "${TEST_DIR}/order/full"
+  touch "${TEST_DIR}/order/full/filler"
+
+  # The idea here is that sometimes (depending on how golang randomizes the
+  # config) the empty dir will have the contents of full in it, but sometimes
+  # it won't depending on whether the devices below are processed in order or
+  # not. This should not be racy, and they should *always* be processed in path
+  # order, so the filler file should always be there.
+  lxc config device add foo order disk source="${TEST_DIR}/order" path=/mnt
+  lxc config device add foo orderFull disk source="${TEST_DIR}/order/full" path=/mnt/empty
+
+  lxc start foo
+  lxc exec foo -- cat /mnt/empty/filler
+  lxc stop foo --force
+}
+
 test_config_profiles() {
   ensure_import_testimage
 
@@ -90,7 +107,7 @@ test_config_profiles() {
 
   # setting an invalid config item should error out when setting it, not get
   # into the database and never let the user edit the container again.
-  ! lxc config set foo raw.lxc "lxc.notaconfigkey = invalid"
+  ! lxc config set foo raw.lxc lxc.notaconfigkey=invalid
 
   lxc profile create stdintest
   echo "BADCONF" | lxc profile set stdintest user.user_data -
@@ -119,9 +136,10 @@ test_config_profiles() {
 
   # test live-adding a nic
   lxc start foo
-  ! lxc config show foo | grep -q "raw.lxc"
+  lxc exec foo -- cat /proc/self/mountinfo | grep -q "/mnt1.*ro,"
+  ! lxc config show foo | grep -q "raw.lxc" || false
   lxc config show foo --expanded | grep -q "raw.lxc"
-  ! lxc config show foo | grep -v "volatile.eth0" | grep -q "eth0"
+  ! lxc config show foo | grep -v "volatile.eth0" | grep -q "eth0" || false
   lxc config show foo --expanded | grep -v "volatile.eth0" | grep -q "eth0"
   lxc config device add foo eth2 nic nictype=bridged parent=lxdbr0 name=eth10
   lxc exec foo -- /sbin/ifconfig -a | grep eth0
@@ -133,6 +151,7 @@ test_config_profiles() {
   mkdir "${TEST_DIR}/mnt2"
   touch "${TEST_DIR}/mnt2/hosts"
   lxc config device add foo mnt2 disk source="${TEST_DIR}/mnt2" path=/mnt2 readonly=true
+  lxc exec foo -- cat /proc/self/mountinfo | grep -q "/mnt2.*ro,"
   lxc exec foo -- ls /mnt2/hosts
   lxc stop foo --force
   lxc start foo
@@ -170,13 +189,17 @@ test_config_profiles() {
 
   testloopmounts
 
+  test_mount_order
+
   lxc delete foo
 
   lxc init testimage foo
   lxc profile apply foo onenic,unconfined
   lxc start foo
 
-  lxc exec foo -- cat /proc/self/attr/current | grep unconfined
+  if [ -e /sys/module/apparmor ]; then
+    lxc exec foo -- cat /proc/self/attr/current | grep unconfined
+  fi
   lxc exec foo -- ls /sys/class/net | grep eth0
 
   lxc stop foo --force
