@@ -1,10 +1,10 @@
-#!/bin/sh
-
 test_devlxd() {
   ensure_import_testimage
 
+  # shellcheck disable=SC2164
   cd "${TEST_DIR}"
   go build -tags netgo -a -installsuffix devlxd ../deps/devlxd-client.go
+  # shellcheck disable=SC2164
   cd -
 
   lxc launch testimage devlxd
@@ -16,5 +16,5 @@ test_devlxd() {
   lxc config set devlxd user.foo bar
   lxc exec devlxd devlxd-client user.foo | grep bar
 
-  lxc stop devlxd --force
+  lxc delete devlxd --force
 }
