@@ -1,5 +1,3 @@
-#!/bin/sh
-
 test_fdleak() {
   LXD_FDLEAK_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
   chmod +x "${LXD_FDLEAK_DIR}"
@@ -23,6 +21,8 @@ test_fdleak() {
       lxc delete "leaktest${i}"
     done
 
+    sleep 5
+
     exit 0
   )
   afterfds=$(/bin/ls "/proc/${pid}/fd" | wc -l)
