@@ -1,5 +1,3 @@
-#!/bin/sh
-
 test_common_fuidshift() {
   # test some bad arguments
   fail=0
@@ -48,6 +46,11 @@ test_root_fuidshift() {
 }
 
 test_fuidshift() {
+  if ! which fuidshift >/dev/null 2>&1; then
+    echo "==> SKIP: No fuidshift binary could be found"
+    return
+  fi
+
   if [ "$(id -u)" -ne 0 ]; then
     test_nonroot_fuidshift
   else
