@@ -1,5 +1,3 @@
-#!/bin/sh
-
 test_cpu_profiling() {
   LXD3_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
   chmod +x "${LXD3_DIR}"
@@ -7,9 +5,14 @@ test_cpu_profiling() {
   lxdpid=$(cat "${LXD3_DIR}/lxd.pid")
   kill -TERM "${lxdpid}"
   wait "${lxdpid}" || true
+  export PPROF_TMPDIR="${TEST_DIR}/pprof"
   echo top5 | go tool pprof "$(which lxd)" "${LXD3_DIR}/cpu.out"
   echo ""
 
+  # Cleanup following manual kill
+  rm -f "${LXD3_DIR}/unix.socket"
+  find "${LXD3_DIR}" -name shmounts -exec "umount" "-l" "{}" \; >/dev/null 2>&1 || true
+
   kill_lxd "${LXD3_DIR}"
 }
 
@@ -32,6 +35,7 @@ test_mem_profiling() {
     timeout=$((timeout-1))
   done
 
+  export PPROF_TMPDIR="${TEST_DIR}/pprof"
   echo top5 | go tool pprof "$(which lxd)" "${LXD4_DIR}/mem"
   echo ""
 
