@@ -1,5 +1,3 @@
-#!/bin/sh
-
 gen_second_cert() {
   [ -f "${LXD_CONF}/client2.crt" ] && return
   mv "${LXD_CONF}/client.crt" "${LXD_CONF}/client.crt.bak"
@@ -12,6 +10,7 @@ gen_second_cert() {
 }
 
 test_remote_url() {
+  # shellcheck disable=2153
   for url in "${LXD_ADDR}" "https://${LXD_ADDR}"; do
     lxc_remote remote add test "${url}" --accept-certificate --password foo
     lxc_remote finger test:
@@ -21,6 +20,7 @@ test_remote_url() {
     lxc_remote remote remove test
   done
 
+  # shellcheck disable=2153
   urls="${LXD_DIR}/unix.socket unix:${LXD_DIR}/unix.socket unix://${LXD_DIR}/unix.socket"
   if [ -z "${LXD_OFFLINE:-}" ]; then
     urls="images.linuxcontainers.org https://images.linuxcontainers.org ${urls}"
@@ -28,7 +28,6 @@ test_remote_url() {
 
   for url in ${urls}; do
     lxc_remote remote add test "${url}"
-    lxc_remote finger test:
     lxc_remote remote remove test
   done
 }
@@ -80,13 +79,24 @@ test_remote_admin() {
 }
 
 test_remote_usage() {
+  # shellcheck disable=2039
+  local LXD2_DIR LXD2_ADDR
+  LXD2_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
+  chmod +x "${LXD2_DIR}"
+  spawn_lxd "${LXD2_DIR}"
+  LXD2_ADDR=$(cat "${LXD2_DIR}/lxd.addr")
+
+  ensure_import_testimage
+  ensure_has_localhost_remote "${LXD_ADDR}"
+
   lxc_remote remote add lxd2 "${LXD2_ADDR}" --accept-certificate --password foo
 
   # we need a public image on localhost
-  img=$(lxc_remote image export localhost:testimage "${LXD_DIR}/foo" | grep -o "foo.*")
+
+  lxc_remote image export localhost:testimage "${LXD_DIR}/foo"
   lxc_remote image delete localhost:testimage
-  sum=$(sha256sum "${LXD_DIR}/${img}" | cut -d' ' -f1)
-  lxc_remote image import "${LXD_DIR}/${img}" localhost: --public
+  sum=$(sha256sum "${LXD_DIR}/foo.tar.xz" | cut -d' ' -f1)
+  lxc_remote image import "${LXD_DIR}/foo.tar.xz" localhost: --public
   lxc_remote image alias create localhost:testimage "${sum}"
 
   lxc_remote image delete "lxd2:${sum}" || true
@@ -118,7 +128,12 @@ test_remote_usage() {
   lxc_remote image show lxd2:bar | grep -q "public: true"
   ! lxc_remote image show bar
   lxc_remote delete pub
+
+  # test spawn from public server
+  lxc_remote remote add lxd2-public "${LXD2_ADDR}" --public --accept-certificate
+  lxc_remote init lxd2-public:bar pub
   lxc_remote image delete lxd2:bar
+  lxc_remote delete pub
 
   # Double launch to test if the image downloads only once.
   lxc_remote init localhost:testimage lxd2:c1 &
@@ -138,4 +153,6 @@ test_remote_usage() {
   lxc_remote info lxd2:c1
   lxc_remote stop lxd2:c1 --force
   lxc_remote delete lxd2:c1
+
+  kill_lxd "$LXD2_DIR"
 }
